from django.db import models
from jungle.organization.models import Organization, OrganizationForm
from jungle.payment.models import PaymentForm
from jungle.userprofile.models import Office
from jungle.invoice.models import Invoice
from jungle.procedure.models import Procedure

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

TYPE_CODES = (
    ('D', 'DARE'),
    ('A', 'AVERE'),
)

############################
# ESTRATTO CONTO           #
############################
class accounting (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        verbose_name="Azienda",
    )
    organization_form = models.ForeignKey (
        OrganizationForm,
        verbose_name="Forma Societaria",
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        verbose_name="Forma Pagamento",
    )
    office = models.ForeignKey (
        Office,
        verbose_name="Ufficio",
    )
    invoice = models.ForeignKey (
        Invoice,
        verbose_name="Fattura",
    )
    procedure = models.ForeignKey (
        Procedure,
        verbose_name="Pratica",
    )
    ## campi
    type = models.CharField (
        max_length=1,
        default='D',
        choices=TYPE_CODES,
        verbose_name="Tipo",
    )
    date_operation = models.DateField (
        verbose_name="Data Operazione",
    )
    date_money = models.DateField (
        verbose_name="Data Valuta",
    )
    amount = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Importo",
        default=0.00,
    )

    class Admin:
        pass

    class Meta:
        verbose_name = "Estratto Conto"
        verbose_name_plural = "Estratti Conto"         