from django.db import models
#from jungle.organization.models import Organization

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

###############################
# INDIRIZZO                   #
###############################
class Address (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    #organization = models.ManyToManyField(
        #"Organization",
        #verbose_name="Anagrafica",
        #db_table="organization_organization_address",
        #null=True,
        #blank=True,
    #)
    address_scope = models.ManyToManyField (
        "AddressScope",
        verbose_name="Finalita'",
        #filter_interface=models.HORIZONTAL,
    )
    address_type = models.ForeignKey (
        "AddressType",
        verbose_name="Via, Viale, ...",
    )
    ## Campi
    name = models.CharField (
        max_length=100,
        verbose_name="Indirizzo",
    )
    number = models.CharField (
        max_length=5,
        blank=True,
        verbose_name="Civico",
    )
    crossed = models.CharField (
        max_length=5,
        blank=True,
        verbose_name="Barrato",
    )
    scale = models.CharField (
        max_length=5,
        blank=True,
        verbose_name="Scala",
    )
    floor = models.CharField (
        max_length=3,
        blank=True,
        verbose_name="Piano",
    )
    zip = models.CharField (
        max_length=5,
        verbose_name="CAP",
    )
    locality = models.CharField (
        max_length=100,
        verbose_name="Localita",
    )
    ## FIXME da sistemare (rifNazione, rifProvincia)
    province = models.CharField (
        max_length=2,
        verbose_name="Prov.",
    )
    country = models.CharField (
        max_length=2,
        verbose_name="Nazione",
        default="IT",
    )

    class Admin:
        list_filter = ("status", "address_scope", "address_type", "province", "country", )
        search_fields = ("name", "zip", "locality",)
        date_hierarchy = "date_create"
        list_display = ("status", "address_type", "name", "number", "crossed", "scale", "floor", "zip", "locality", "province", "country",)
        list_display_links = ("name", "address_type",)
        fields = (
            (None, {
                "fields": (
                    ("status",),
                    ("address_scope",),
                    ("address_type", "name",),
                    ("number", "crossed", "scale", "floor",),
                    ("zip", "locality", "province", "country",),
                ),
            }),
        )

    class Meta:
        verbose_name = "Indirizzo"
        verbose_name_plural = "Indirizzi"


    def __unicode__(self):
        address = [
            self.name,self.crossed,self.locality,
            self.province,self.country
        ]
        str_addr = u" ".join(address)

#######################################
# INDIRIZZO FINALITA                  #
#######################################
class AddressScope (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Finalita'",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "^name", )
        date_hierarchy = "date_create"
        list_display = ("status", "name", )
        list_display_links = ("name", )
        list_per_page = 20
        ordering = ("odv","name",)
        fields = (
            (None, {
                "fields": (
                    ("status",),
                    ("name", "odv",),
                ),
            }),
        )

    class Meta:
        verbose_name = "Finalita' Indirizzo"
        verbose_name_plural = "Finalita' Indirizzo"

    def __unicode__(self):
        return unicode(self.name)

###################################
# INDIRIZZO TIPO                  #
###################################
class AddressType (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=30,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Tipo",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "^code", "^name", )
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name", )
        list_display_links = ("code", "name", )
        list_per_page = 20
        fields = (
            (None, {
                "fields": (
                    ("status",),
                    ("code", "name", "odv",),
                ),
            }),
        )
        ordering = ("odv","code",)

    class Meta:
        verbose_name = "Tipo Indirizzo"
        verbose_name_plural = "Tipi Indirizzo"

    def __unicode__(self):
        return unicode(self.name)