from django.db import models
from jungle.organization.models import Organization

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

SHOW_AS = (
    ('F', 'Free',),
    ('B','Busy',),
)

ACCESS_TO = (
    ('P','Personal',),
    ('S','Shared',),
)

############################
# AGENDA                   #
############################
class Agenda (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    agenda_category = models.ForeignKey (
        "AgendaCategory",
        verbose_name="Categoria",
    )
    organization = models.ManyToManyField (
        Organization,
        verbose_name="Ragione Sociale",
        filter_interface=models.HORIZONTAL,
    )
    ## Campi
    title = models.CharField (
        max_length=100,
        verbose_name="Titolo",
    )
    description = models.TextField (
        verbose_name="Descrizione",
    )
    date_time_begin = models.DateTimeField (
        verbose_name="Data Ora Inizio",
    )
    date_time_end = models.DateTimeField (
        blank=True,
        null=True,
        verbose_name="Data Ora Fine",
    )
    duration = models.TimeField (
        blank=True,
        null=True,
        verbose_name="Durata (ore)",
    )
    no_time = models.BooleanField (
        blank=True,
        null=True,
        verbose_name="Nessuna ora",
    )
    alarm = models.BooleanField (
        blank=True,
        null=True,
        verbose_name="Promemoria",
    )
    show_as = models.CharField (
        max_length=1,
        choices=SHOW_AS,
        verbose_name="Mostra come",
    )
    access_to = models.CharField (
        max_length=1,
        choices=ACCESS_TO,
        verbose_name="Accesso",
    )
    note = models.TextField (
        blank=True,
        null=True,
        verbose_name="Note",
    )

    class Admin:
        list_filter = ("status", "agenda_category", "date_time_begin", "date_time_end",)
        search_fields = ("title",)
        date_hierarchy = "date_create"
        list_display = ("status", "agenda_category", "title", "date_time_begin", "date_time_end", "duration", "show_as", "access_to", )
        list_display_links = ("title",)
        list_per_page = 30
        ordering = ("-date_time_begin","agenda_category",)
        fields = (
            ("*** AGENDA ***", {"fields": (
                ("status",),
                ("agenda_category",),
                ("title",),
                ("description",),
            ),}),
            ("*** DATA / ORA ***", {"fields": (
                ("date_time_begin",),
                ("date_time_end",),
                ("duration","no_time",),
            ),}),
            ("*** ALTRO ***", {"fields": (
                ("alarm", "show_as", "access_to",),
            ),}),
            ("*** INVITATI ***", {"fields":(
                ("organization",),
            ),}),
            ("*** NOTE ***", {"fields": ("note",)}),
        )

    class Meta:
        verbose_name = "Appuntamento"
        verbose_name_plural = "Appuntamenti"

    def __unicode__(self):
        return unicode(self.title)

############################
# AGENDA CATEGORIA         #
############################
class AgendaCategory (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Categoria",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 30
        ordering = ("odv","name",)
        fields = (
            (None, {"fields": (
                ("status", "name", "odv",),
            )}),
        )

    class Meta:
        verbose_name = "Categoria Agenda"
        verbose_name_plural = "Categorie Agenda"

    def __unicode__(self):
        return unicode(self.name)