from django.conf import settings
from jungle.views.decorators import login_required
from django.utils import simplejson
from jungle.views import mgeneric as generic
from jungle.asterisk.models import cdr, Record
from django.http import HttpResponse, HttpResponseNotFound, HttpResponseForbidden
from django.db import transaction
import time, datetime
import os, os.path
import telnetlib

def get_formatted_time(s):
    ftime= "0s"
    m,s=divmod(s,60)
    h,m=divmod(m,60)
    d,h=divmod(h,24)
    if s > 0:
        ftime = "%ss" % s
    if m > 0:
        ftime = "%sm %s" % (m, ftime)
    if h > 0:
        ftime = "%sh %s" % (h, ftime)
    if d > 0:
        ftime = "%sg %s" % (m, ftime)
    
    return ftime

def get_nome_interno(id):
    nome = ""
    if id.find("<") > -1:
        nome = id.split("<")[0]
    return nome.strip()
    
def click_to_call(request):
    HOST = "192.168.100.250"
    PORT = "5038"
    user = "caller"
    password = "YTR11&^qw"

    #------------------------------------------------------------------------------------------
    #edit the below variable values to reflect your system/information
    #------------------------------------------------------------------------------------------    
    #specify the channel (extension) you want to receive the call requests with
    #e.g. SIP/XXX, IAX2/XXXX, ZAP/XXXX, etc
    strChannel = "SIP/140"
    
    #specify the context to make the outgoing call from.  By default, AAH uses from-internal
    #Using from-internal will make you outgoing dialing rules apply
    strContext = "from-internal"
    
    #specify the amount of time you want to try calling the specified channel before hangin up
    strWaitTime = "30"
    
    #specify the priority you wish to place on making this call
    strPriority = "1"
    
    #specify the maximum amount of retries
    strMaxRetry = "2"
    
    #--------------------------------------------------------------------------------------------
    #Shouldn't need to edit anything below this point to make this script work
    #--------------------------------------------------------------------------------------------
    #get the phone number from the posted form
    phoneNumber = str(request.POST.get('numero', ''))
    #specify the caller id for the call
    strCallerId = "Web Call"
    
    tn = telnetlib.Telnet(HOST, PORT)
    
    #tn.read_until("login: ")
    #tn.write(user + "\n")
    #if password:
    #    tn.read_until("Password: ")
    #    tn.write(password + "\n")
    
    tn.write("Action: login\r\n")
    tn.write("Events: off\r\n")
    tn.write("Username: %s\r\n" % user)
    tn.write("Secret: %s\r\n\r\n" % password)
    tn.write("Action: originate\r\n")
    tn.write("Channel: %s\r\n" % strChannel)
    tn.write("WaitTime: %s\r\n" % strWaitTime)
    tn.write("CallerId: %s\r\n" % strCallerId)
    tn.write("Exten: %s\r\n" % phoneNumber)
    tn.write("Context: %s\r\n" % strContext)
    tn.write("Priority: %s\r\n\r\n" % strPriority)
    tn.write("Action: Logoff\r\n\r\n")
    time.sleep(2)
    tn.close()
    return HttpResponse("OK")

@login_required
def download_registrazione(request, record_id):
    if request.user.has_perm("asterisk.can_download_record"):
        try:
            r = Record.objects.get(record=record_id)
            if not r.recall or not os.path.exists(r.recall):
                raise Exception
            a = open(r.recall)
            response = HttpResponse(a, mimetype='audio/x-wav')
            response['Content-Disposition'] = 'attachment; filename=%s' % r.recall.split('/')[-1]
            return response    
        except:
            return HttpResponseNotFound('<h1>Registrazione inesistente</h1>')            
    else:
        return HttpResponseForbidden("<h1>Accesso Negato</h1>")
        

@login_required
@transaction.commit_manually
def delete_registrazione(request, record_id):
    if request.user.has_perm("asterisk.can_delete_record"):
        try:
            r = Record.objects.get(record=record_id)
            if not r.recall or not os.path.exists(r.recall):
                raise Exception
            
            f_record = r.recall
            
            # Cancellazione della registrazione dal database
            #
            r.delete()
            
            # Cancellazione della registrazione dal disco fisso
            #
            os.remove(f_record)
        except:
            transaction.rollback()
            raise Exception(e)
        else:
            transaction.commit()
            return HttpResponse("OK")
    else:
        return HttpResponseForbidden("<h1>Accesso Negato</h1>")

        
@login_required
def lista(request, model, **kw):
    if model == 'cdr':
        return generic.render_template(request, 'asterisk/lista_chiamate.mako', **kw)
    elif model == 'record':
        return generic.render_template(request, 'asterisk/lista_registrazioni.mako', **kw)

@login_required
def make_call(request, **kw):
    return generic.render_template(request, 'asterisk/make_call.mako', **kw)

@login_required
def get_search_data(request, model):       
    user = request.user
    get =  request.GET.copy()

    if model == 'cdr':
        model_s = CdrSearch()
        template_name = 'asterisk/lista_chiamate.mako'
    elif model == 'record':
        model_s = RecordSearch()
        template_name = 'asterisk/lista_registrazioni.mako'        

    start = get.get('start', 0)
    limit = get.get('limit', 100)
    groupby = get.get('groupBy', '')
    orderby = get.get('sort', model_s.default_sort)
    direction = get.get('dir', 'DESC')
    
    queryset = model_s.get_query_set()

    # Gestione della ricerca
    #
    from jungle.views.fast_search import search2
    filter_cnd, oldorder_by, form = search2(request, model, template_name=template_name)
    
    try:
        filter_cnd = model_s.change_filter(filter_cnd)
    except:
        pass
        
    if filter_cnd:
        queryset = queryset.filter(**filter_cnd)

            
    total = queryset.count()
    queryset = model_s.order_by(queryset, groupby, orderby, direction)
    queryset = queryset[int(start):int(start)+int(limit)]

    dati = model_s.get_dati(queryset)
        
    data = '{"success": true, "total": %s, "data": %s}' % (total, simplejson.dumps(dati, ensure_ascii=False))
    return HttpResponse(data, mimetype='text/javascript;')


class CdrSearch:
    default_sort = 'calldate'
    
    def get_query_set(self):
        queryset = cdr.objects.all()
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        if groupby:
            return queryset.order_by(groupby, orderby)
        else:
            return queryset.order_by(orderby)
    
    def change_filter(self, filter_cnd):
        if filter_cnd.has_key('calldate__icontains'):
            d = filter_cnd['calldate__icontains']
            del filter_cnd['calldate__icontains']        
            filter_cnd['calldate__year'] = d.year
            filter_cnd['calldate__month'] = d.month
            filter_cnd['calldate__day'] = d.day        
            
        return filter_cnd
    
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['calldate'] = x.calldate.strftime("%m/%d/%Y %H:%M")
            d['calldate_for_group'] = x.calldate.strftime("%m/%d/%Y")
            nome = get_nome_interno(x.clid)
            if nome:
                d['src'] = "%s (%s)" % (x.src, nome)
            else:
                d['src'] = x.src
            d['dst'] = x.dst
            d['dcontext'] = x.get_dcontext_display()
            d['duration'] = get_formatted_time(x.duration)
            d['billsec'] = get_formatted_time(x.billsec)
            d['disposition'] = x.disposition            
            dati.append(d)
        return dati        

class RecordSearch:
    default_sort = 'ts'
    
    def get_query_set(self):
        queryset = Record.objects.all()
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        if groupby:
            return queryset.order_by(groupby, orderby)
        else:
            return queryset.order_by(orderby)

    def change_filter(self, filter_cnd):
        if filter_cnd.has_key('ts__icontains'):
            d = filter_cnd['ts__icontains']
            del filter_cnd['ts__icontains']        
            filter_cnd['ts__year'] = d.year
            filter_cnd['ts__month'] = d.month
            filter_cnd['ts__day'] = d.day        
            
        return filter_cnd
            
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.record
            d['ts'] = x.ts.strftime("%m/%d/%Y %H:%M")
            d['ts_for_group'] = x.ts.strftime("%m/%d/%Y")            
            d['caller'] = x.caller
            d['callee'] = x.callee
            d['recin'] = x.recin
            d['recout'] = x.recout
            if x.recall:
                d['recall'] = x.recall.split('/')[-1]
            else:
                d['recall'] = ''
            dati.append(d)
        return dati            