from django.db import models
from jungle.userprofile.models import Profile, Office

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

##########################
# CALL                   #
##########################
class Call (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    assigned_to = models.ForeignKey (
        Profile,
        verbose_name="Assegnato a",
    )
    office = models.ForeignKey (
        Office,
        verbose_name="Ufficio",
    )
    call_category = models.ForeignKey (
        "CallCategory",
        verbose_name="Categoria",
    )
    ## Campi
    subject = models.CharField (
        max_length=100,
        verbose_name="Oggetto",
    )
    description = models.TextField (
        verbose_name="Descrizione",
    )
    date_time_begin = models.DateTimeField (
        verbose_name="Data Ora Inizio",
    )
    date_time_end = models.DateTimeField (
        blank=True,
        verbose_name="Data Ora Fine",
    )
    duration = models.TimeField (
        blank=True,
        verbose_name="Durata (ore)",
    )
    reminder = models.BooleanField (
        verbose_name="Promemoria",
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )

    class Admin:
        list_filter = ("status", "assigned_to", "office", "call_category",)
        search_fields = ("subject",)
        date_hierarchy = "date_create"
        list_display = ("status", "call_category", "assigned_to", "office", "subject", "date_time_begin", "date_time_end", "duration", "reminder",)
        list_display_links = ("subject",)
        list_per_page = 30
        ordering = ("-date_time_begin", "office", "assigned_to",)
        fields = (
            ("*** CHIAMATA ***", {"fields": (
                ("status",),
                ("call_category", "office", "assigned_to", "reminder",),
                ("subject",),
                ("description",),
            ),}),
            ("*** DATA / ORA ***", {"fields": (
                ("date_time_begin",),
                ("date_time_end",),
                ("duration",),
            ),}),
            ("*** NOTE ***", {"fields": (
                ("note",),
            ),}),
        )

    class Meta:
        verbose_name = "Chiamata"
        verbose_name_plural = "Chiamate"
    
    def __unicode__(self):
        return unicode(self.subject)    

##########################
# CALL CATEGORIA         #
##########################
class CallCategory (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Categoria",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 30
        ordering = ("odv","name",)
        fields = (
            (None, {"fields": (
                ("status", "name", "odv",),
            )}),
        )

    class Meta:
        verbose_name = "Categoria Chiamata"
        verbose_name_plural = "Categorie Chiamata"

    def __unicode__(self):
        return unicode(self.name)    