from django.db import models
from jungle.organization.models import Organization, OrganizationForm
from jungle.payment.models import PaymentForm
from jungle.userprofile.models import Office
from jungle.store.models import Store, StoreCause
from jungle.product.models import Product, ProductType, Vat


# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

#########################
# DDT                   #
#########################
class Ddt (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        verbose_name="Azienda",
    )
    organization_form = models.ForeignKey (
        OrganizationForm,
        verbose_name="Forma Societaria"
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        null=True,
        blank=True,
        verbose_name="Forma Pagamento",
    )
    office = models.ForeignKey (
        Office,
        verbose_name="Ufficio",
    )
    #bank = models.ForeignKey (
    #)
    #list = models.ForeignKey (
    #)
    store_cause = models.ForeignKey (
        StoreCause,
        verbose_name="Causale Magazzino",
        related_name = 'ddt_doc',
    )
    ## Campi
    number = models.IntegerField (
        verbose_name="Numero",
    )
    year = models.DateField (
        verbose_name="Anno"
    )
    date = models.DateField (
        verbose_name="Data"
    )
    total_income = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Imponibile",
        default=0.00,
    )
    total_vat = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Imponibile",
        default=0.00,
    )
    total_fee = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Spesa",
        default=0.00,
    )
    total_discount = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Sconto",
        default=0.00,
    )
    ## invoice = 1 se fatturato
    invoice = models.BooleanField (
        verbose_name="Fatturato",
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )

    class Admin:
        list_filter = ("invoice","store_cause", "year")
        search_fields = ("office", "organization", )
        date_hierarchy = "date_create"
        list_per_page = 30
        ordering = ("-year", "-number",)
        list_display = ("office", "organization", "year", "number", "date", "total_income", "total_fee", "total_discount", "invoice",)
        list_display_links = ("organization", "number",)
        fields = (
            (None, {"fields": (
                ("status", "office", "store_cause",),
                ("organization", "organization_form", "payment_form",),
                ("year", "number", "date",),
                ("total_income", "total_vat", "total_fee", "total_discount",),
                ("invoice",),
                ("note",),
            ),}),
        )

    class Meta:
        verbose_name = "DDT"
        verbose_name_plural = "DDT"

    def __unicode__(self):
        return unicode(str(self.date)+" "+str(self.number))    

############################
# RIGA DDT                 #
############################
class DdtRow (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ddt = models.ForeignKey (
        Ddt,
        verbose_name = "Numero DDT",
    )
    ### da aggiungere anche anno
    product_type = models.ForeignKey (
        ProductType,
        verbose_name = "Tipo Prodotto",
    )
    product = models.ForeignKey (
        Product,
        verbose_name = "Prodotto",
    )
    vat = models.ForeignKey (
        Vat,
        blank=True,
        null=True,
        verbose_name = "IVA",
    )
    store_cause = models.ForeignKey (
        StoreCause,
        verbose_name="Causale Magazzino",
    )
    ## Campi
    quantity = models.IntegerField (
        verbose_name="Quantita'",
        default=0,
    )
    price = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        default=0.00,
        verbose_name="Prezzo",
    )
    fee = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Spesa",
        default=0.00,
    )
    discount = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Sconto",
        default=0.00,
    )
    total_income = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Totale Imponibile",
        default=0.00,
    )
    total_vat = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Totale IVA",
        default=0.00,
    )
    total_fee = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Totale Spesa",
        default=0.00,
    )
    total_discount = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Totale Sconto",
        default=0.00,
    )
    invoice = models.BooleanField (
        verbose_name="Fatturato",
    )

    class Admin:
        list_filter = ("invoice","store_cause",)
        search_fields = ("office", "organization", "ddt",)
        date_hierarchy = "date_create"
        list_per_page = 30
        ordering = ("store_cause", "-ddt",)
        list_display = ("ddt", "store_cause", "product_type", "product", "quantity", "price", "fee", "discount", "invoice", "total_income", "total_vat", "total_fee", "total_discount",)
        list_display_links = ("product",)
        fields = (
            (None, {"fields": (
                ("status", "ddt", "store_cause",),
                ("product_type", "product", "vat",),
                ("quantity", "price", "fee", "discount",),
                ("total_income", "total_vat", "total_fee", "total_discount",),
                ("invoice",),
            ),}),
        )

    class Meta:
        verbose_name = "Riga DDT"
        verbose_name_plural = "Righe DDT"
