from django.db import models

TYPES = (
    ('html', 'html'),
    ('txt', 'txt'),
    ('rst', 'rst'),
    )
class Document(models.Model):
    title = models.CharField(max_length=200)
    slug = models.CharField(max_length=50, unique=True, prepopulate_from=('title',))
    doc_path = models.CharField(max_length=200,
        help_text="Relative to the docs directory in cinemino SVN. No extension.")
    last_updated = models.DateTimeField(auto_now=True)
    doc_type  = models.CharField(max_length=4, choices=TYPES)

    class Meta:
        db_table = 'docs_document'
        ordering = ('title',)

    class Admin:
        fields = (
            (None, {'fields': ('title', 'slug', 'doc_path', 'doc_type')}),
        )
        list_display = ('title', 'doc_path')

    def __unicode__(self):
        return unicode(self.title)
    
    def get_absolute_url(self):
        return "/documentation/%s/" % self.slug

    def get_content(self, ext="html"):
        import os
        from django.conf import settings
        doc_path = os.path.join(settings.DJANGO_DOCUMENT_ROOT_PATH,
                                "%s.%s" % (self.doc_path, self.doc_type))
            
        if os.path.exists(doc_path):
            self._doc_content = open(doc_path).read()
        else:
            self._doc_content = ''
        return self._doc_content

    def get_html(self):
        return self.ReST( self.get_content())


    def get_toc(self):
        try:
            return self._toc_content
        except AttributeError:
            import os
            from django.conf import settings
            toc_path = os.path.join(
                settings.DJANGO_DOCUMENT_ROOT_PATH, "%s_toc.html" % self.doc_path)
            if os.path.exists(toc_path):
                self._toc_content = open(toc_path).read()
            else:
                self._toc_content = ''
            return self._toc_content

    def ReST(self, value):
        from django.conf import settings
        try:
            from docutils.core import publish_parts
        except ImportError:
            if settings.DEBUG:
                raise template.TemplateSyntaxError, "Error in {% restructuredtext %} filter: The Python docutils library isn't installed."
            return value
        else:
            parts = publish_parts(source=value, writer_name="html4css1")

            return parts["fragment"]

    
