import os, re
from django.conf import settings
from jungle.template.mako_loader import render_to_response
from django.template import RequestContext
from jungle.templatetags.jungle_tags import ReST
from django.contrib.auth.views import redirect_to_login
from django.views.generic.simple import redirect_to

HIDDEN = re.compile('.svn|CSV|.hg|.backups|.*~|.#.*|#.*')

class FileNotFound(Exception): pass
class Doc(object):
    def __init__(self, name=None, url=None, isdir=False):
        self.isdir = isdir
        self.name = name
        self.url = url or name

    def __repr__(self):
        # just for debug
        return self.name

def tree(request, document_root=None, document_url=None):
    # crete a Doc instance so we have .name, .url and .isdir available in
    # templates.
    # Sorting is hard so I need a key to sort and a dictionary to pick from
#    top_dir = settings.DOCUMENT_ROOT_PATH

    ret = {}
    for root, dirs, files in os.walk(document_root):
        ## get rid of HIDDEN
        dirs = filter(lambda d: not re.match(HIDDEN, d), dirs)
        dirs.sort()
        for dir in dirs:
            full_path = "%s/%s" % (root, dir)
            
            relative_path = full_path.replace(document_root, '')
            ret[relative_path] = Doc(relative_path, None, True )
        files.sort()
        for file in files:
            full_path = "%s/%s" % (root, file)
            relative_path = full_path.replace(document_root, '')
            if not file.endswith('~'):
                ret[relative_path] = Doc(relative_path, None, False)

    keys = ret.keys()
    keys.sort()
    docs = [ret[p] for p in keys]

    return render_to_response(
        "docs/document_list.mako",
        {'doclist': docs, 'tree' : True,
         'document_url': document_url,
         'path': re.sub('.*/%s' % document_url, '', request.path),},
         context_instance=RequestContext(request),)
    
def doc_view(request, path, template_name="docs/document_detail.mako",
             document_root=None, document_url=None, login_required=False):
    """
    return the document content, according to extension:
    html, text: as is
    rst:        filtered throught ReST
    """
    if login_required and not request.user.is_authenticated():
        return redirect_to_login(request.path)

    if 'mode' in request.GET and request.GET['mode'] == 'tree':
        return tree(request, document_root=document_root,
                    document_url=document_url)
    
    if isdir(path, document_root=document_root):
        ## we need a slash **in the browser** so that url is correctly built
        if path and  not path.endswith('/'):
            URL = '%s/%s/' % (document_url, path)
            return redirect_to(request, URL)
        for name in ('index.rst', 'index.html'):
            indexpath = document_root + '/'+ path +  name
            if os.path.exists(indexpath):
                return doc_view(request, path + name,
                                document_root=document_root,
                                document_url=document_url)
        context = {
            'doclist': doclist(path, document_root=document_root,
                               document_url=document_url),
            'document_url': document_url,
            ## FIXME: wrong if a DOC subdir exists
            'path': re.sub('.*/%s' % document_url, '', request.path),
            'tree' : False
            }
        return render_to_response("docs/document_list.mako",
                                  context,
                                  context_instance=RequestContext(request),)
                                  

    try:
        content = get_content_from_file(path, document_root=document_root)
    except FileNotFound:
        from django.http import Http404
        raise Http404

    doc = Doc()
    doc.content = content
    doc.name = os.path.basename(path)
    doc.path    = path
    context = {'obj': doc,
               'document_url': document_url,
               'path': re.sub('.*/%s' % document_url, '', request.path),
               'doclist': doclist(os.path.dirname(path),
                                  document_root=document_root,
                                  document_url=document_url),
               'tree' : False,
               }
    return render_to_response(template_name,
                              context,
                              context_instance=RequestContext(request),)     


def isdir(path, document_root=None):
    full_path = os.path.join(document_root, path)
        
    if os.path.isdir(full_path):
        return full_path

    return False

def doclist(path, document_root=None, document_url=None):
    """return a list of doc instances whose ..url attr is the path
    """
    import sys
    document_url = document_url.lstrip('/')
    ## NOTE: lstrip is important!  os.path.join('/misc/src/hg/', '/') = '/'
    ##       that is you get the whole root tree!
    full_path = os.path.join(document_root, path.lstrip('/'))

    doc_list = []
    docs = {}
    
    ## hide away some pattern
    listdir = filter(lambda d: not re.match(HIDDEN, d), os.listdir(full_path))
    listdir.sort()

    path = path.rstrip('/')
    for i in listdir:
        doc = Doc()
        doc.name = i
        doc.url = "%s" % (i)
        doc.isdir = os.path.isdir("%s%s" % (full_path, i) )
        if doc.isdir:
            doc.url += "/"
            
        docs[i] = doc

    keys = docs.keys()
    keys.sort()
    dir_list = [docs[key] for key in keys if docs[key].isdir]
    doc_list = [docs[key] for key in keys if not docs[key].isdir]

    return dir_list + doc_list 
        
def get_content_from_file(path, document_root=None):

    path = path.rstrip('/')
    root, ext = os.path.splitext(path)

    if not ext:
        exts = ('.html', '.txt', '.rst')
    else:
        exts = [ext]
        
    for ext in exts:
        doc_path = os.path.join(document_root, "%s%s" % (root, ext))
        if os.path.exists(doc_path):
            content = open(doc_path).read()
            if ext == '.rst':
                content = ReST(content)
            return content

    raise FileNotFound

