from django.db import models

STATUS_CODES = (
    ('0', 'Non attivo'),
    ('1', 'Attivo'),
)

#############################
# AMBIENT                   #
#############################
class Ambient (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    map = models.ManyToManyField (
        "Map",
        db_table="domotics_ambient_map",
        related_name="r_map",
        null=True,
        blank=True,
    )

    name = models.CharField(
        max_length=50,
        verbose_name="Nome",
    )
    description = models.CharField (
        max_length=200,
        verbose_name="Descrizione",
    )

    class Admin:
        pass
        #list_display=('status', 'name', 'description',)
        #list_display_links=('name',)
        #list_per_page=30
        #ordering=('name', 'description',)
        #search_fields=('name', 'description',)

    class Meta:
        verbose_name='Ambiente'
        verbose_name_plural='Ambienti'
    
    def __unicode__(self):
        return unicode(self.name+' '+self.description)    

#############################
# CATEGORY                  #
#############################
class Category (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    name = models.CharField(
        max_length=50,
        verbose_name="Nome",
    )
    description = models.CharField (
        max_length=200,
        verbose_name="Descrizione",
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Gruppo'
        verbose_name_plural='Gruppi'

    def __unicode__(self):
        return unicode(self.name+' '+self.description)    

#############################
# VIRTUAL                   #
#############################

TYPE_VIRTUAL = (
    ('D', 'Digital'),
    ('A', 'Analogic'),
    ('S', 'Security'),
    ('M', 'Scenario'),
)

class Virtual (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        verbose_name="Stato",
    )
    ## ForeignKey
    ambient = models.ForeignKey (
        Ambient,
        verbose_name="Ambiente",
    )
    category = models.ForeignKey (
        Category,
        verbose_name="Categoria",
    )
    name = models.CharField(
        max_length=50,
        verbose_name="Nome",
    )
    description = models.CharField (
        max_length=200,
        verbose_name="Descrizione",
    )
    type = models.CharField (
        max_length=2,
        choices=TYPE_VIRTUAL,
        verbose_name="Tipo",
    )
    range = models.CharField (
        max_length=50,
        blank=True,
        null=True,
        verbose_name="Range",
    )
    icon = models.CharField(
        max_length=50,
        verbose_name="Icona",
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Oggetto virtuale'
        verbose_name_plural='Oggetti virtuali'

    def __unicode__(self):
        return unicode(self.name+' '+self.description)
    
##################
# DEVICE         #
##################

TYPE_DEVICE = (
    (0, 'Output'),
    (1, 'Input'),
)

class Device (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        verbose_name="Stato",
    )

    name = models.CharField(
        max_length=50,
        verbose_name="Nome",
    )
    description = models.CharField (
        max_length=200,
        verbose_name="Descrizione",
    )
    module = models.IntegerField (
        max_length=5,
        verbose_name="Modulo",
    )
    number = models.IntegerField (
        max_length=5,
        verbose_name="Numero",
    )
    type = models.CharField (
        max_length=2,
        choices=TYPE_DEVICE,
        verbose_name="Tipo",
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Oggetto device'
        verbose_name_plural='Oggetti device'

    def __unicode__(self):
        return unicode(self.name+' '+self.description)
    
##################
# MESSAGE        #
##################

TYPE_MESSAGE = (
    ('E', 'Email'),
    ('S', 'SMS'),
)

class Message (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )

    name = models.CharField(
        max_length=50,
        verbose_name="Nome",
    )
    description = models.CharField (
        max_length=200,
        verbose_name="Descrizione",
    )
    type = models.CharField (
        max_length=2,
        choices=TYPE_MESSAGE,
        verbose_name="Tipo",
    )
    recipient = models.CharField (
        max_length=200,
        verbose_name="Destinatario",
    )
    message = models.CharField (
        max_length=200,
        verbose_name="Testo",
    )
    class Admin:
        pass

    class Meta:
        verbose_name='Messaggio'
        verbose_name_plural='Messaggi'

    def __unicode__(self):
        return unicode(self.name+' '+self.description)    

##################
# TIMER          #
##################

TYPE_TIMER = (
    ('T', 'Timer'),
)

class Timer (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )

    name = models.CharField(
        max_length=50,
        verbose_name="Nome",
    )
    description = models.CharField (
        max_length=200,
        verbose_name="Descrizione",
    )
    type = models.CharField (
        max_length=2,
        choices=TYPE_TIMER,
        verbose_name="Tipo",
    )
    start_time = models.TimeField (
        verbose_name="Ora inizio",
    )
    stop_time = models.TimeField (
        verbose_name="Ora termine",
    )
    days = models.CommaSeparatedIntegerField (
        max_length = 100,
        verbose_name = "Giorni",
        blank=True,
        null=True,
    )
    months = models.CommaSeparatedIntegerField (
        max_length = 36,
        verbose_name = "Mesi",
        blank=True,
        null=True,
    )
    wdays = models.CommaSeparatedIntegerField (
        max_length = 14,
        verbose_name = "Giorni della settimana",
        blank=True,
        null=True,
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Timer'
        verbose_name_plural='Timer'

    def __unicode__(self):
        return unicode(self.name+' '+self.description)    

##################
# MAP            #
##################

class Map (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ambient = models.ManyToManyField (
        Ambient,
        db_table="domotics_ambient_map",
        related_name="r_ambient",
        null=True,
        blank=True,
    )

    name = models.CharField(
        max_length=50,
        verbose_name="Nome",
    )
    description = models.CharField (
        max_length=200,
        verbose_name="Descrizione",
    )
    zoom = models.IntegerField (
        verbose_name="Zoom",
    )
    top = models.IntegerField (
        verbose_name="Top",
    )
    left = models.IntegerField (
        verbose_name="Left",
    )
    file = models.FileField (
        upload_to = "maps_domotics/",
        verbose_name="File mappa",
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Mappa'
        verbose_name_plural='Mappe'

    def __unicode__(self):
        return unicode(self.name+' '+self.description)    
    

##################
# POSITION       #
##################

class Position (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    virtual = models.ForeignKey (
        Virtual,
        verbose_name="Oggetto Virtuale",
    )
    map = models.ForeignKey (
        Map,
        verbose_name="Mappa",
    )
    pos_x = models.IntegerField (
        null=True,
        blank=True,
        verbose_name="Posizione X",
    )
    pos_y = models.IntegerField (
        null=True,
        blank=True,
        verbose_name="Posizione Y",
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Posizione'
        verbose_name_plural='Posizioni'

    def __unicode__(self):
        return unicode(self.virtual.name+' '+self.map.name)    
    

##################
# PROG           #
##################

class Prog (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )

    name = models.CharField (
        max_length=50,
        verbose_name="Nome",
    )
    odv = models.IntegerField (
        verbose_name="ODV",
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Programmazione'
        verbose_name_plural='Programmazioni'

    def __unicode__(self):
        return unicode(self.name)
    
##################
# RULE           #
##################

class Rule (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )

    prog = models.ForeignKey (
        Prog,
        verbose_name="Regola",
    )
    if_rule = models.CharField (
        max_length=200,
        verbose_name="IF",
    )
    then_rule = models.CharField (
        max_length=200,
        verbose_name="THEN",
    )
    else_rule = models.CharField (
        max_length=200,
        verbose_name="ELSE",
    )
    odv = models.IntegerField (
        verbose_name="ODV",
    )

    class Admin:
        pass

    class Meta:
        verbose_name='Regola'
        verbose_name_plural='Regole'

    def __unicode__(self):
        return unicode(self.prog.name+' '+self.if_rule)    

##################
# ICON           #
##################

class Icon (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.CharField (
        max_length=5,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    category = models.ForeignKey(
        Category,
        verbose_name="Categoria",
    )
    icon_status = models.CharField (
        max_length=5,
        default='1',
        verbose_name="Stato icona",
    )
    background = models.CharField (
        max_length=7, 
        default='#ffffff',
        verbose_name="Sfondo",
    )
    file = models.FileField (
        upload_to = "icons_domotics/",
        verbose_name="File icona",
    )
    class Admin:
        pass

    class Meta:
        verbose_name='Icona'
        verbose_name_plural='Icone'

    def __unicode__(self):
        return unicode(self.category.name)

