from django.db import models

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

#########################
# FAQ                   #
#########################
class Faq (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    type = models.ForeignKey (
        "FaqType",
        verbose_name="Tipo Faq",
    )
    ## Campi
    question = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Domanda",
    )
    answer = models.TextField (
        unique=True,
        verbose_name="Risposta",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "type", )
        search_fields = ("question",)
        date_hierarchy = "date_create"
        list_display = ("status", "type", "question","answer",)
        list_display_links = ("question","answer",)
        list_per_page = 20
        ordering = ("odv","question",)
        fields = (
            (None, {"fields": (
                ("status", "type", ),
                ("question",),
                ("answer",),
                ("odv",),
            ),}),
        )

    class Meta:
        verbose_name = "Faq"
        verbose_name_plural = "Faq"

    def __unicode__(self):
        return unicode(self.question+" "+self.answer)    

#############################
# TIPO FAQ                  #
#############################
class FaqType (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Tipo",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 20
        ordering = ("odv","name",)
        fields = (
            (None, {"fields": (
                ("status", "name", "odv",),
            ),}),
        )

    class Meta:
        verbose_name = "Tipo Faq"
        verbose_name_plural = "Tipi Faq"

    def __unicode__(self):
        return unicode(self.name)