from jungle.organization.models import Organization, OrganizationGroup
from django.contrib.auth.models import User
from django.db import models
from django.conf import settings
from django.db.models import signals
from django.dispatch import dispatcher

try:
    LOCATION = settings.LOCATION
except:
    LOCATION = '/'

# STATUS_CODES = (
#     ( 1, 'Active'),
#     ( 0, 'Inactive')
# )

# N_R = (
#     (1, 'Nuovo'),
#     (2, 'Letto'),
# )

STATUS_CODES = (
    (2, 'Received'),
    (3, 'Queued'),
    (4, 'Sent'),
    (5, 'Failed'),
    (6, 'Multi Queued'),
    (7, 'Multi Sent'),
)

PRIORITIES = (
    (127, 'Low'),
    (120, 'Normal'),
    (100, 'Hight'),
    (50, 'Urgent'),
    )
#########################
# FAX                   #
#########################

class Fax (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
        help_text="La 'Data Creazione' viene generata automaticamente",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
        help_text="La 'Data Modifica' viene modificata ogni volta che viene modificato il record",
    )
    status = models.IntegerField (
        max_length=3,
        default='3',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## Campi
    job = models.CharField (
        blank=True,
        null =True,
        max_length=15,
        verbose_name="Codice",
        help_text="Codice assegnato da Hylafax"
    )
    keywords = models.CharField (
        blank=True,
        null =True,
        max_length=100,
        verbose_name="Parole chiave ricerca",
    )
    pages = models.PositiveSmallIntegerField (
        blank=True,
        null =True,
        max_length=3,
        verbose_name="Numero Pagine",
    )
    date_required = models.DateTimeField (
        blank=True,
        null =True,
        verbose_name="Ora spedizione",
        help_text="Ora richiesta di spedizione, default: subito.",
    )
    date_sent = models.DateTimeField (
        blank=True,
        null =True,
        verbose_name="Tempo",
    )
    ## submitter/receiver
    submitter = models.ForeignKey (
        User,
        blank=True,
        null =True,
        verbose_name='Inviato da',
    )
    recipient = models.CharField (
        blank=True,
        null =True,
        max_length=100,
        verbose_name="Destinatario",
    )
    recipient_number = models.CharField (
        blank=True,
        null =True,
        max_length=16,
        verbose_name="Numero fax Destinatario",
    )
    recipient_list = models.ForeignKey (
        OrganizationGroup,
        blank=True,
        null =True,
        verbose_name="Lista di Destinatari",
        help_text=u"Non e' assicurato l'invio del Fax a tutti i membri della lista, potrebbero non possederne uno.",
    )
    sender = models.CharField (
        blank=True,
        null =True,
        max_length=100,
        verbose_name="Mittente",
    )
    sender_number = models.CharField (
        blank=True,
        null =True,
        max_length=16,
        verbose_name="Numero fax Mittente",
    )
    duration = models.CharField (
        blank=True,
        null =True,
        max_length=10,
        verbose_name="Durata",
    )

    device = models.CharField (
        blank=True,
        null =True,
        max_length=5,
        verbose_name="Device",
    )
    file = models.FileField (
        null=True,
        upload_to='%s%s' % (settings.MEDIA_ROOT, '/fax/'),
        verbose_name="File",
    )
    error = models.CharField (
        blank=True,
        null =True,
        max_length=150,
        verbose_name="Errore",
    )
    subject = models.CharField (
        max_length=100,
        verbose_name="Oggetto",
    )
    cover = models.BooleanField (
        default=False,
        verbose_name="Copertina",
    )
    read = models.IntegerField (
        null=True, blank=True,
        choices=((0,'Not Read'), (1,'Read')),
        verbose_name="Letto",
    )
    priority = models.PositiveSmallIntegerField (
        blank=True,
        null =True,
        default=127,
        verbose_name=u"Priority",
        choices=PRIORITIES
    )
#    def rename(self):
#        import os
#        from django.conf import settings 
#        MR = settings.MEDIA_ROOT
#        fname = os.path.join(MR, self.file)
#
#        head, tail = os.path.split(fname)
#        OLD = os.path.join(MR, head, tail)
#        
#        name, ext = os.path.splitext(tail)
#        if self.status in (2, 3, 6, 7):
#            name = "out-%s" % self.id + ext
#        else:
#            name = "in-%s" % self.id + ext
#        NEW = os.path.join(MR, head, name)
#
#        head, tail = os.path.split(self.file)
#        self.file = os.path.join(head, name)
#        os.rename(OLD, NEW)
#        super(Fax, self).save()

#    def new_name(self):
#        import os
#        name, ext = os.path.splitext(self.file)
#        if self.status in ('Q', 'S'):
#            name = "out-%s" % self.job + ext
#        else:
#            name = "in-%s" % self.job + ext
#        return "fax/%s" % name

    def save(self):
        from jungle.fax.sendfax import FaxQueue

        if self.file:
            super(Fax, self).save()

            if self.recipient_list:
                self.loop()
            else:
                f = FaxQueue(fax=self).send()

    def hsave(self):
        """save da hylafax, non serve riaccodare"""
        super(Fax, self).save()
        
    def loop(self):
        for dest in self.recipient_list.organization.all():
            if dest.fax:
                f = Fax(
                    subject = self.subject,
                    submitter = self.submitter,
                    priority = self.priority,
                    keywords = self.keywords,
                    recipient = unicode(dest),
                    recipient_number = dest.fax,
                    file = self.file,
                    #date_required = self.date_required,
                    status = 6,
                )
                f.save()

    class Admin:
        list_filter = ("status", "date_create", "status", "device", "read","priority",)
        search_fields = ("^sender", "recipient_number", "subject",)
        date_hierarchy = "date_create"
        list_per_page = 20
        list_display = ("status",  "status", "recipient", "recipient_number", "sender", "sender_number", "subject","pages", "date_sent", "read", "priority",)
        list_display_links = ("subject",)
        ordering = ("-date_create", "status","device",)
        fields = (
            (None, { "fields": (
                ("status", "job", "status", "pages", "device",),
                ("recipient", "recipient_number", "sender", "sender_number",),
                ("subject", "file", "cover",),
                ("read", "priority", "error",),
            ),}),
        )
    class Meta:
        verbose_name = "Fax"
        verbose_name_plural = "Fax"

    def __unicode__(self):
        return unicode(self.subject)
    
SEND_MODE = (
    (1, 'Fax'),
    (2, 'Email'),
)

class SendFax(models.Model):
    date_required = models.DateTimeField (
        blank=True,
        null =True,
        verbose_name="Ora spedizione",
        help_text="Ora richiesta di spedizione, default: subito.",
    )
    submitter = models.ForeignKey (
        User,
        blank=True,
        null =True,
        verbose_name='Inviato da'
    )
    status = models.IntegerField (
        max_length=3,
        default='3',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    groups = models.ManyToManyField(
        OrganizationGroup,
        blank=True,
        null =True,
    )
    organizations = models.ManyToManyField(
        Organization,
        blank=True,
        null =True,
    )
    recipient_numbers = models.CharField (
        blank=True,
        null =True,
        max_length=255,
        verbose_name="Destinatari non in anagrafica",
        help_text="Separati dal carattere 'punto e virgola'.",
    )
    mode = models.PositiveSmallIntegerField(choices = SEND_MODE)
    subject = models.CharField (
        max_length=100,
        verbose_name="Oggetto",
    )
    keywords = models.CharField (
        blank=True,
        null =True,
        max_length=100,
        verbose_name="Parole chiave ricerca",
    )
    forward = models.IntegerField(
        default=0,
    )
    priority = models.PositiveSmallIntegerField (
        blank=True,
        null =True,
        default=127,
        verbose_name=u"Priority",
        choices=PRIORITIES
    )
    file = models.FileField (
        null=True,
        upload_to="fax",
        verbose_name="File",
    )

    class Meta:
        pass
        #create_db_table = False

    def save(self):
        pass

def add_group():
    from jungle.functions.tools import add_group_with_app_permissions
    
    add_group_with_app_permissions('Fax', Fax._meta.app_label)

dispatcher.connect(add_group, signal=signals.post_syncdb, weak=False)
