import subprocess as sub
from django.conf import settings
from jungle.fax.models import *
from django.newforms.models import form_for_model
from django.views.generic.simple import redirect_to

from django.conf import settings

try:
    LOCATION = settings.LOCATION
except:
    LOCATION = '/'

class ExecErrorException(Exception): pass

class FaxQueue(object):
    def __init__(self, fax):
        self.fax = fax

    def send(self):
        if not self.fax.recipient_number:
            if self.fax.recipient_list:
                return
        info = {
            'path'         :  '/usr/bin/sendfax',
            'start_time'   :  'now',
            #'start_time'   :  self.fax.date_required or 'now',
            'fax_number'   :  self.fax.recipient_number,
            'file_name'    :  '%s/static/%s' % (settings.CWD, self.fax.file),
            'priority'     :  self.fax.priority or 127,
            }
        
        # start_time = d.split('.')[0]
        # time.strptime(d, "%Y-%m-%d %H:%M:%S")
        # [HH:MM [AM|PM]|now] [+ N period | month DD | dayofweek ]
        # 2007-02-27 17:26:23.843810

        cmd = "%(path)s -a %(start_time)s -P %(priority)s -D -n " + \
              "-d %(fax_number)s %(file_name)s"
        cmd = cmd % info

        ## exec...
        p =  sub.Popen(cmd.split(), stdout=sub.PIPE, stderr=sub.PIPE)
        p.wait()
        (stdout, stderr) = p.communicate()

        ## check for return code
        if p.returncode:
            raise ExecErrorException("stderr: %s - returncode: %s %s" %
                                     (stderr, p.returncode, cmd)) 
        if not stdout:
            raise ExecErrorException("Missing stdout from sendfax")
        else:
            ## output from sendfax:
            ## request id is 879 (group id 879) for host localhost (1 file)
            self.fax.job = stdout.split()[3]
            self.fax.hsave()

class SendFaxForm(form_for_model(SendFax)):
    def __init__(self, *args, **kwargs):
        super(SendFaxForm, self).__init__(*args, **kwargs)
        from django.newforms.widgets import Select
        self.fields['priority'].widget = Select(choices=PRIORITIES)
        self.fields['mode'].widget = Select(choices=SEND_MODE)

    def save(self):
        from jungle.fax.models import Fax
        import os
        
        try:
            last_fax = Fax.objects.all().order_by('-id')[0]
        except:
            last_fax = None
        
        if last_fax:
            last_id_fax = last_fax.id
        else:
            last_id_fax = 1

        if self.cleaned_data['mode'] == 1: # fax send
            if self.cleaned_data['forward'] == 0: # fax send
                filename = '%s%s%s' % (settings.MEDIA_ROOT, '/fax/',
                                                   self.cleaned_data['file'][0]['filename'])
                ## ex rename method
                head, tail = os.path.split(filename)
                name, ext = os.path.splitext(tail)
                name = "out-%s%s" % (last_id_fax, ext)

                filename = os.path.join(head, name)

                open(filename, 'w').write(self.cleaned_data['file'][0]['content'])

                filename = '/'.join(filename.split('/')[-2:])
            else: # fax forward
                filename = ''.join(s for s in self.cleaned_data['file'] if s)

            if self.cleaned_data['organizations']:
                for o in self.cleaned_data['organizations']:
                    if o.fax:
                        f = Fax(
                            subject = self.cleaned_data['subject'],
                            priority = self.cleaned_data['priority'],
                            submitter = self.cleaned_data['submitter'],
                            keywords = self.cleaned_data['keywords'],
                            recipient = unicode(o),
                            recipient_number = o.fax,
                                 file = filename,
                            #date_required = self.date_required,
                            status = self.cleaned_data['status'],
                        )
                        f.save()

            if self.cleaned_data['groups']:
                for g in self.cleaned_data['groups']:
                    f = Fax(
                            subject = self.cleaned_data['subject'],
                            priority = self.cleaned_data['priority'],
                            submitter = self.cleaned_data['submitter'],
                            keywords = self.cleaned_data['keywords'],
                            recipient = "Gruppo: " + unicode(g),
                            recipient_list = g,
                             file = filename,
                            #date_required = self.date_required,
                            status = self.cleaned_data['status'],
                    )
                    f.save()

            if self.cleaned_data['recipient_numbers']: 
                for n in self.cleaned_data['recipient_numbers'].split(';'):
                    to = n.split(':')
                    f = Fax(
                        subject = self.cleaned_data['subject'],
                            priority = self.cleaned_data['priority'],
                            submitter = self.cleaned_data['submitter'],
                            keywords = self.cleaned_data['keywords'],
                            recipient = to[0],
                        recipient_number = to[1],
                            file = filename,
                            #date_required = self.date_required,
                            status = self.cleaned_data['status'],
                    )
                    f.save()

        elif self.cleaned_data['mode'] == 2: # mail send
            from jungle.functions.mail import send_mail

            to_list = []
            if self.cleaned_data['organizations']:
                for o in self.cleaned_data['organizations']:
                    if o.email:
                        to_list.append(o.email)

            if self.cleaned_data['groups']:
                for g in self.cleaned_data['groups']:
                    for o in g.organization.all():
                        if o.email:
                            to_list.append(o.email)

            if self.cleaned_data['recipient_numbers']: 
                for n in self.cleaned_data['recipient_numbers'].split(';'):
                    to_list.append(n)

            for to in to_list:
                sender = self.cleaned_data['submitter'].email
                subject = self.cleaned_data['subject']
                text = 'Inoltro Fax via email.\nConsultare il file allegato.'
                files = ['%s/%s' % (settings.MEDIA_ROOT, ''.join(s for s in self.cleaned_data['file'] if s) )]

                send_mail(sender, to, subject, text, files)
