from django.db import models
from django.newforms import ModelForm
from jungle.functions.conf import *

##########################
# CATEGORY               #
##########################
STATUS_CODES = (
    (1, 'Attivo'),
    (0, 'Inattivo'),
)

PATH_FWADD = get_from_local_settings('PATH_FWADD', '/etc/argo/fw.web')
#PATH_FWADD = get_from_local_settings('PATH_FWADD', '/tmp/fw.add')

###########################
# FW RULE                 #
###########################
PROTOCOL = (
    ('TCP', 'TCP'), 
    ('UDP', 'UDP'), 
    ('GRE', 'GRE')
)

RULE_TYPE = (
    ('DMZ', 'DMZ'), 
    ('PXY', 'PROXY')
)

class DnatRule(models.Model):
    ## CAMPI GENERICI
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Date Create",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "Date Last Modify",
    )

    ## CAMPI MODELLO
    status = models.IntegerField(
        choices = STATUS_CODES,
        db_index = True,
        verbose_name = "status",
    )
        
    ip_src = models.CharField(
        max_length = 20,        
        verbose_name = "Source IP",
        help_text = "Per ip singolo : 83.211.71.213, per rete : 192.168.100.0/24, per qualsiasi ip : 0.0.0.0/0"
    )
    
    dst_pre = models.CharField(
        max_length = 20,
        verbose_name = "Destination IP Before",
    )
    
    dst_nat = models.CharField(
        max_length = 20,
        verbose_name = "Destination IP After"
    )

    proto = models.CharField (
        max_length = 3,
        verbose_name = "Protocol",
        choices = PROTOCOL
    )
    
    dst_pre_port = models.CharField (
        verbose_name = "Source Port",
        max_length = 5,
    )
       
    dst_nat_port = models.CharField (
        verbose_name = "Destination Port",
        max_length = 5,
    )
           
    snat = models.CharField(
        null = True, blank = True,
        max_length = 20,
        verbose_name = "SNAT IP"
    )

    type = models.CharField (
        null = True, blank = True,
        max_length = 3,
        verbose_name = "Type",
        choices = RULE_TYPE
    )
    
    note = models.TextField(
        null = True, blank = True,
        verbose_name = "Note"
    )
    
    def save(self):
        super(DnatRule, self).save()
        self.crea_fw_file()

    def delete(self):        
        super(DnatRule, self).delete()
        self.crea_fw_file()
            
    def crea_fw_file(self):
        f = open(PATH_FWADD, 'w')
        
        for r in DnatRule.objects.filter(status=1):
            f.write(r.getRuleString())
            f.write("\n")
        
        f.close()
        
    def getRuleString(self):
        s = "dnat1"
        
        if self.ip_src:
            s += " %s" % self.ip_src

        s += " %s %s" % (self.dst_pre, self.dst_nat)
        
        if self.proto:
            s += " %s" % self.proto
        
        if self.dst_pre_port and self.dst_nat_port:
            s += " %s@%s" % (self.dst_pre_port, self.dst_nat_port)
        elif self.dst_pre_port:
            s += " %s" % self.dst_pre_port            
        elif self.dst_nat_port:
            s += " %s" % self.dst_nat_port
        if self.snat:
            s += " %s" % self.snat
            
        return s
        
    def __unicode__(self):
        return u"%s@%s -> %s@%s" % (self.dst_pre, self.dst_pre_port, self.dst_nat, self.dst_nat_port )

    class Admin:
        pass
