# -*- coding: utf-8 -*-

from jungle.views.decorators import login_required
from jungle.views import mgeneric as generic
from jungle.firewall.models import DnatRule
from django.utils import simplejson
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.conf import settings
from datetime import timedelta
#from jungle.functions import log
from jungle.functions.conf import *
import time, commands, os
from django.db.models.query import Q

@login_required
def add(request, model=None, **kw):
    t = request.GET.get('t', None)
    if model == 'dnatrule':
        if t:
            kw['post_save_redirect'] = '%sfirewall/dnatrule/list/?t=%s' % (settings.LOCATION, 
                                                                          request.GET['t'])
        else:
            kw['post_save_redirect'] = '%sfirewall/dnatrule/' % (settings.LOCATION,)
    return generic.create_object(request, model, **kw)

    
@login_required
def update(request, model=None, **kw):
    t = request.GET.get('t', None)
    if model == 'dnatrule':
        if t:
            kw['post_save_redirect'] = '%sfirewall/dnatrule/list/?t=%s' % (settings.LOCATION, 
                                                                          request.GET['t'])
        else:
            kw['post_save_redirect'] = '%sfirewall/dnatrule/' % (settings.LOCATION,)


    return generic.update_object(request, model, ext="_update", **kw)
    
@login_required
def object_list(request, model, filter_cnd={}, order_by='-date_create',
                search_params={}, form=None, **kw):

    if model == 'dnatrule':
        queryset = DnatRule.objects.none()
            
    return generic.jobject_list(request, queryset, model, form,
                                filter_cnd, order_by, **kw)
    
@login_required
def object_detail(request, model, object_id):
    return generic.object_detail(request, model, object_id)

@login_required
def change_status(request, model):
    id = request.POST['keyID']
    status = request.POST['value']
    f = DnatRule.objects.get(pk=id)
    f.status = status
    f.save()
    return HttpResponse("OK")

@login_required
def reload_firewall(request):
    last_line = "Contattare l'amministratore"
    status, output = commands.getstatusoutput('sudo /sbin/firewall restart')
    return HttpResponse("OK")

   
@login_required
def get_search_data(request, model):       
    user = request.user
    get =  request.GET.copy()

    ### CERTIFICATE ###
    if model == 'dnatrule':
        model_s = DnatRuleSearch()

    start = get.get('start', 0)
    limit = get.get('limit', 25)
    groupby = get.get('groupBy', '')
    orderby = get.get('sort', model_s.default_sort)
    direction = get.get('dir', 'ASC')
        
    queryset = model_s.get_query_set(request)

    # Gestione della ricerca
    #
    from jungle.views.fast_search import search2
    filter_cnd, oldorder_by, form = search2(request, model)

    if filter_cnd:
        queryset = queryset.filter(**filter_cnd)

            
    total = queryset.count()
    queryset = model_s.order_by(queryset, groupby, orderby, direction)
    queryset = queryset[int(start):int(start)+int(limit)]

    dati = model_s.get_dati(queryset)
        
    data = '{"success": true, "total": %s, "data": %s}' % (total, simplejson.dumps(dati, ensure_ascii=False))
    return HttpResponse(data, mimetype='text/javascript;')


class DnatRuleSearch:
    default_sort = 'proto'
    typeRule = None
    
    def get_query_set(self, request):
        queryset = DnatRule.objects.all()
        self.typeRule = request.GET.get('t', None)
        if self.typeRule == 'dmz':
            queryset = queryset.filter(type='DMZ')
        elif self.typeRule == 'proxy':
            queryset = queryset.filter(type='PXY')
        else:
            queryset = queryset.filter(Q(type__isnull=True) |
                                       Q(type=''))
        
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):            
        #if dir == 'DESC':
        #    orderby = "-%s" % orderby
        return queryset.order_by('dst_pre', 'dst_pre_port', 'dst_nat', 'dst_nat_port')
        
    def get_dati(self, queryset):
        dati = []
        
        for x in queryset:           
            d = {}
            d['id'] = x.id
            d['proto'] = x.proto
            d['ip_src'] = x.ip_src
            d['dst_pre'] = x.dst_pre
            d['dst_nat'] = x.dst_nat
            d['status'] = x.get_status_display()
            d['dst_pre_port'] = x.dst_pre_port
            d['dst_nat_port'] = x.dst_nat_port
            d['snat'] = x.snat
            d['note'] = x.note
            if self.typeRule:
                d['type_rule'] = '?t=%s' % self.typeRule
            d['grouping_field'] = "%s : %s  ---->  Destinazione: %s : %s" % (x.dst_pre, x.dst_pre_port, x.dst_nat, x.dst_nat_port)
            
            dati.append(d)
        return dati        
