from django.conf import settings
from jungle.views.decorators import login_required
from jungle.views import mgeneric as generic
from jungle.mailserver.models import Account, Domain, Alias, Vacation
#import forms as mailforms
from django.contrib.auth.models import Group
from jungle.template.mako_loader import render_to_response
from django.template import RequestContext

@login_required
def add_postmaster(request, **kw):
    from django.contrib.auth.models import User, Group
    import forms as mailforms
    
    data = {'username' : request.POST['new_postmaster']}
    u = User(**data)
    u.set_password('password')
    try:
        u.save()
    except:
        request.user.message_set.create(message="Postmaster NOT created because the username is in use.")
    if u.id:
        g = Group.objects.get(name = 'Postmaster')
        u.groups.add(g)
        u.save()
        request.user.message_set.create(message="Password is 'password', change it immediately.")

    kw['extra_context'] = {'form' : mailforms.DomainForm()}

    return render_to_response('mailserver/domain_form.mako',
               dictionary=kw['extra_context'], context_instance=RequestContext(request))

@login_required
def object_list(request, model='domain', filter_cnd={}, order_by='',
                search_params={}, form=None, **kw):

    if model == 'domain':
        if request.user.has_perm('mailserver.supermaster_perm'):
            queryset = Domain.objects.all().order_by('name')
        elif request.user.has_perm('mailserver.postmaster_perm'):
            queryset = Domain.objects.filter(postmaster = request.user).order_by('name')
        else:
            queryset = Domain.objects.none()

    if model == 'account':
        if request.user.has_perm('mailserver.supermaster_perm'):
            queryset = Account.objects.all().order_by('account')
        elif request.user.has_perm('mailserver.postmaster_perm'):
            queryset = Account.objects.filter(domain__postmaster = request.user).order_by('account')
        else:
           queryset = Account.objects.none()

    if model == 'alias':
        if request.user.has_perm('mailserver.supermaster_perm'):
            queryset = Alias.objects.all().order_by('alias')
        elif request.user.has_perm('mailserver.postmaster_perm'):
            queryset = Alias.objects.filter(domain__postmaster = request.user).order_by('alias')
        else:
            queryset = Alias.objects.none()

    if model == 'vacation':
        if request.user.has_perm('mailserver.supermaster_perm'):
            queryset = Vacation.objects.all().order_by('date_start')
        elif request.user.has_perm('mailserver.postmaster_perm'):
            queryset = Vacation.objects.filter(account__domain__postmaster = request.user).order_by('date_start')
        else:
            queryset = Vacation.objects.none()

    return generic.jobject_list(request, queryset, model, form,
                                filter_cnd, order_by, **kw)


@login_required
def add(request, model, alias_name=None, domain_id=None, **kw):
    import forms as mailforms
    redirect = ''

    form = None

    if model == 'alias':
        form = mailforms.AliasesForm
        if alias_name and domain_id:
            redirect = request.META['HTTP_REFERER']
        else:
            redirect = '%smailserver/alias/list/' % settings.LOCATION

    if model == 'account':
        #from forms import AccountForm
        form = mailforms.AccountForm
        redirect = '%smailserver/account/list/' % settings.LOCATION

    if model == 'domain':
        if not request.user.has_perm('mailserver.supermaster_perm'):
            request.user.message_set.create(message="You don't have the permissions to create domains.")

            return render_to_response('/error.mako',
               dictionary={}, context_instance=RequestContext(request))

        form = mailforms.DomainForm
    
    if model == 'vacation':
        form = mailforms.VacationForm
        redirect = '%smailserver/vacation/list/' % settings.LOCATION
        
    if redirect:
        kw.update({ 'post_save_redirect' : redirect })

    if 'popup' in request.GET:
        kw['post_save_redirect'] = '/views/close_popup/'

    return generic.create_object(request, model, form=form, **kw)

@login_required
def update(request, model=None, **kw):
    import forms as mailforms
    form = None

    if model == 'account':
        form = mailforms.AccountForm

    if model == 'domain':
        form = mailforms.DomainForm

        if not request.user.has_perm('mailserver.supermaster_perm'):
            request.user.message_set.create(message="You don't have the permissions to edit domains.")

            return render_to_response('/error.mako',
                       dictionary={}, context_instance=RequestContext(request))

    redirect = ''

    if request.method == 'POST' and 'pwd' in request.POST:
        return change_pwd(request, model, **kw)

    if model == 'alias':
        form = mailforms.AliasForm
        redirect = '%smailserver/alias/list/' % settings.LOCATION

    if model == 'vacation':
        redirect = '%smailserver/vacation/list/' % settings.LOCATION
        
    if redirect:
        kw.update({ 'post_save_redirect' : redirect })

    return generic.update_object(request, model, ext="_update", form=form, **kw)

@login_required
def alias_detail(request, alias_name, domain_id):

    queryset = Alias.objects.filter(alias__exact=alias_name,
                                        domain__exact=Domain.objects.get(pk=domain_id))

    return generic.object_list(request, queryset,
                                 template_name='mailserver/alias_detail.mako',
                             extra_context={})

@login_required
def object_detail(request, model, object_id):
    return generic.object_detail(request, model, object_id)

@login_required
def change_pwd(request, model, object_id, **kw):

    a = Account.objects.get(pk=object_id)
    a.pwd = a.set_password(request.POST['pwd'])

    super(Account, a).save()

    return render_to_response('/close_popup.mako', dictionary={},
                              context_instance=RequestContext(request))

@login_required
def delete_account_or_domain(request, model, object_id, **kw):
    from django.views.generic.simple import redirect_to

    obj = None
    dir_to_delete = None
    go_to = ''

    mail_home = '/home/vmail'

    if model == 'account':
        obj = Account.objects.get(pk=object_id)
        dir_to_delete = '%s/%s/%s/' % (
            bool(obj.domain.name) and bool(obj.account) and mail_home, obj.domain.name, obj.account)
        go_to = '/mailserver/domain/%s/' % obj.domain.id
        domain = obj.domain
    elif model == 'domain':
        obj = Domain.objects.get(pk=object_id)
        dir_to_delete = '%s/%s/' % (bool(obj.name) and mail_home, obj.name)
        go_to = '/mailserver/domain/'
        domain = obj

    if obj and dir_to_delete and (request.user.has_perm('mailserver.supermaster_perm') or
                       (request.user in domain.postmaster.all() and model == 'account')):
        import os
        obj.delete()
        request.user.message_set.create(
            message="%s %s deleted." % (model, str(obj)) )
        import subprocess
        command = 'sudo -u postfix /usr/lib/python2.4/site-packages/jungle/apps/mailserver/scripts/delete_maildir.py %s' % dir_to_delete
        output = subprocess.Popen(command.split(), stdout=subprocess.PIPE).communicate()[0]

    return redirect_to(request, go_to or request.META['HTTP_REFERER'])
