from django.db import models

# Create your models here

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

##########################
# NEWS                   #
##########################
class News (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    type = models.ForeignKey (
        "NewsType",
        verbose_name="Tipo News",
    )
    ## Campi
    title = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Titolo",
    )
    date = models.DateTimeField (
        verbose_name="Data",
    )
    field = models.TextField (
        verbose_name="Corpo",
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )
    ## da inserire file ##
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "type", "date", )
        search_fields = ("type","date","title",)
        date_hierarchy = "date_create"
        list_display = ("status", "type","date","title",)
        list_display_links = ("type","date","title",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "type", "title",),
                ("field",),
                ("note"),
                ("odv"),
            ),}),
        )
        ordering = ("date","type ","odv",)

    class Meta:
        verbose_name = "News"
        verbose_name_plural = "News"

    def __unicode__(self):
        return unicode(self.date.isoformat()+" "+self.title)
    
##############################
# TIPO NEWS                  #
##############################
class NewsType (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Tipo",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 30
        fields = (
            (None,{"fields":(
                ("status", "name", "odv",),
            ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Tipo News"
        verbose_name_plural = "Tipi News"

    def __unicode__(self):
        return unicode(self.name)
    
############################
# POSTIT                   #
############################
class Postit (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    type = models.ForeignKey (
        "PostitType",
        verbose_name="Tipo Postit",
    )
    ## Campi
    title = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Titolo",
    )
    date = models.DateTimeField (
        verbose_name="Data",
    )
    field = models.TextField (
        verbose_name="Corpo",
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "type", "date",)
        search_fields = ("type","date","title",)
        date_hierarchy = "date_create"
        list_display = ("status", "type","date","title",)
        list_display_links = ("type","date","title",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "type", "title",),
                ("date",),
                ("field",),
                ("note",),
                ("odv",),
            ),}),
        )
        ordering = ("date","type","odv",)

    class Meta:
        verbose_name = "Post-it"
        verbose_name_plural = "Post-it"

    def __unicode__(self):
        return unicode(self.date.isoformat()+" "+self.title)
    
################################
# TIPO POSTIT                  #
################################
class PostitType (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Tipo",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "name", "odv",),
            ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Tipo Post-it"
        verbose_name_plural = "Tipi Post-it"

    def __unicode__(self):
        return unicode(self.name)    