#from jungle.middleware.thread_local import get_current_user
from jungle.openvpn.models import Certificate
from django import newforms as forms
from django.newforms import ModelForm
from django.db import transaction
from django.conf import settings
from jungle.db.formfield import ffc

from OpenSSL import crypto
from jungle.openvpn.certgen import *   # yes yes, I know, I'm lazy
from  datetime import *

from jungle.template import mako_loader as mloader
from django.template import RequestContext

import tarfile, tempfile
import os, os.path

from jungle.functions.conf import *
from jungle.forms.fields import JungleDateField

class CertificateForm(ModelForm):       
    class Meta:
        model = Certificate

    def __init__(self, *args, **kwargs):
        super(CertificateForm, self).__init__(*args, **kwargs)
        self.fields['start_date'] = JungleDateField(required=False)
        self.fields['stop_date'] = JungleDateField(required=False)

    def buf(self, filename):
        f = open(filename)
        buf = f.read()
        f.close()
        return buf
           
                   
    def create_tar_gz(self, certificate):
        os.chdir(settings.TAR_PATH)
        id = certificate.id
        #user = ""
        #if self.cleaned_data['user']:
        #    user = self.cleaned_data['user'].username

        self.CN = self.cleaned_data['user'].username
        CONF_DEFAULT_FILE = os.path.join(settings.CONF_PATH, "mk_vpn.conf")
        CA_KEY = settings.KEYSFILE_PATH
        CA_CERT = settings.CERTFILE_PATH
        
        KEY_NAME = 'pkey%s.key' % id
        CERT_NAME = "cert%s.crt" % id
        CONF_USER_FILE = "%s.ovpn" % settings.CONF_NAME
        TAR_NAME = "%s" % self.cleaned_data['user'].username
    
        # CA private key and certificate
        cakey = crypto.load_privatekey(crypto.FILETYPE_PEM, self.buf(CA_KEY))
        cacert = crypto.load_certificate(crypto.FILETYPE_PEM, self.buf(CA_CERT))
        
        # Creazione coppia di chiavi
        user_prkey = createKeyPair(TYPE_RSA, 1024)
        str_user_prkey = crypto.dump_privatekey(crypto.FILETYPE_PEM, user_prkey)
        
        # Creazione file chiavi pubbliche e private
        #f_pkey = tempfile.NamedTemporaryFile(prefix='%s' % user,suffix = '.pkey', mode='w+b')
        f_pkey = open(KEY_NAME, 'w')
        f_pkey.write(str_user_prkey)
        f_pkey.write('\n')
        f_pkey.flush()
        
        # creazione certificato
        req = createCertRequest(user_prkey, CN=self.CN)
        
        start_delta = self.cleaned_data['start_date'] or date.today() - timedelta(days=1)
        start_delta = start_delta - date.today()
        notBefore = start_delta.days * 86400

        stop_delta = self.cleaned_data['stop_date'] or date.today() + timedelta(days=365)
        stop_delta = stop_delta - date.today()
        notAfter = stop_delta.days * 86400
                                
        seconds = (notBefore, notAfter)
        cert = createCertificate(req, (cacert, cakey), 1000, seconds) 
        #f_crt = tempfile.NamedTemporaryFile(prefix='%s' % user,suffix = '.crt', mode='w+b')
        f_crt = open(CERT_NAME, 'w')
        f_crt.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
        f_crt.write('\n')        
        f_crt.flush()
                
        # Creazione file di configurazione .opvn
        c = open(CONF_DEFAULT_FILE)
        remote = ''
        for line in c.readlines():
            if line.startswith('REMOTE='):
                remote = line[line.index('=')+1:]
        
        CONF_VAR = {'remote': remote, 'cert': CERT_NAME, 'key': KEY_NAME,
                    'use_password': getattr(settings, "USE_PASSWORD", True)}
        if remote:
            conf_file = mloader.render_to_response("openvpn/client_conf.mako", CONF_VAR)
        else:
            #TO DO: RAISE EXCEPTION
            pass

        #f_conf = tempfile.NamedTemporaryFile(prefix='%s' % user,suffix = '.%s' % DATI_CONF['EXT'], mode='w+b')
        f_conf = open(CONF_USER_FILE, 'w')
        f_conf.write(conf_file.content)
        f_conf.write('\n')
        f_conf.flush()
        
        ext_cert = getattr(settings, 'EXT_CERT', '.tar.gz')
        if ext_cert == 'zip':
            self.create_zip(TAR_NAME, KEY_NAME, CERT_NAME, CONF_USER_FILE, CA_CERT)
        else:
            self.create_tar(TAR_NAME, KEY_NAME, CERT_NAME, CONF_USER_FILE, CA_CERT)
            
        os.remove(KEY_NAME)
        os.remove(CERT_NAME)
        os.remove(CONF_USER_FILE)        
        
    
    def create_tar(self, tar_name, key_name, cert_name, conf_user_file, ca_cert):
        tar = tarfile.open("%s.tar.gz" % tar_name, "w:gz")
        
        for path, name in ((key_name, key_name), (cert_name, cert_name), 
                           (conf_user_file, conf_user_file), (ca_cert, "ca.crt")):
            tarinfo = tar.gettarinfo(path, 'openvpn/'+ name)
            tar.addfile(tarinfo, file(path))
        tar.close()

    def create_zip(self, tar_name, key_name, cert_name, conf_user_file, ca_cert):
        import zipfile
        z = zipfile.ZipFile("%s.zip" % tar_name, "w")
        z.write(key_name, '/openvpn/%s' % key_name)
        z.write(cert_name, '/openvpn/%s' % cert_name)
        z.write(conf_user_file, '/openvpn/%s' % conf_user_file)
        z.write(ca_cert, '/openvpn/ca.crt')
        z.close()
           
    @transaction.commit_manually
    def save(self):
        try:
            certificate = super(CertificateForm, self).save()
            self.create_tar_gz(certificate)
            
            # Salvo nel database anche il COMMON NAME associato al certificato
            #
            c = Certificate.objects.get(pk=certificate.id)
            c.cn = self.CN
            c.save()
            
            c.refresh_rules()
                            
        except Exception, e:
            transaction.rollback()
            raise Exception(e)
        else:
            transaction.commit()
            return certificate


class CertificateFormUpdate(ModelForm):
    class Meta:
        model = Certificate
            
    def __init__(self, *args, **kwargs):
        self.cert = Certificate.objects.get(id=args[0]['id'])
        kwargs['instance'] = self.cert
        super(CertificateFormUpdate, self).__init__(*args, **kwargs)
            
    def save(self):
        new_cert = super(CertificateFormUpdate, self).save()
        new_cert.refresh_rules()
        return new_cert
