var NUM_PER_PAGE = 25; // Numero di righe visualizzate in ogni pagina della griglia
var grid_cert;

Ext.onReady(function(){
    var b_riavvia = new Ext.Button({renderTo:'restart_openvpn',text:'Riavvia Openvpn'});
    b_riavvia.on('click', confirm_restart);
    
    var b_onoff = new Ext.Button({renderTo:'endis_openvpn',text:'Accendi/Spegni Openvpn'});
    b_onoff.on('click', confirm_onoff);
        
    /***********************
     * RENDER FUNCTIONS
     ***********************/

    function renderEdit(value, p, record){
        return String.format("<a href='/openvpn/certificate/edit/"+record.id+"/'><img src='/static/img/edit.gif' title=\"Modifica Certificato\" /></a>");
    }

    function renderDownload(value, p, record){
        return String.format("<a href='/openvpn/downloads/"+record.data.utente+"."+record.data.estensione+"'><img src='/static/img/download.gif' title=\"Scarica Certificato\" /></a>");
    }

    function renderDelete(value, p, record){
        return String.format("<a href='#' onclick='confirm_delete(\""+record.id+"\", \""+record.data.description+"\");return false'><img src='/static/img/delete.gif' title=\"Cancella Certificato\" /></a>");
    }
    
    /***********************
     * END RENDER FUNCTIONS
     ***********************/

    var myProxy = new Ext.data.HttpProxy({
        url: '/openvpn/certificate/ajax/list/',
        method: 'GET'
    });

    var myReader = new Ext.data.JsonReader({
        root: 'data',
        totalProperty: 'total',
        id: 'id',
        fields: [
            {name: 'utente'},
            {name: 'status'},
            {name: 'description'},
            {name: 'start_date'},
            {name: 'stop_date'},
            {name: 'cn'},
            {name: 'estensione'},
            {name: 'group'},
            {name: 'edit'},
        ]
    });

    var cm = new Ext.grid.ColumnModel([
        {header: "Stato", width:15, dataIndex: 'status'},
        {header: "User", width:15, dataIndex: 'utente', summaryType: 'max',
            summaryRenderer: function(v, params, data){return 'Totale';}},

        {header: "Descrizione", width:30, dataIndex: 'description', summaryType: 'count',
            summaryRenderer: function(v, params, data){
                                return ((v === 0 || v > 1) ? v +' Certificati' : '1 Certificato');
                             }},

        {header: "Start", width:15, renderer: Ext.util.Format.dateRenderer('d/m/Y'), 
            dataIndex: 'start_date', sortable : true},

        {header: "Stop", width:15, renderer: Ext.util.Format.dateRenderer('d/m/Y'), 
            dataIndex: 'stop_date', sortable : true},

        {header: "Common Name", width:25, dataIndex: 'cn'},

        {header: "Gruppo", width:20, dataIndex: 'group', sortable : true},

        {header: "", width: 10, align: "center", renderer: renderEdit, resizable: false, dataIndex: 'icon_m'},
        {header: "", width: 10, align: "center", renderer: renderDownload, resizable: false, dataIndex: 'icon_d'},
        {header: "", width: 10, align: "center", renderer: renderDelete, resizable: false, dataIndex: 'icon_d'}
    ]);

    // Rendo le colonne sortabili di defautl
    cm.defaultSortable = false;

    // create the Data Store
    ds = new Ext.data.GroupingStore({
        proxy: myProxy,
        reader: myReader,
        remoteSort: true,
        groupField: 'group',
        remoteGroup: true
    });

    ds.setDefaultSort('start_date', 'ASC');

    grid_cert = new Ext.grid.GridPanel({
        el:'grid_cert',
        title:'Lista Certificati',
        width:'80%',
        loadMask: true,        
        viewConfig: {forceFit: true},
        autoHeight : true,
        enableColumnHide : false, // Tolgo dal context menu degli header il valore "Columns"
        store: ds,
        cm: cm,

        view: new Ext.grid.GroupingView({
            forceFit:true,
            enableNoGroups:false,
            enableGrouping:false,
            sortAscText: 'Ordinamento crescente',
            sortDescText: 'Ordinamento decrescente',
            groupByText : 'Raggruppa per questo campo',
            groupTextTpl: '{text}'
        }),

        // bottom toolbars
        bbar: new Ext.PagingToolbar({
            pageSize: NUM_PER_PAGE,
            store: ds,
            displayInfo: true,
            displayMsg: 'Visualizzo certificati {0} - {1} di {2}',
            emptyMsg: "Nessun certificato da visualizzare"
        }),

        plugins: new Ext.grid.GroupSummary()
    });

    grid_cert.render();
    // trigger the data store load
    ds.load({params:{start:0, limit:NUM_PER_PAGE}});

    // Aggiungo la ricerca via Ext
    ext_search = new ExtSearch({'url':'/openvpn/certificate/ajax/list/',
                                'grid':grid_cert,
                                'button_id':'cerca'});
});

function confirm_restart() {
    ext_confirm = new ExtConfirm({
                        'url': '/openvpn/restart_openvpn',
                        'msgConfirm': "Sei sicuro di voler riavviare il servizio 'openvpn'?",
                        'progressText': "Riavvio in corso. Attendere..",
                        'success_msg' : "Il servizio e' stato riavviato correttamente"
                 });
    ext_confirm.show();    
}

function confirm_onoff() {
    ext_confirm = new ExtConfirm({
                        'url': '/openvpn/onoff_openvpn',
                        'msgConfirm': "Sei sicuro di voler accendere/spegnere il servizio 'openvpn'?",
                        'progressText': "Accensione/Spegnimento in corso. Attendere..",
                        'success_msg' : "Il servizio e' stato acceso/spento correttamente.",
                        'post_confirm' : reload_page
                 });
    ext_confirm.show();
}

function reload_page() {
    window.location.reload();
}

function confirm_delete(id, descrizione) {
    ext_confirm = new ExtConfirm({
                        'url': '/openvpn/certificate/ajax/del/'+id+'/',
                        'msgConfirm': "Sei sicuro di voler cancellare il certificato '<b>" + descrizione + "</b>. Cancellando il certificato verranno eliminati anche tutti i log delle connessioni relative a questo certificato. Procedere?",
                        'progressText': "Cancellazione in corso. Attendere..",
                        'success_msg' : "Il certificato e' stato cancellato con successo",
                        'post_confirm' : reload_grid
                 });
    ext_confirm.show();    
}

function reload_grid() {
    grid_cert.getStore().reload();
}