var NUM_PER_PAGE = 25; // Numero di righe visualizzate in ogni pagina della griglia
var grid;

Ext.onReady(function(){    
    
    var myProxy = new Ext.data.HttpProxy({
        url: '/openvpn/connection/ajax/list/',
        method: 'GET'
    });

    var myReader = new Ext.data.JsonReader({
        root: 'data',
        totalProperty: 'total',
        id: 'id',
        fields: [
            {name: 'cn'},
            {name: 'description'},
            {name: 'start_date'},
            {name: 'stop_date'},
            {name: 'ip_client'}
        ]
    });
    
    var cm = new Ext.grid.ColumnModel([
        {header: "Common Name", width:15, dataIndex: 'cn'},
        {header: "Description", width:25, dataIndex: 'description',
            summaryType: 'max', summaryRenderer: function(v, params, data){return 'Totale';}},
        
        {header: "Start", width:15, renderer: Ext.util.Format.dateRenderer('d/m/Y H:i'), 
            dataIndex: 'start_date', summaryType: 'count',
            summaryRenderer: function(v, params, data){
                                return ((v === 0 || v > 1) ? v +' Connessioni' : '1 Connessione');
                             }},
            
        {header: "Stop", width:15, renderer: Ext.util.Format.dateRenderer('d/m/Y H:i'), 
            dataIndex: 'stop_date'},
            
        {header: "IP Client", width:15, dataIndex: 'ip_client'}
    ]);

    // Rendo le colonne sortabili di defautl
    cm.defaultSortable = false;
    
    // create the Data Store
    ds = new Ext.data.GroupingStore({
        proxy: myProxy,
        reader: myReader,
        remoteSort: true,
        groupField: 'description',
        remoteGroup: true
    });
    
    ds.setDefaultSort('stop_date', 'DESC');
    
    grid = new Ext.grid.GridPanel({
        el:'grid_conn',
        title:'Lista Connessioni',        
        width:'98%',
        loadMask: true,        
        viewConfig: {forceFit: true},      
        autoHeight : true,
        enableColumnHide : false, // Tolgo dal context menu degli header il valore "Columns"
        store: ds,
        cm: cm,
                
        view: new Ext.grid.GroupingView({
            forceFit:true,
            enableNoGroups:false,
            enableGrouping:false,
            sortAscText: 'Ordinamento crescente',
            sortDescText: 'Ordinamento decrescente',
            groupByText : 'Raggruppa per questo campo',
            groupTextTpl: '{text}'
        }),

        // bottom toolbars
        bbar: new Ext.PagingToolbar({
            pageSize: NUM_PER_PAGE,
            store: ds,
            displayInfo: true,
            displayMsg: 'Visualizzo connessioni {0} - {1} di {2}',
            emptyMsg: "Nessuna connessione da visualizzare"
        }),
                
        plugins: new Ext.grid.GroupSummary()        
    });
    
    grid.render();
	    
    // trigger the data store load
    ds.load({params:{start:0, limit:NUM_PER_PAGE}});

    // Aggiungo la ricerca via Ext
    ext_search = new ExtSearch({'url':'/openvpn/connection/ajax/list/',
                                'grid':grid,
                                'button_id':'cerca'});
    ext_search.search();
});