from django.db import models
from django.conf import settings
from django.contrib.auth.models import User
from datetime import datetime
from jungle.functions.conf import *
from jungle.template import mako_loader as mloader
import os, os.path

###########################
# KEY                     #
###########################
class Key(models.Model):
    ## CAMPI GENERICI
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Date Create",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "Date Last Modify",
    )
    
    ## FOREIGNKEY
    user = models.ForeignKey(
        User,
        null = True, blank = True,
        verbose_name = "user",
    )
        
    identity = models.CharField(
        null = True, blank = True,                                
        verbose_name = "identity",
        max_length = 20
    )
    
    ## CAMPI MODELLO
    path_pub = models.FilePathField(
        path="/etc/openvpn/keys",
        verbose_name = "Path public",
    )
    path_priv = models.FilePathField(
        path="/etc/openvpn/keys",
        verbose_name = "Path private",
    )

    class Admin:
        pass

    def __unicode__(self):
        return unicode(self.path)


class Utenti(models.Model):
    ## CAMPI GENERICI
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Date Create",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "Date Last Modify",
    )

    username =  models.CharField(
        verbose_name = "username",
        max_length = 255,
        unique=True
    )
        
    password =  models.CharField(
        verbose_name = "password",
        max_length = 255
    )
     
    nome =  models.CharField(
        null = True, blank = True,                   
        verbose_name = "nome",
        max_length = 255
    )

    cognome =  models.CharField(
        null = True, blank = True,
        verbose_name = "cognome",
        max_length = 255
    )
    
    def __unicode__(self):
        return unicode(self.username)    

################################
# CERTIFICATE                  #
################################
STATUS = (
    (1, "Active"),
    (0, "Disactive"),
)

class Certificate(models.Model):
    ## CAMPI GENERICI
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Date Create",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "Date Last Modify",
    )
    status = models.IntegerField(
        choices = STATUS,
        db_index = True,
        verbose_name = "status",
    )
    ## FOREIGNKEY
    user = models.ForeignKey(
        Utenti,
        verbose_name = "user",
    )

    group = models.ForeignKey(
        "FwGroup",
        null = True, blank = True,        
        verbose_name = "group",
    )
    
    ## CAMPI MODELLO
    start_date = models.DateField(
        null = True, blank = True,
        verbose_name = "From",
    )
    stop_date =  models.DateField(
        null = True, blank = True,
        verbose_name = "To",
    )
    description = models.CharField(
        max_length = 255,
        verbose_name = "Description",
    )
    email = models.EmailField(
        null = True, blank = True,
        max_length = 30,
        verbose_name = "email",
    )
    cn = models.CharField(
        null = True, blank = True,                          
        max_length = 100,
        verbose_name = "Common Name",        
    )
    # Richiesta di certificato - File temporaneo
    csr = models.FilePathField(
        null = True, blank = True,                               
        path="/etc/openvpn/keys/crt",
        verbose_name = "Certificate Request",
    )

    class Admin:
        pass

    def refresh_rules(self):
        fw_file = os.path.join(settings.CERT_RULES_PATH, "%s.fw" % self.cn)
        
        if os.path.exists(fw_file):
            os.remove(fw_file) 
        
        if self.group:
            in_rules, fw_rules = self.group.get_rules(self.cn)
        
            conf_file = mloader.render_to_response("openvpn/default_cert_fw.mako", 
                                                   {'cn':self.cn,
                                                    'fw_rules':fw_rules,
                                                    'in_rules':in_rules})
        
            f = open(fw_file, 'w')    
            f.write(conf_file.content)  
            f.write('\n')  
            f.close()        

    def get_file_certificate(self):
        f_path = os.path.join(settings.TAR_PATH, self.cn)

    def delete_connections(self):
        Connection.objects.filter(certificate=self).delete()
    
    def delete_certificate_file(self):
        fname = "%s.%s" % (self.cn, settings.EXT_CERT)
        f_path = os.path.join(settings.TAR_PATH, fname)
        try:
            os.remove(f_path)
        except:
            pass
        
    def __unicode__(self):
        return unicode(self.description)


###############################
# CONNECTION                  #
###############################
class Connection(models.Model):
    ## CAMPI GENERICI
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Date Create",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "Date Last Modify",
    )
    ## FOREIGNKEY
    certificate = models.ForeignKey(
        Certificate,
        verbose_name = "Certificate",
    )
    start_date = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Start"
    )
    stop_date = models.DateTimeField(
        blank = True, null = True,
        verbose_name = "Stop"
    )
    ip_client = models.CharField(
        null = True, blank = True,
        max_length = 20,
        verbose_name = "Client IP",
    )    

    class Admin:
        pass

    def __unicode__(self):
        return unicode(self.certificate)


###########################
# FW RULE                 #
###########################
PROTOCOL = (
    ('TCP', 'TCP'), 
    ('UDP', 'UDP'), 
    ('ICM', 'ICMP')
)

ACTION = (
    ('AC', 'ACCEPT'), 
    ('DR', 'DROP')
)

TABLE = (
    ('F', 'FILTER'),
)

CHAIN = (
    ('IN', 'INPUT'), 
    ('FW', 'FORWARD')
)

class FwRule(models.Model):
    ## CAMPI GENERICI
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Date Create",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "Date Last Modify",
    )

    ## FOREIGNKEY
    group = models.ManyToManyField(
        "FwGroup",
    )

    ## CAMPI MODELLO
    ip_src = models.CharField(
        null = True, blank = True,
        max_length = 20,        
        verbose_name = "Source IP",
    )
    
    ip_dest = models.CharField(
        null = True, blank = True,
        max_length = 20,
        verbose_name = "Destination IP",
    )
    
    port_dest = models.IntegerField(
        null = True, blank = True,
        verbose_name = "Destination Port"
    )

    proto = models.CharField (
        null = True, blank = True,
        max_length = 3,
        verbose_name = "Protocol",
        choices = PROTOCOL
    )
    
    rules = models.CharField (
        null = True, blank = True,
        verbose_name = "Rules",
        max_length = 255,
    )
    
    action = models.CharField (
        verbose_name = "Action",
        max_length = 2,
        choices = ACTION
    )

    table = models.CharField (
        max_length = 1,
        verbose_name = "Table",
        choices = TABLE
    )
    
    chain = models.CharField (
        max_length = 2,
        verbose_name = "Chain",
        choices = CHAIN
    )    
    
    order = models.IntegerField(
        verbose_name = "Order"
    )

    def delete(self):
        gruppi = [ g for g in self.group.all() ]
        
        super(FwRule, self).delete()

        for g in gruppi:
            l_cert = Certificate.objects.filter(group__exact=g.id)
            for c in l_cert:
                c.refresh_rules()

               
    def getRuleString(self, cn):
        s = ""
        
        if self.chain == 'IN':
            s = "iptb -A tuni_%s" % cn

        if self.chain == 'FW':
            s = "iptb -A tunf_%s" % cn
                    
        if self.ip_dest:
            s = s + " -d %s" % self.ip_dest
        
        if self.proto:
            s = s + " -p %s" % self.get_proto_display()
            
        if self.port_dest:
            s = s + " --dport %s" % self.port_dest
        
        s = s + " -j %s" % self.get_action_display()
        return s
        
    def __unicode__(self):
        return u"%s" % (self.ip_dest or self.rules )

    class Admin:
        pass


############################
# FW GROUP                 #
############################
class FwGroup(models.Model):
    ## CAMPI GENERICI
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "Date Create",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "Date Last Modify",
    )

    name = models.CharField (
        max_length = 100,
        verbose_name = "Name",
        unique=True
    )  
    
    # commentare quando si fa il syncdb iniziale
    #rule = models.ManyToManyField(
    #    FwRule,
    #    blank = True, null = True,
    #    db_table = 'openvpn_fwrule_group'
    #)
    
    def get_rules(self, cn):        
        groups = FwGroup.objects.get(id=self.id)

        fw_rules = ""
        in_rules = ""
        
        for rule in groups.fwrule_set.order_by('chain','order'):
            r = rule.getRuleString(cn)
            if rule.chain == 'IN':
                in_rules += "%s\n" % r
                
            if rule.chain == 'FW':
                fw_rules += "%s\n" % r

        return [in_rules, fw_rules]

        
    def __unicode__(self):
        return u"%s" % (self.name)    

    class Admin:
        pass    


############################
# FW GROUP RULES           #
############################
#class FwGroupsRules(models.Model):
#    rule = models.ForeignKey(FwRule)
#    group = models.ForeignKey(FwGroup)

#    def __unicode__(self):
#        return u'%s - %s' % (self.rule, self.group)    