# -*- coding: utf-8 -*-

from jungle.views.decorators import login_required
from jungle.views import mgeneric as generic
from jungle.openvpn.models import Key, Certificate, Connection, FwRule, FwGroup, Utenti
from django.utils import simplejson
from django.http import HttpResponse, HttpResponseForbidden, HttpResponseNotFound
from django.shortcuts import render_to_response
from django.conf import settings
from datetime import timedelta
import os.path, commands
from jungle.functions.conf import *
import mimetypes
from django.db import transaction

@login_required
def add(request, model=None, **kw):
    if model == 'certificate':
        from certificate_form import CertificateForm
        return generic.create_object(request, model, form=CertificateForm, **kw)

    if model == 'utenti':
        from utenti_form import UtentiForm
        kw['post_save_redirect'] = '%sopenvpn/utenti/' % (settings.LOCATION,)
        return generic.create_object(request, model, form=UtentiForm, **kw)

    if model == 'fwrule':
        from fwrules_form import FwRuleForm
        kw['post_save_redirect'] = '%sopenvpn/fwrule/' % (settings.LOCATION,)
        return generic.create_object(request, model, form=FwRuleForm, **kw)
                                     
    if model == 'fwgroup':
        kw['post_save_redirect'] = '%sopenvpn/fwgroup/' % (settings.LOCATION,)
    
    return generic.create_object(request, model, **kw)

    
@login_required
def update(request, model=None, **kw):
    if model == 'certificate':
        from certificate_form import CertificateFormUpdate  
        kw['post_save_redirect'] = '%sopenvpn/certificate/' % (settings.LOCATION,)        
        return generic.update_object(request, model, form=CertificateFormUpdate, **kw)

    if model == 'fwrule':
        from fwrules_form import FwRuleFormUpdate  
        kw['post_save_redirect'] = '%sopenvpn/fwrule/' % (settings.LOCATION,)        
        return generic.update_object(request, model, form=FwRuleFormUpdate, **kw)

    if model == 'fwgroup':
        kw['post_save_redirect'] = '%sopenvpn/fwgroup/' % (settings.LOCATION,)

    if model == 'utenti':
        from utenti_form import UtentiFormUpdate
        kw['post_save_redirect'] = '%sopenvpn/utenti/' % (settings.LOCATION,)
        return generic.update_object(request, model, form=UtentiFormUpdate, ext="_update", **kw)
    
    return generic.update_object(request, model, ext="_update", **kw)
    

@login_required
def object_list(request, model, filter_cnd={}, order_by='-date_create',
                search_params={}, form=None, **kw):

    if model == 'certificate':
        queryset = Certificate.objects.all()
    if model == 'fwrule':
        queryset = FwRule.objects.all()
    if model == 'fwgroup':
        queryset = FwGroup.objects.all()        
    if model == 'connection':
        queryset = Connection.objects.all()        
    if model == 'utenti':
        queryset = Utenti.objects.all()        

    return generic.jobject_list(request, queryset, model, form,
                                filter_cnd, order_by, **kw)
    
@login_required
def object_detail(request, model, object_id):
    #print request
    #r = HttpResponse(open('/home/marco/cert/cert.tar.gz'))
    #r['Content-Disposition'] = 'attachment; filename=cert.tar.gz'
    #r['Content-Type']= 'application/octet-stream'
    #return r
    return generic.object_detail(request, model, object_id)

@login_required
@transaction.commit_manually
def delete_ajax_object(request, model, object_id):
    msg = ""

    try:
        if model == 'utenti':
            q_user = Utenti.objects.get(pk=object_id)
            q_cert = Certificate.objects.filter(user=q_user)
            if q_cert:
                msg = "Attenzione! L'utente ha un certificato associato. Se si vuole eliminare l'utente bisogna prima effettuare la cancellazione del relativo certificato."
            else:
                q_user.delete()
                msg = "OK"
                
        if model == 'certificate':
            q_cert = Certificate.objects.get(pk=object_id)
            q_cert.delete_connections()
            q_cert.delete_certificate_file()
            q_cert.delete()
            msg = "OK"
    except:
        transaction.rollback()
        raise Exception(e)
    else:
        transaction.commit()
        return HttpResponse(msg)            
                        
    return HttpResponse(msg)
    
def get_status_openvpn():
    piddir="/var/run/openvpn.server-tcp.status"
    stato = "SPENTO"
    if os.path.exists(piddir):
        stato="ACCESO"
    
    return stato

@login_required
def onoff_openvpn(request):
    stato = get_status_openvpn()
    if stato == "ACCESO":
        status, output = commands.getstatusoutput("sudo /etc/init.d/openvpn stop")
    else:
        status, output = commands.getstatusoutput("sudo /etc/init.d/openvpn start")
    return HttpResponse("OK")

@login_required
def restart_openvpn(request):
    status, output = commands.getstatusoutput("sudo /etc/init.d/openvpn restart")
    return HttpResponse("OK")

@login_required
def download_certificato(request, cert_name):
    if request.user.is_staff:
        f_path = os.path.join(settings.TAR_PATH, cert_name)

        if os.path.exists(f_path):
            a = open(f_path)
            mt = mimetypes.guess_type(f_path)[0]
            response = HttpResponse(a, mimetype='application/%s' % mt)
            response['Content-Disposition'] = 'attachment; filename=%s' % cert_name
            return response
        else:
            return HttpResponseNotFound('<h1>Certificate not found</h1>')
    else:
        return HttpResponseForbidden("<h1>Access Denied</h1>")

@login_required
def get_search_data(request, model):       
    user = request.user
    get =  request.GET.copy()

    ### CERTIFICATE ###
    if model == 'certificate':
        model_s = CertificateSearch()
    if model == 'connection':
        model_s = ConnectionSearch()
    if model == 'fwrule':
        model_s = FwRuleSearch()        
    if model == 'fwgroup':
        model_s = FwGroupSearch()        
    if model == 'utenti':
        model_s = UtentiSearch()


    start = get.get('start', 0)
    limit = get.get('limit', 25)
    groupby = get.get('groupBy', '')
    orderby = get.get('sort', model_s.default_sort)
    direction = get.get('dir', 'ASC')
        
    queryset = model_s.get_query_set()

    # Gestione della ricerca
    #
    from jungle.views.fast_search import search2
    filter_cnd, oldorder_by, form = search2(request, model)
    
    if 'change_filter_cnd' in dir(model_s):
        filter_cnd = model_s.change_filter_cnd(filter_cnd)        

    # Nella ricerca della lista delle regole non si puo usare il 
    # normale metodo perche' deve essere fatta seguendo 
    # una ManyToMany relation.
    # Quindi, solo in questo caso, la ricerca viene effettuata nel metodo get_dati
    #
    if model != 'fwrule':
        if filter_cnd:
            queryset = queryset.filter(**filter_cnd)

            
    total = queryset.count()
    queryset = model_s.order_by(queryset, groupby, orderby, direction)
    queryset = queryset[int(start):int(start)+int(limit)]

    if model == 'fwrule':
        dati = model_s.get_dati(queryset, filter_cnd)
    else:
        dati = model_s.get_dati(queryset)
        
    data = '{"success": true, "total": %s, "data": %s}' % (total, simplejson.dumps(dati, ensure_ascii=False))
    return HttpResponse(data, mimetype='text/javascript;')


class CertificateSearch:
    default_sort = 'start_date'
    
    def get_query_set(self):
        queryset = Certificate.objects.all()
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by('group', orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            if x.user:
                d['utente'] = x.user.username
            else:
                d['utente'] = ''
            d['description'] = x.description
            if x.start_date:
                d['start_date'] = x.start_date.strftime("%m/%d/%Y")

            if x.stop_date:
                d['stop_date'] = x.stop_date.strftime("%m/%d/%Y")
                
            d['cn'] = x.cn
            d['status'] = x.get_status_display()
            d['estensione'] = settings.EXT_CERT
            if x.group:
                d['group'] = x.group.name
            else:
                d['group'] = ' '
            dati.append(d)
        return dati        


class UtentiSearch:
    default_sort = 'username'
    
    def get_query_set(self):
        queryset = Utenti.objects.all()
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by(orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['username'] = x.username
            if x.nome:
                d['nome'] = x.nome
            else:
                d['nome'] = ""
                
            if x.cognome:
                d['cognome'] = x.cognome
            else:
                d['cognome'] = ""
                
            dati.append(d)
        return dati        
            
        
class ConnectionSearch:
    default_sort = '-stop_date'
    
    def get_query_set(self):
        return Connection.objects.all()

    def change_filter_cnd(self, filter_cnd):
        date_from = None
        date_to = None
        
        if filter_cnd.has_key('stop_date__isnull'):
            return filter_cnd

        if filter_cnd.has_key('from_date__exact'):
            date_from = filter_cnd['from_date__exact']
            del filter_cnd['from_date__exact']
        if filter_cnd.has_key('to_date__exact'):
            date_to = filter_cnd['to_date__exact']
            del filter_cnd['to_date__exact']
        
        if date_from and date_to:
            filter_cnd['start_date__range'] = (date_from, date_to + timedelta(days=1))
        elif date_from:
            filter_cnd['start_date__gte'] = date_from
        elif date_to:
            filter_cnd['start_date__lt'] = date_to + timedelta(days=1)
            
        if filter_cnd.has_key('description__icontains'):
            desc = filter_cnd['description__icontains']
            del filter_cnd['description__icontains']
            filter_cnd['certificate__description__icontains'] = desc
        return filter_cnd
    
    def order_by(self, queryset, groupby, orderby, dir):
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by('certificate', orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['cn'] = x.certificate.cn
            d['description'] = x.certificate.description
            d['start_date'] = x.start_date.strftime("%m/%d/%Y %H:%M")
            if x.stop_date:
                d['stop_date'] = x.stop_date.strftime("%m/%d/%Y %H:%M")
            d['ip_client'] = x.ip_client
            dati.append(d)    
        return dati
    
class FwRuleSearch:
    default_sort = 'name'
    
    def get_query_set(self):
        return FwGroup.objects.all()
    
    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by('name')       
        
    def get_dati(self, queryset, filter_cnd):
        dati = []
        for x in queryset:
            # Per ogni set di regole del gruppo eseguo il filtraggio in base
            # alle condizioni di ricerca
            #
            for y in x.fwrule_set.filter(**filter_cnd).order_by('chain', 'order'):
                d = {}
                d['id'] = y.id
                d['group'] = x.name
                d['table'] = y.get_table_display()
                d['chain'] = y.get_chain_display()
                d['d_ip'] = y.ip_dest
                d['proto'] = y.get_proto_display()
                d['d_port'] = y.port_dest
                d['action'] = y.get_action_display()
                d['order'] = y.order
                dati.append(d)    
        return dati    
    
class FwGroupSearch:
    default_sort = 'name'
    
    def get_query_set(self):
        queryset = FwGroup.objects.all()
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by(orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['name'] = x.name
            dati.append(d)
        return dati            