from django.db import models
from jungle.ref.models import Month

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

####################################
# PAGAMENTO FORMA                  #
####################################
class PaymentForm (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Forma di Pagamento",
    )
    description = models.TextField (
        blank=True,
        verbose_name="Descrizione",
    )
    fee = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Spesa",
        default=0.00,
    )
    discount = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Sconto",
        default=0.00,
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name", "fee", "discount",)
        list_display_links = ("code", "name",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "code", "name",),
                ("description",),
                ("fee", "discount",),
                ("odv",),
            ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Forma Pagamento"
        verbose_name_plural = "Forme Pagamento"

    def __unicode__(self):
        return unicode(self.name)
    
###################################
# RATA PAGAMENTO                  #
###################################
class PaymentInstallment (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    payment_form = models.ForeignKey (
        'PaymentForm',
        verbose_name="Forma di Pagamento",
    )
    ## Campi
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Rata",
    )
    description = models.TextField (
        blank=True,
        verbose_name="Descrizione",
    )
    amount = models.IntegerField (
        default=100,
        verbose_name="Quota %",
    )
    days = models.IntegerField (
        default=0,
        verbose_name="Giorni",
    )
    end_month = models.BooleanField (
        verbose_name="Fine Mese",
    )
    discount = models.DecimalField (
        max_digits=4,
        decimal_places=2,
        verbose_name="Sconto",
        default=0.00,
    )
    fee = models.DecimalField (
        max_digits=4,
        decimal_places=2,
        verbose_name="Spesa",
        default=0.00,
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "payment_form",)
        search_fields = ("name",)
        date_hierarchy = "date_create"
        list_display = ("status", "payment_form", "name", "amount", "days", "end_month", "fee", "discount",)
        list_display_links = ("name",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "payment_form", "name",),
                ("description",),
                ("amount", "days", "end_month",),
                ("fee", "discount",),
                ("odv",),
            ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Rata Pagamento"
        verbose_name_plural = "Rate Pagamento"

##########################################
# PAGAMENTO MESE ESCLUSO                 #
##########################################
class OutMonth (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    payment_form = models.ForeignKey (
        "PaymentForm",
        verbose_name="Forma di Pagamento",
    )
    #month = models.ForeignKey (
        #"Month",
        #verbose_name="Mese",
    #)
    ## Campi
    month_delay = models.IntegerField (
        default=0,
        verbose_name="Tra mesi",
    )
    day_of_month = models.IntegerField (
        default=0,
        verbose_name="Giorno",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "payment_form",)
        search_fields = ("name",)
        date_hierarchy = "date_create"
        list_display = ("status", "payment_form", "month_delay", "day_of_month",)
        list_display_links = ("month_delay",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "payment_form", "month_delay", "day_of_month", "odv",),
            ),}),
        )
        ordering = ("payment_form", "odv","name",)

    class Meta:
        verbose_name = "Mese Escluso"
        verbose_name_plural = "Mesi Esclusi"