from django.db import models

##########################
## DATI AZIENDA PER RID ##
##########################
class OrganizationRID(models.Model):
    code_SIA = models.CharField (
        max_length=5,
        help_text="MITTENTE: Codice assegnato dalla Sia all'Azienda\nMittente; e' censita sul Directory",
    )
    code_ABI = models.CharField (
        max_length=5,
        help_text="RICEVENTE ABI: Codice ABI della banca assuntrice cui\ndevono essere inviate le disposizioni; e' censita sul Directory",
    )
    code_CAB = models.CharField (
        max_length=5,
        help_text="RICEVENTE CAB: Codice CAB della banca assuntrice cui\ndevono essere inviate le disposizioni",
    )
    bank_account = models.CharField (
        max_length=12,
        help_text="CONTO ORDINANTE: Codice Conto Corrente del cliente\nordinante l'incasso"
    )
    payment_description = models.CharField (
        max_length=20,
        help_text = "NOME SUPPORTO: Campo di libera composizione da\nparte dell'Azienda Mittente; dev'essere univoco nell'ambito della data\ndi creazione e a parita' di mittente e ricevente",
    )
    description1 = models.CharField (
        max_length=30,
        help_text="1o SEGMENTO: Primo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente ordinante",
    )
    description2 = models.CharField (
        max_length=30,
        help_text="2o SEGMENTO: Secondo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente ordinante",
    )
    description3 = models.CharField (
        max_length=30,
        help_text="3o SEGMENTO: Terzo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente ordinante",
    )
    class Admin:
        pass
##########################
## RID                  ##
##########################
class RID(models.Model):
    code_head_RID = models.CharField (
        max_length=2,
        default="IR",
        help_text="TIPO RECORD TESTA",
        ## record testa
    )
    code_end_RID = models.CharField (
        max_length=2,
        default="EF",
        help_text="TIPO RECORD CODA",
        ## record coda
    )
    code_SIA = models.CharField (
        max_length=5,
        help_text="CODICE AZIENDA: Codice assegnato dalla Sia all'Azienda Mittente; e' censita sul Directory",
        ## record testa
        ## record coda
        ## record 10
    )
    code_ABI = models.IntegerField (
        max_length=5,
        help_text="ABI BANCA ASSUNTRICE: Codice ABI della banca assuntrice\ncui devono essere inviate le disposizioni; e' censita sul Directory",
        ## zerofill
        ## record testa
        ## record coda
        ## record 10
    )
    code_CAB = models.IntegerField (
        max_length=5,
        help_text="CAB BANCA: Codice CAB della banca assuntrice cui devono essere inviate le disposizioni",
        ## zerofill
        ## record 10
    )
    bank_account = models.CharField (
        max_length=12,
        help_text="CONTO ORDINANTE: Codice Conto Corrente del cliente ordinante l'incasso"
        ## zerofill
        ## record 10
    )
    date_send = models.CharField (
        max_length=6,
        help_text="DATA CREAZIONE: Data di creazione del 'flusso' da parte dell'Azienda mittente nel formato GGMMAA",
        ## record testa
        ## record coda
    )
    payment_description = models.CharField (
        max_length=20,
        help_text="NOME SUPPORTO: Campo di libera composizione da parte dell'Azienda Mittente;\ndev'essere univoco nell'ambito della data di creazione e a parita' di mittente e ricevente",
        ## record testa
        ## record coda
    )
    description1 = models.CharField (
        max_length=30,
        help_text="1o SEGMENTO: Primo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente ordinante",
        ## record 20
    )
    description2 = models.CharField (
        max_length=30,
        help_text="2o SEGMENTO: Secondo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente ordinante",
        ## record 20
    )
    description3 = models.CharField (
        max_length=30,
        help_text="3o SEGMENTO: Terzo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente ordinante",
        ## record 20
    )
    payments_number = models.IntegerField (
        max_length=7,
        help_text="NUMERO DISPOSIZIONI: Numero delle disposizioni\n(richieste di incasso RID contenute nel flusso)",
        ## record coda
    )
    total_negatives = models.DecimalField (
        max_digits=15,
        decimal_places=2,
        help_text="TOTALE IMPORTI NEGATIVI: Importo totale delle disposizioni contenute nel flusso",
        ## record coda
    )
    total_positives = models.DecimalField (
        max_digits=15,
        decimal_places=2,
        help_text="TOTALE IMPORTI POSITIVI: Importo totale delle disposizioni\ncontenute nel flusso - deve essere valorizzato con 'zeri'",
        ## record coda
    )
    records_number = models.IntegerField (
        max_length=7,
        help_text="NUMERO RECORD: Numero dei record che compongono il flusso (comprensivo del record di testa e di coda)",
        ## record coda
    )
    code_money = models.CharField (
        max_length=1,
        default="E",
        help_text="CODICE DIVISA: Assume il valore fisso 'E' (Euro)",
        ## record testa
        ## record coda
        ## record 10
    )
    class Admin:
        pass
##########################
## RIGA RID             ##
##########################
class RIDRow(models.Model):
    rid = models.ForeignKey (
            "RID",
    )
    rid_number = models.IntegerField (
        max_length=7,
        help_text="NUMERO PROGRESSIVO: Numero della disposizione all'interno del flusso. Inizia con 1 ed e' progressivo di 1",
        ## record corpo
    )
    date_guaranty = models.DateField (
        max_length=6,
        help_text="DATA DECORRENZA GARANZIA: Data di decorrenza della garanzia; deve essere obbligatoriamente\nvalorizzata nelle disposizioni di incasso derivanti da un allineamento elettronico\narchivi originato da Azienda e contiene la scadenza della prima rata nel formato GGMMAA",
        ## record 10
    )
    date_limit = models.DateField (
        null=True,
        blank=True,
        max_length=6,
        help_text="DATA LIMITE PAGAMENTO: Da non valorizzare - Se valorizzata,\ne' la data di addebito posteriore a quella di scadenza (data\nvaluta destinatario) e indica che si tratta della presentazione di un RID 'scaduto')",
        ## record 10
    )
    date_money = models.DateField (
        max_length=6,
        help_text="DATA VALUTA DESTINATARIO: Data di valuta che la banca\ndestinataria deve applicare al cliente destinatario nel formato GGMMAA",
        ## record 10
    )
    amount = models.DecimalField (
        max_digits=13,
        decimal_places=2,
        help_text="IMPORTO: Importo della disposizione",
        ## record 10
    )
    customer_code = models.CharField (
        max_length=16,
        help_text="CODICE CLIENTE DEBITORE: Codice con il quale il debitore e' conosciuto dal creditore",
        ##record 10
    )
    customer_code_ABI = models.IntegerField (
        help_text="ABI BANCA DOMICILIATARIA: Codice ABI della banca domiciliataria delle richieste di incasso",
        ## zerofill
        ## record 10
    )
    customer_code_CAB = models.IntegerField (
        help_text="CAB BANCA DOMICILIATARIA: Codice ABI della banca domiciliataria delle richieste di incasso",
        ## zerofill
        ## record 10
    )
    customer_bank_account = models.CharField (
        max_length=12,
        help_text="CONTO CORRENTE DEBITORE: Codice Conto Corrente del cliente debitore",
        ## zerofill
        ## record 10
    )
    customer_description1 = models.CharField (
        max_length=30,
        help_text="1o SEGMENTO: Primo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente debitore",
        ## record 30
    )
    customer_description2 = models.CharField (
        blank=True,
        null=True,
        max_length=30,
        help_text="2o SEGMENTO: Secondo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente debitore",
        ## record 30
    )
    customer_description3 = models.CharField (
        blank=True,
        null=True,
        max_length=30,
        help_text="3o SEGMENTO: Terzo segmento di 30 caratteri:\ndenominazione o ragione sociale del cliente debitore",
        ## record 30
    )
    customer_address = models.CharField (
        max_length=30,
        help_text="INDIRIZZO DEBITORE: Via, numero civico e/o nome della frazione del cliente debitore",
        ## record 40
    )
    customer_ZIP = models.CharField (
        max_length=5,
        help_text="CAP DEBITORE: Codice di avviamento postale del cliente debitore",
        ## record 40
    )
    customer_city = models.CharField (
        max_length=23,
        help_text="COMUNE DEBITORE: Comune del cliente debitore",
        ## record 40
    )
    customer_province = models.CharField (
        max_length=2,
        help_text="PROVINCIA DEBITORE: Sigla della provincia del cliente debitore",
        ## record 40
    )
    payment_description1 = models.CharField (
        max_length=45,
        help_text="1o SEGMENTO: Primo segmento di 45 caratteri:\nriferimenti del debito",
        ## record 50
    )
    payment_description2 = models.CharField (
        blank=True,
        null=True,
        max_length=45,
        help_text="2o SEGMENTO: Secondo segmento di 45 caratteri:\nriferimenti del debito",
        ## record 50
    )
    class Admin:
        pass
###################
## CARTA CREDITO ##
###################
class CreditCardRow(models.Model):
    action_number = models.IntegerField (
        max_length=1,
        ##4: authorization, 1: purchase, 5: capture (se precedente 4) 2: credit (se precedente 1 o 5)
    )
    date_send = models.DateField (
    )
    customer_card_brand = models.CharField (
        blank=True,
        null=True,
        max_length=20,
        ##(VISA, MC, AMEX, DINERS)
        ## viene inserito dal flusso di ritorno
    )
    customer_card_number = models.CharField (
        max_length=16,
    )
    customer_card_exp = models.CharField (
        max_length=4,
        ##(AAMM: data scadenza)
    )
    customer_card_CVV2 = models.CharField (
        blank=True,
        null=True,
        max_length=3,
        ##(numero 3 cifre retro carta)
    )
    customer_member = models.CharField (
        blank=True,
        null=True,
        max_length=50,
        ##(nome e cognome proprietario carta)
    )
    customer_address = models.CharField (
        blank=True,
        null=True,
        max_length=50,
    )
    customer_ZIP = models.CharField (
        blank=True,
        null=True,
        max_length=5,
        help_text="CAP DEBITORE: Codice di avviamento postale del cliente debitore",
        ## record 40
    )
    curr = models.IntegerField (
        max_length=3,
        default=978,
        ##(978 divisa EURO)
    )
    amount = models.DecimalField (
        max_digits=13,
        decimal_places=2,
    )
    payment_description1 = models.CharField (
        max_length=50,
        blank=True,
        null=True,
    )
    payment_description2 = models.CharField (
        max_length=50,
        blank=True,
        null=True,
    )
    payment_description3 = models.CharField (
        max_length=50,
        blank=True,
        null=True,
    )
    payment_description4 = models.CharField (
        max_length=50,
        blank=True,
        null=True,
    )
    payment_description5 = models.CharField (
        max_length=50,
        blank=True,
        null=True,
    )
    trans_id_ref = models.IntegerField (
        max_length=20,
        blank=True,
        null=True,
        ##(da utilizzare solo per action 2 e 5 - assegnato da banca)
    )
    track_id = models.IntegerField (
        blank=True,
        null=True,
        max_length=10,
        #(numero univoco - potrebbe essere l'id stesso)
    )
    ## risposta ##
    status = models.IntegerField (
        max_length=1,
        blank=True,
        null=True,
        ##(0: esito tecnico negativo, 1: esito tecnico positivo)
    )
    error = models.CharField (
        max_length=25, # FIXME bastano 25?
        blank=True,
        null=True,
        ##(tipo di errore - se status=0)
    )
    result = models.CharField (
        max_length=30,
        blank=True,
        null=True,
        ##(CAPTURED, NOT CAPTURED)
    )
    auth = models.IntegerField (
        max_length=20,
        blank=True,
        null=True,
        ##(codice rilasciato dalla banca solo se Action = 1,4 e il result > 0)
    )
    ref = models.IntegerField (
        max_length=20,
        blank=True,
        null=True,
        ##(codice rilasciato dalla banca solo se Action = 1,4)
    )
    AVR = models.CharField (
        max_length=2,
        blank=True,
        null=True,
        default='NA',
        ##('NA': non assegnato)
    )
    date_money = models.DateField (
        blank=True,
        null=True,
        ##(MMGG)
    )
    trans_id = models.IntegerField (
        max_length=20,
        blank=True,
        null=True,
        ##(codice rilasciato dalla banca)
    )
    class Admin:
        pass
