from django.db import models
from jungle.organization.models import Organization, OrganizationForm, Person
from jungle.payment.models import PaymentForm
from jungle.userprofile.models import Office
from jungle.store.models import Store, StoreCause
from jungle.product.models import Product, ProductType, Vat
from jungle.address.models import Address, AddressType
from django.contrib.auth.models import User
from jungle.templatetags.jungle_tags import ugettext as _
# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

def get_next_pos():
    from datetime import date
    try:
        number = Procedure.objects.filter(date__year = date.today().year).order_by('-number')[0].number + 1
    except:
        number = 1

    return number

def get_next_neg():
    from datetime import date
    try:
        number = Procedure.objects.filter(date__year = date.today().year).order_by('number')[0].number
    except:
        number = 0

    return number - 1

###############################
# PRATICA                     #
###############################
class Procedure (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        verbose_name="Azienda",
    )
    organization_form = models.ForeignKey (
        OrganizationForm,
        verbose_name="Forma Societaria"
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        null=True,
        blank=True,
        verbose_name="Forma Pagamento",
    )
    office = models.ForeignKey (
        Office,
        verbose_name="Ufficio",
    )
    #bank = models.ForeignKey (
    #)
    #list = models.ForeignKey (
    #)
    store_cause = models.ForeignKey (
        StoreCause,
        verbose_name="Causale Magazzino"
    )
    ## Campi
    number = models.IntegerField (
        blank=True,
        verbose_name="Numero",
    )
    year = models.DateField (
        verbose_name="Anno"
    )
    date = models.DateField (
        verbose_name="Data"
    )
    ddt = models.BooleanField (
        verbose_name="DDT",
    )
    invoice = models.BooleanField (
        verbose_name="Fatturato",
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )

    def save(self):
        if not self.number:
            self.number = get_next_pos()
        super(Procedure, self).save()

    def totals(self):
        ## total_income, total_fee, total_discount, total
        rows = ProcedureRow.objects.filter(procedure__id = self.id)
        for row in rows:
            self.total_income += row.totals_income
            self.total_fee += row.total_fee
            self.total_discount += row.total_discount
            self.total += row.total

    def totals_vat(self):
        ## vat, total_income, total_vat, total_fee, total_discount, total (per ogni aliquota iva)
        rows = ProcedureRow.objects.filter(procedure__id = self.id).order_by('iva_id')

    class Admin:
        pass

    class Meta:
        verbose_name = "Ordine"
        verbose_name_plural = "Ordini"

    def __unicode__(self):
        return unicode(str(self.date)+" "+str(self.number))

##################################
# RIGA PRATICA                   #
##################################
class ProcedureRow (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    procedure = models.ForeignKey (
        Procedure,
        verbose_name = "Numero Ordine",
    )
    ### da aggiungere anche anno
    product_type = models.ForeignKey (
        ProductType,
        verbose_name = "Tipo Prodotto",
    )
    product = models.ForeignKey (
        Product,
        verbose_name = "Prodotto",
    )
    vat = models.ForeignKey (
        Vat,
        verbose_name = "IVA",
    )
    store_cause = models.ForeignKey (
        StoreCause,
        verbose_name="Causale Magazzino",
    )
    ## Campi
    quantity = models.IntegerField (
        verbose_name="Quantita'",
        default=0,
    )
    price = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        default=0.00,
        verbose_name="Prezzo",
    )
    fee = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Spesa",
        default=0.00,
    )
    discount = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Sconto",
        default=0.00,
    )
    ddt = models.BooleanField (
        verbose_name="DDT",
    )
    invoice = models.BooleanField (
        verbose_name="Fatturato",
    )

    def total_income(self):
        return self.quantity * self.price

    def total_fee(self):
        return self.quantity * self.fee

    def total_discount(self):
        return self.quantity * self.discount

    def total(self):
        return self.total_income + self.total_fee - self.total_discount

    class Admin:
        pass

    class Meta:
        verbose_name = "Riga Ordine"
        verbose_name_plural = "Righe Ordini"


#########################################
# ASSISTENZA                            #
#########################################
STATUS_CODES = (
    (0, _('closed')),
    (1, _('open')),
    (2, _('standby')),
    (3, _('waiting for input')),
    (4, _('not repeatable')),
)

PRIORITY_CODES = (
    (0, _('very urgent')),
    (1, _('urgent')),
    (2, _('important')),
    (3, _('normal')),
    (4, _('low')),
    (5, _('someday')),
)

class Assistance (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## Foreignkey
    submitter = models.ForeignKey (
        User,
        related_name="sub_assistance",
        verbose_name="Submitter",
    )
    manager = models.ManyToManyField (
        User,
        related_name='manager',
        verbose_name='Responsabile'
    )
    assigned_to = models.ForeignKey (
        User,
        blank=True, null=True,
        related_name='worker',
        verbose_name="Assegnato a",
    )
    organization = models.ForeignKey (
        Organization,
        verbose_name="Ragione Sociale",
    )
    address = models.ForeignKey (
        Address,
        verbose_name="Indirizzo Esecuzione Lavori",
    )
    person = models.ForeignKey (
        Person,
        verbose_name="Persona di Riferimento",
    )
    priority = models.IntegerField (
        default=1,
        choices=PRIORITY_CODES,
        verbose_name="Priority",
    )
    ## Campi modello
    description_product = models.TextField (
        null=True,
        verbose_name="Descrizione Prodotto",
    )
    description_defect = models.TextField (
        null=True,
        verbose_name="Descrizione Difetto",
    )
    description_job = models.TextField (
        null=True,
        verbose_name="Descrizione Lavoro",
    )

    class Admin:
        pass

#########################################
# RIGA ASSISTENZA                       #
#########################################

CARRYING_CHOICES = (
    ('Thunder', 'Thunder'),
    ('Proprio', 'Proprio'),
    ('Cliente', 'Cliente'),
    ('Pubblico', 'Pubblico'),
    ('Altro', 'Altro'),
)

class AssistanceRow (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## Foreignkey
    assistance = models.ForeignKey (
        Assistance,
        verbose_name="assistenza",
    )
    submitter = models.ForeignKey (
        User,
        related_name="sub_assistancerow",
        verbose_name="Submitter",
    )
    date_start = models.DateTimeField (
        verbose_name="Start Date",
    )
    date_end = models.DateTimeField (
        verbose_name="End Date",
    )
    work_duration = models.IntegerField (
        verbose_name="Ore Ordinarie",
    )
    extra_duration = models.IntegerField (
        verbose_name="Ore Straordinarie",
    )
    travel_duration = models.IntegerField (
        verbose_name="Ore Viaggio",
    )
    kilometres = models.IntegerField (
        verbose_name="Chilometri",
    )
    overnight_stay = models.BooleanField (
        verbose_name = "Pernottamento",
    )
    carrying = models.CharField (
        max_length = 15,
        choices = CARRYING_CHOICES,
        verbose_name = "Mezzo di Trasporto",
    )
    pay_exit = models.BooleanField (
        verbose_name = "Uscita",
    )

    class Admin:
        pass

##################################
# TOTALI                         #
##################################

##################################
# totale assistenza              #
##################################
class TotalAssistance (models.Model):
    id = models.AutoField('ID', primary_key=True)
    ## ForeignKey
    assistance = models.ForeignKey (
        Assistance,
        verbose_name = "Ordine",
    )
    work_duration = models.IntegerField (
        verbose_name="Ore Ordinarie",
    )
    extra_duration = models.IntegerField (
        verbose_name="Ore Straordinarie",
    )
    travel_duration = models.IntegerField (
        verbose_name="Ore Viaggio",
    )
    kilometres = models.IntegerField (
        verbose_name="Chilometri",
    )
    number_overnight = models.IntegerField (
        verbose_name="Totale Pernottamenti",
    )
    number_pay_exit = models.IntegerField (
        verbose_name="Totale Uscite",
    )

    class Admin:
        pass

    class Meta:
        db_table = "assistance_total_view"
