from django.db import models
from jungle.organization.models import Organization, OrganizationType

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

##############################
# PRODOTTO                   #
##############################
class Product (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    type = models.ForeignKey (
        "ProductType",
        verbose_name="Tipo Prodotto",
    )
    vat = models.ForeignKey (
        "Vat",
        verbose_name="Aliquota IVA",
    )
    # causaleMagazzino
    # foto
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        max_length=100,
        verbose_name="Prodotto",
    )
    description = models.TextField (
        blank=True,
        verbose_name="Descrizione",
    )
    min_supply = models.IntegerField (
        default=0,
        verbose_name="Scorta Minima",
    )
    serialnumber= models.BooleanField (
        verbose_name="Seriale",
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )

    class Admin:
        list_filter = ("status", "type", "vat",)
        search_fields = ("code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "type", "code", "name", "vat", "min_supply", "serialnumber",)
        list_per_page = 30
        fields = (
            (None, {
                "fields": (
                    ("status", "type",),
                    ("code", "name", "vat",),
                    ("description",),
                    ("min_supply", "serialnumber",),
                    ("note",),
                ),
            }),
        )
        ordering = ("name","type",)

    class Meta:
        verbose_name = "Prodotto"
        verbose_name_plural = "Prodotti"

    def __unicode__(self):
        return unicode(self.name)
    
##################################
# PRODOTTO TIPO                  #
##################################
class ProductType (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    #sector = models.ForeignKey (
        #"ProductSector",
        #verbose_name="Settore",
    #)
    #category = models.ForeignKey (
        #"ProductCategory",
        #verbose_name="Categoria",
    #)
    #group = models.ForeignKey (
        #"ProductGroup",
        #verbose_name="Gruppo",
    #)
    subgroup = models.ForeignKey (
        "ProductSubgroup",
        verbose_name="Sottogruppo",
    )
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Nome",
    )
    description = models.TextField (
        blank=True,
        verbose_name="Descrizione",
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status",)
        search_fields = ("code", "subgroup", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "code", "subgroup", "name",)
        list_display_links = ("code", "name",)
        list_per_page = 30
        fields = (
            (None, {
                "fields": (
                    ("status", "code", "name",),
                    ("description",),
                    ("subgroup",),
                    ("note",),
                    ("odv",),
                ),
            }),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Tipo Prodotto"
        verbose_name_plural = "Tipi Prodotto"

    def __unicode__(self):
       return unicode(self.subgroup.group.category.sector.name+" - "+self.subgroup.group.category.name+" - "+self.subgroup.group.name+" - "+self.subgroup.name+" - "+self.name)

###################################
# SETTORE                         #
###################################
class ProductSector (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Nome",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status",)
        search_fields = ("code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name",)
        list_per_page = 30
        fields = (
            (None, {
                "fields": (
                    ("status", "code", "name", "odv",),
                ),
            }),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Settore Prodotto"
        verbose_name_plural = "Settori Prodotto"

    def __unicode__(self):
        return unicode(self.name)
    
#####################################
# CATEGORIA                         #
#####################################
class ProductCategory (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    sector = models.ForeignKey (
        "ProductSector",
        verbose_name="Settore",
    )
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Nome",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status",)
        search_fields = ("sector", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "sector", "code", "name",)
        list_per_page = 30
        fields = (
            (None, {
                "fields": (
                    ("status", "code", "sector", "name", "odv",),
                ),
            }),
        )
        ordering = ("sector", "odv","name",)

    class Meta:
        verbose_name = "Categoria Prodotto"
        verbose_name_plural = "Categorie Prodotto"

    def __unicode__(self):
        return unicode(self.sector.name+" - "+self.name)
    
##################################
# GRUPPO                         #
##################################
class ProductGroup (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    category = models.ForeignKey (
        "ProductCategory",
        verbose_name="Categoria",
    )
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Nome",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status",)
        search_fields = ("category", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "category", "code", "name",)
        list_per_page = 30
        fields = (
            (None, {
                "fields": (
                    ("status", "code", "category", "name", "odv",),
                ),
            }),
        )
        ordering = ("category", "odv","name",)

    class Meta:
        verbose_name = "Gruppo Prodotto"
        verbose_name_plural = "Gruppi Prodotto"

    def __unicode__(self):
        return unicode(self.category.sector.name+" - "+self.category.name+" - "+self.name)

#####################################
# SOTTOGRUPPO                       #
#####################################
class ProductSubgroup (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    group = models.ForeignKey (
        "ProductGroup",
        verbose_name="Gruppo",
    )
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Nome",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status",)
        search_fields = ("group", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "group", "code", "name",)
        list_per_page = 30
        fields = (
            (None, {
                "fields": (
                    ("status", "code", "group", "name", "odv",),
                ),
            }),
        )
        ordering = ("group", "odv","name",)

    class Meta:
        verbose_name = "Sottogruppo Prodotto"
        verbose_name_plural = "Sottogruppi Prodotto"

    def __unicode__(self):
       return unicode(self.group.category.sector.name+" - "+self.group.category.name+" - "+self.group.name+" - "+self.name)


#####################################
# PRODOTTO SERIALE                  #
#####################################
class ProductSerialNumber (models.Model):
# si ricorda che: 
# - "Pratica" movimenta la "Disponibilita"
# - "DDT" movimenta la "Giacenza"
# - "Fattura" genera il "Credito" o il "Debito" ed e" utiizzata per la "Garanzia"
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    dateLastModify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    product = models.ForeignKey (
        "Product",
        verbose_name="Prodotto",
    )
    ## Campi
    serialnumber = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="SerialNumber",
    )

    class Admin:
        list_filter = ("status", "product",)
        search_fields = ("product", "serialnumber",)
        date_hierarchy = "date_create"
        list_display = ("status", "product", "serialnumber",)
        list_display_links = ("product",)
        list_per_page = 30
        fields = (
            (None, { "fields": (
                ("status", "product", "serialnumber",),
            ),}),
        )

    class Meta:
        verbose_name = "Seriale Prodotto"
        verbose_name_plural = "Seriali Prodotto"

    def __unicode__(self):
        return unicode(self.serialnumber)

#############################
# LISTINO                   #
#############################
class List (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    list_type = models.ForeignKey (
        "ListType",
        verbose_name="Tipo Listino",
    )
    product = models.ForeignKey (
        "Product",
        verbose_name="Prodotto",
        null=True,
        blank=True,
    )
    ## Campi
    date_begin = models.DateField (
        verbose_name="Data Inizio Validita",
    )
    date_end = models.DateField (
        verbose_name="Data Fine Validita",
    )
    price = models.DecimalField (
        max_digits=9,
        decimal_places=2,
        verbose_name="Prezzo",
        default=0.00,
    )
    price_tax_included = models.BooleanField (
        verbose_name="Prezzo IVA Inclusa",
    )
    discount1 = models.DecimalField (
        max_digits=4,
        decimal_places=2,
        verbose_name="Sconto",
        default=0.00,
    )
    discount2 = models.DecimalField (
        max_digits=4,
        decimal_places=2,
        verbose_name="Sconto",
        default=0.00,
    )
    note = models.TextField (
        blank=True,
        verbose_name="Note",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "list_type",)
        search_fields = ("product",)
        date_hierarchy = "date_create"
        list_display = ("status", "list_type", "product", "date_begin", "date_end", "price", "price_tax_included", "discount1", "discount2",)
        list_display_links = ("product",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                    ("status", "list_type", "product", ),
                    ("date_begin", "date_end",),
                    ("price", "price_tax_included", "discount1", "discount2",),
                    ("note",),
                    ("odv",),
                ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Listino"
        verbose_name_plural = "Listini"

#################################
# LISTINO TIPO                  #
#################################
class ListType (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    organization_type = models.ForeignKey (
        OrganizationType,
        verbose_name="Tipo Anagrafica",
        null=True,
        blank=True,
    )
    organization = models.ForeignKey (
        Organization,
        verbose_name="Anagrafica",
        null=True,
        blank=True,
    )
    product_type = models.ForeignKey (
        "ProductType",
        verbose_name="Tipo Prodotto",
        null=True,
        blank=True,
    )
    ## Campi
    date_begin = models.DateField (
        verbose_name="Data Inizio Validita",
    )
    date_end = models.DateField (
        verbose_name="Data Fine Validita",
    )
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Tipo Listino",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name", "organization_type", "organization", "product_type", "date_begin", "date_end",)
        list_display_links = ("code", "name",)
        list_per_page = 30

    class Meta:
        verbose_name = "Tipo Listino"
        verbose_name_plural = "Tipi Listino"

    def __unicode__(self):
        return unicode(self.code+" "+self.name)

#########################
# IVA                   #
#########################
class Vat (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Aliquota IVA",
    )
    description = models.TextField (
        unique=True,
        blank=True,
        verbose_name="Descrizione Aliquota IVA",
    )
    aliquot_perc = models.DecimalField (
        max_digits=4,
        decimal_places=2,
        verbose_name="Aliquota IVA",
        default=0.00,
    )
    deduct_perc = models.DecimalField (
        max_digits=4,
        decimal_places=2,
        verbose_name="Indetraibilita' IVA",
        default=0.00,
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name", "aliquot_perc", "deduct_perc",)
        list_display_links = ('name',)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                    ("status", "code", "name", ),
                    ("description",),
                    ("aliquot_perc", "deduct_perc",),
                    ("odv",),
                ),}),
        )
        ordering = ("odv","name",)

    class Meta:
        verbose_name = "Aliquota IVA"
        verbose_name_plural = "Aliquote IVA"

    def __unicode__(self):
        return unicode(self.name)
