from jungle.middleware.thread_local import get_current_user
from jungle.proxy.models import ACL, Category, ACL_Category, IPGroup
from jungle.proxy.models import PERMS
from django import newforms as forms
from django.newforms import ModelForm

class ACLForm(ModelForm):
    class Meta:
        model = ACL
    
    def __init__(self, *args, **kwargs):
        super(ACLForm, self).__init__(*args, **kwargs)
        
        for i, category in enumerate(Category.objects.all()):
            self.fields['category_%s' % category.name] = forms.ChoiceField(label=category.name, choices=PERMS, initial=2)
            self.fields['order_%s' % category.name] = forms.IntegerField(label="ORDINE", required=False,
                                                                      widget=forms.TextInput(attrs={'size':'2'}))
            

    def save(self):
        acl = super(ACLForm, self).save()
        order_default = 0
        
        for category in Category.objects.all():
            c = ACL_Category()
            c.category_id = category.id
            c.acl_id = acl.id
            c.perms = self.cleaned_data['category_%s' % category.name]

            if self.cleaned_data['category_%s' % category.name] != '0' and self.cleaned_data['order_%s' % category.name] is None:
                c.ordinamento = order_default
                order_default += 1
            else:
                c.ordinamento = self.cleaned_data['order_%s' % category.name]
            c.save()
        
        acl.crea_file()
        return acl
    
    
class ACLFormUpdate(ACLForm):
    def __init__(self, *args, **kwargs):
        self.acl = ACL.objects.get(id=args[0]['id'])
        kwargs['instance'] = self.acl
        
        super(ACLFormUpdate, self).__init__(*args, **kwargs)
        
        rules = ACL_Category.objects.filter(acl=self.acl)
        for rule in rules:
            if 'category_%s' % rule.category.name not in self.data:
                self.data.setlist('category_%s' % rule.category.name, [rule.perms])
                self.data.setlist('order_%s' % rule.category.name, [rule.ordinamento])
    
    def save(self):
        b = ACL_Category.objects.filter(acl=self.acl)
        b.delete()
        self.acl.crea_file()
        return super(ACLFormUpdate, self).save()


class IPGroupForm(ModelForm):
    class Meta:
        model = IPGroup
    
    def __init__(self, *args, **kwargs):
        super(IPGroupForm, self).__init__(*args, **kwargs)    