from django.db import models
from django.newforms import ModelForm
from jungle.functions.conf import *

##########################
# CATEGORY               #
##########################
STATUS_CODES = (
    (1, 'Attivo'),
    (0, 'Inattivo'),
)

TYPE_CAT = (
    ("w", "White List"),
    ("b", "Black List"),
)

PATH_CAT = get_from_local_settings('PATH_CAT', '/var/lib/squidguard/db')
PATH_LOG = get_from_local_settings('PATH_LOG', '/var/log/squid')
PATH_PROXY_CONF = get_from_local_settings('PATH_PROXY_CONF', '/etc/squid/squidGuard.conf')
REDIRECT = get_from_local_settings('URL_REDIRECT', 'http://127.0.0.1/squid_ad.html?caddr=%a&cname=%n&user=%i&group=%s&url=%u&target=%t')
DEFAULT_POLITIC = get_from_local_settings('DEFAULT_POLITIC', 'none')

class Category(models.Model):
    ## Campi generici
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "data creazione",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "data modifica",
    )
    status = models.IntegerField(                                
        max_length = 1,
        default = "1",
        choices = STATUS_CODES,
        verbose_name = "stato",
    )
    ## Campi modello
    name = models.CharField(
        max_length = 50,
        verbose_name = "categoria",
    )
    type = models.CharField(
        max_length = 1,
        choices = TYPE_CAT,
        verbose_name = "tipo",
    )
    domain = models.FilePathField(
        blank = True, null = True,
        path = PATH_CAT,
        verbose_name = "domain",
    )
    url = models.FilePathField(
        blank = True, null = True,
        path = PATH_CAT,
        verbose_name = "url",
    )
    expression = models.FilePathField(
        blank = True, null = True,
        path = PATH_CAT,
        verbose_name = "expression",
    )
    log = models.FilePathField(
        blank = True, null = True,
        path = PATH_LOG,
        verbose_name = "log",
    )
    redirect = models.URLField(
        blank = True, null = True,
        max_length = 100,
        verify_exists = False,
        verbose_name = "URL Redirect"
    )
    user_domain =  models.TextField(
        blank = True, null = True,
        verbose_name = "Lista Domains",
    )
    user_url =  models.TextField(
        blank = True, null = True,                                 
        verbose_name = "Lista Urls",
    )
    user_expression =  models.TextField(
        blank = True, null = True,                                        
        verbose_name = "Lista Expressions",
    )    
    
    editable = models.BooleanField(default=True, null=False)

    #def save(self):        
    #    super(Category, self).save()
    #    ACL().crea_file()
        
    class Admin:
        pass

    class Meta:
        verbose_name = "Categoria"
        verbose_name_plural = "Categorie"
        unique_together = (("name"), ("type"))

    def __unicode__(self):
        return u"%s" % self.name

##########################
# TIME GROUP             #
##########################

#class TimeGroup(models.Model):
#    ## Campi generici
#    date_create = models.DateTimeField(
#        auto_now_add = True,
#        verbose_name = "data creazione",
#    )
#    date_last_modify = models.DateTimeField(
#        auto_now = True,
#        verbose_name = "data modifica",
#    )
#    status = models.IntegerField(
#        max_length = 1,
#        default = "1",
#        choices = STATUS_CODES,
#        verbose_name = "stato",
#    )
#    ## Campi modello
#    name = models.CharField(
#        max_length = 50,
#        unique = True,
#        verbose_name = "gruppo time"
#    )
#
#    class Admin:
#        pass
#
#    class Meta:
#        verbose_name = "Gruppo Time"
#        verbose_name_plural = "Gruppi Time"
#
#    def __unicode__(self):
#        return u"%s" % self.name

##########################
# TIME RULE              #
##########################
TYPE_CAT = (
    ("w", "Settimanale"),
    ("t", "Data"),
)

class TimeRule(models.Model):
    ## Campi generici
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "data creazione",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "data modifica",
    )
    status = models.IntegerField(
        max_length = 1,
        default = "1",
        choices = STATUS_CODES,
        verbose_name = "stato",
    )
    ## Campi modello
    name = models.CharField(
        max_length = 50,
        verbose_name = "Descrizione",
    )
#    type = models.CharField(
#        max_length = 1,
#        choices = TYPE_CAT,
#        verbose_name = "tipo",
#    )
    rule = models.TextField(
        verbose_name = "regola",
    )

    def save(self):        
        super(TimeRule, self).save()
        ACL().crea_file()
        
    def get_clean_name(self):
        return self.name.strip().replace(" ", "_").lower()
    
    class Admin:
        pass

    class Meta:
        verbose_name = "Regola Tempo"
        verbose_name_plural = "Regole Tempo"
        unique_together = (("name"),)

    def __unicode__(self):
        return u"%s" % self.name

##########################
# IP GROUP               #
##########################

class IPGroup(models.Model):
    ## Campi generici
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "data creazione",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "data modifica",
    )
    status = models.IntegerField(
        max_length = 1,
        default = "1",
        choices = STATUS_CODES,
        verbose_name = "stato",
    )    
    ## Campi modello
    name = models.CharField(
        max_length = 50,
        unique = True,
        verbose_name = "gruppo",
    )
    
    ip_list = models.TextField(
        verbose_name = "Lista Ip",
    )

    odv = models.IntegerField(
        blank = True, null = True,                              
        default=0,
        verbose_name = "odv",
    )

    def save(self):        
        super(IPGroup, self).save()
        ACL().crea_file()
            
    def get_clean_name(self):
        return self.name.strip().replace(" ", "_").lower()
        
    class Admin:
        pass

    class Meta:
        verbose_name = "Gruppo IP"
        verbose_name_plural = "Gruppi IP"

    def __unicode__(self):
        return u"%s" % self.name

       
##########################
# IP LIST                #
##########################

#class IPList(models.Model):
#    ## Campi generici
#    date_create = models.DateTimeField(
#        auto_now_add = True,
#        verbose_name = "data creazione",
#    )
#    date_last_modify = models.DateTimeField(
#        auto_now = True,
#        verbose_name = "data modifica",
#    )
#    status = models.IntegerField(
#        max_length = 1,
#        default = "1",
#        choices = STATUS_CODES,
#        verbose_name = "stato",
#    )
#    ## Foreign key
#    ip_group = models.ForeignKey(
#        IPGroup,
#        verbose_name = "Gruppo IP"
#    )
#    ## Campi modello
#    ip_start = models.IPAddressField(
#        unique = True,
#        verbose_name = "IP inizio",
#    )
#    ip_end = models.IPAddressField(
#        blank = True, null = True,
#        unique = True,
#        verbose_name = "IP fine",
#    )
#
#    class Admin:
#        pass
#
#    class Meta:
#        verbose_name = "Lista IP"
#        verbose_name_plural = "Lista IP"
#
#    def __unicode__(self):
#        return u"%s - %s" % (self.ip_start, self.ip_end)

##########################
# ACL                    #
##########################
DEFAULT_POLICY = (
    ("all", "Accesso Consentito"),
    ("none", "Accesso Negato"),
)
class ACL(models.Model):
    ## Campi generici
    date_create = models.DateTimeField(
        auto_now_add = True,
        verbose_name = "data creazione",
    )
    date_last_modify = models.DateTimeField(
        auto_now = True,
        verbose_name = "data modifica",
    )
    status = models.IntegerField(
        max_length = 1,
        default = "1",
        choices = STATUS_CODES,
        verbose_name = "stato",
    )
    ## Foreign key
    ip_group = models.ForeignKey(
        IPGroup,
        verbose_name = "Gruppo IP",
    )
    time_rule = models.ForeignKey(
        TimeRule,
        verbose_name = "Gruppo Tempo",
    )

    default = models.CharField(
        max_length = 4,
        default = "all",
        choices = DEFAULT_POLICY,
        verbose_name = "politica default",
    )

    odv = models.IntegerField(
        blank = True, null = True,                              
        default=0,
        verbose_name = "odv",
    )

    def delete(self):        
        super(ACL, self).delete()
        self.crea_file()

                    
    def crea_file(self):
        f = open(PATH_PROXY_CONF, 'w')
        f.write("logdir %s\n" % PATH_LOG)
        f.write("dbhome %s\n\n" % PATH_CAT)
        
        # TIME RULES        
        for t in TimeRule.objects.filter(status=1):
            rules = t.rule.replace("\r\n", "\n")
            f.write("time %s {\n" % t.get_clean_name())
            #f.write("     %s\n" % t.rule.replace("\n", "\n     "))
            f.write("     %s\n" % rules.replace("\n", "\n     "))
            f.write("}\n\n")
        
        
        # GROUP RULES
        for g in IPGroup.objects.filter(status=1).order_by('odv'):
            iplists = g.ip_list.replace("\r\n", "\n")
            iplists = iplists.split('\n')
            f.write("src %s {\n" % g.get_clean_name())
            for ip in iplists:
                if ip:
                    f.write("     ip   %s\n" % ip.replace("\n", "\n     "))
            f.write("}\n\n")
        
        # CATEGORY
        for category in Category.objects.all():
            f.write("dest %s {\n" % category.name)
            f.write("     domainlist %s\n" % category.domain)
            if category.url:
                f.write("     urllist    %s\n" % category.url)
            f.write("}\n\n")
            
        # ACL
        f.write("acl {\n")
        for acl in ACL.objects.filter(status=1, ip_group__status=1, time_rule__status=1).order_by('odv'):        
            acl_cat = ACL_Category.objects.filter(acl=acl.id, ordinamento__isnull=False).order_by('ordinamento')            
                        
            str_perm = ""
            f.write("     %s within %s {\n" % (acl.ip_group.get_clean_name(), acl.time_rule.get_clean_name()))                
            
            for cat in ACL_Category.objects.filter(acl=acl.id, ordinamento__isnull=False).order_by('ordinamento'):
                
                if cat.perms == 1: str_perm += "%s " % cat.category.name
                if cat.perms == 2: str_perm += "!%s " % cat.category.name
                
            str_perm += "%s" % acl.default
            f.write("          pass %s\n" % str_perm)
            f.write("          redirect %s\n" % REDIRECT)
            f.write("     }\n\n")

        # Politica di default
        f.write("    default {\n")
        f.write("          pass %s\n" % DEFAULT_POLITIC)
        f.write("          redirect %s\n" % REDIRECT)
        f.write("     }\n\n")            
        
        f.write("}")
        f.close()
        
    def get_string_perms(self):
        acl_cat = ACL_Category.objects.filter(acl=self.id).order_by('ordinamento')
        n = ""
        for c in acl_cat:
            if c.perms == 0:
                continue
            
            if c.perms == 1:
                n += "<span style='color:green'>%s</span> " % c.category.name
            
            if c.perms == 2:
                n += "<span style='color:red'>!%s</span> " % c.category.name
            
        return n
    
    class Admin:
        pass

    class Meta:
        verbose_name = "ACL"
        verbose_name_plural = "ACL"
        unique_together = (("ip_group"), ("time_rule"))

    def __unicode__(self):
        return u"%s %s" % (self.ip_group, self.time_rule)

PERMS = (
        (0, 'NON GESTITA'),
        (1, 'CONSENTI'),
        (2, 'NEGA')
        )
    
class ACL_Category(models.Model):
    acl = models.ForeignKey(ACL)
    category = models.ForeignKey(Category)
    perms = models.IntegerField(
        max_length = 1,
        choices = PERMS
    )
    ordinamento = models.IntegerField(
        blank = False, null = True,
        max_length = 2
    )

    class Meta:
        verbose_name = "Categoria ACL"
        verbose_name_plural = "Categorie ACL"
        
    def __unicode__(self):
        return u"%s %s" % (self.category.name, self.get_perms_display())

###############################
# ACCESS LOG                  #
###############################
class Access_Log(models.Model):
    timestamp = models.DateTimeField(
        verbose_name = "data",
    )
    elapsed = models.PositiveIntegerField(
        verbose_name = "durata millisecondi",
    )
    client = models.IPAddressField(
        verbose_name = "IP Client",
    )
    action = models.CharField(
        max_length = 50,
        verbose_name = "risposta",
    )
    code = models.PositiveIntegerField(
        verbose_name = "codice risposta",
    )
    size = models.PositiveIntegerField(
        verbose_name = "dimensione in byte",
    )
    method = models.CharField(
        max_length = 20,
        verbose_name = "metodo richiesta",
    )
    domain_dest = models.CharField(
        max_length = 255,
        verbose_name = "dominio richiesto",
    )
    ident = models.CharField(
        max_length = 30,
    )
    hierarchy = models.CharField(
        max_length = 255,
    )
    content = models.CharField(
        max_length = 255,
    )
    
