# -*- coding: utf-8 -*-

from jungle.views.decorators import login_required
from jungle.views import mgeneric as generic
from jungle.proxy.models import Category, TimeRule, IPGroup
from jungle.proxy.models import ACL, ACL_Category, Access_Log
from jungle.proxy.models import PATH_CAT, PATH_LOG
from django.utils import simplejson
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.conf import settings
from datetime import timedelta
#from jungle.functions import log
from jungle.functions.conf import *
import time, commands, os

SQUID_CONF_FILE = get_from_local_settings('SQUID_CONF_FILE', '/etc/squid/squid.conf')

@login_required
def add(request, model=None, **kw):
    if model == 'category':
        from category_form import CategoryForm        
        kw['post_save_redirect'] = '%sproxy/category/' % (settings.LOCATION,)
        return generic.create_object(request, model, form=CategoryForm, **kw)
                                     
    if model == 'timerule':
        kw['post_save_redirect'] = '%sproxy/timerule/' % (settings.LOCATION,)

    if model == 'ipgroup':
        from acl_form import IPGroupForm
        kw['post_save_redirect'] = '%sproxy/ipgroup/' % (settings.LOCATION,)
        return generic.create_object(request, model, form=IPGroupForm, **kw)        

    if model == 'acl':
        from acl_form import ACLForm
        kw['post_save_redirect'] = '%sproxy/acl/' % (settings.LOCATION,)
        return generic.create_object(request, model, form=ACLForm, **kw)

    return generic.create_object(request, model, **kw)

    
@login_required
def update(request, model=None, **kw):
    if model == 'category':
        from category_form import CategoryFormUpdate        
        kw['post_save_redirect'] = '%sproxy/category/' % (settings.LOCATION,)
        return generic.update_object(request, model, form=CategoryFormUpdate, **kw)
                                    
    if model == 'timerule':
        kw['post_save_redirect'] = '%sproxy/timerule/' % (settings.LOCATION,)

    if model == 'ipgroup':
        kw['post_save_redirect'] = '%sproxy/ipgroup/' % (settings.LOCATION,)

    if model == 'acl':
        from acl_form import ACLFormUpdate
        kw['post_save_redirect'] = '%sproxy/acl/' % (settings.LOCATION,)
        return generic.update_object(request, model, form=ACLFormUpdate, **kw)

    return generic.update_object(request, model, ext="_update", **kw)
    
@login_required
def object_list(request, model, filter_cnd={}, order_by='-date_create',
                search_params={}, form=None, **kw):
    
    if model == 'category':
        queryset = Category.objects.none()
    if model == 'timerule':
        queryset = TimeRule.objects.none()
    if model == 'ipgroup':
        queryset = IPGroup.objects.none()
    if model == 'acl':
        queryset = ACL.objects.none()
    if model == 'access_log':
        queryset = Access_Log.objects.none()
        

    return generic.jobject_list(request, queryset, model, form,
                                filter_cnd, order_by, **kw)

@login_required
def object_detail(request, model, object_id):
    return generic.object_detail(request, model, object_id)

def get_status_squid():
    stato="ACCESO"
    status, output = commands.getstatusoutput("sudo squid -k check")
    if output.find("ERROR") > -1:
        stato = 'SPENTO'
    
    return stato

def get_status_msn():
    stato="ABILITATO"
    f = open(SQUID_CONF_FILE, 'r')
    r = f.readlines()
    f.close()

    for l in r:
        if l.find('http_access deny msn_req') >= 0:
            if l[0] != '#':
                stato="DISABILITATO"
                break
    return stato

def onoff_squid(request):
    stato = get_status_squid()
    if stato == "ACCESO":
        status, output = commands.getstatusoutput("sudo squid -k kill")
    else:
        status, output = commands.getstatusoutput("sudo /etc/init.d/squid restart")
        time.sleep(2)
    return HttpResponse("OK")

def onoff_msn(request):
    stato = get_status_msn()
    f = open(SQUID_CONF_FILE, 'r')
    r = f.readlines()
    f.close()
    l_file = []
    for l in r:
        if stato == "ABILITATO":
            l = l.replace("#http_access deny msn_req","http_access deny msn_req")
            l = l.replace("#http_reply_access deny","http_reply_access deny")
        else:
            l = l.replace("http_access deny msn_req","#http_access deny msn_req")
            l = l.replace("http_reply_access deny","#http_reply_access deny")
        l_file.append(l)

    f = open(SQUID_CONF_FILE, 'w')
    f.writelines(l_file)
    f.close()
    restart_squid(request)

    return HttpResponse("OK")

def restart_squid(request):
    status, output = commands.getstatusoutput("sudo squid -k kill")
    return function_squid(request, 'sudo /etc/init.d/squid restart')


@login_required
def function_squid(request, command):
    last_line = "Contattare l'amministratore"
    #ret_code, output, error = execute(command)    
    status, output = commands.getstatusoutput(command)
    time.sleep(1)
    if output.find("failed") < 0:
      log_path = get_from_local_settings('PATH_LOG', '/var/log/squid/')
      log_file = os.path.join(log_path, 'squidGuard.log')
      f = open(log_file, 'r')
      st_results = os.stat(log_file) 
      st_size = st_results[6]
      f.seek(st_size - 100)      
      last_line = f.readlines()[-1]
      f.close()
      if last_line.find("ready for request") > -1:
        return HttpResponse("OK")
        
    return HttpResponse("ERRORE: %s." % last_line)

@login_required
def change_status(request, model):
    id = request.POST['keyID']
    status = request.POST['value']
    acl = ACL.objects.get(pk=id)
    acl.status = status
    acl.crea_file()
    acl.save()
    return HttpResponse("OK")

@login_required
def get_search_data(request, model):
    user = request.user
    get =  request.GET.copy()

    ### CERTIFICATE ###
    if model == 'category':
        model_s = CategorySearch()
    if model == 'timerule':
        model_s = TimeRuleSearch()
    if model == 'ipgroup':
        model_s = IPGroupSearch()
    if model == 'acl':
        model_s = ACLSearch()

    start = get.get('start', 0)
    limit = get.get('limit', 25)
    groupby = get.get('groupBy', '')
    orderby = get.get('sort', model_s.default_sort)
    direction = get.get('dir', 'ASC')
        
    queryset = model_s.get_query_set()

    # Gestione della ricerca
    #
    from jungle.views.fast_search import search2
    filter_cnd, oldorder_by, form = search2(request, model)
    
    if filter_cnd:
        queryset = queryset.filter(**filter_cnd)

            
    total = queryset.count()
    queryset = model_s.order_by(queryset, groupby, orderby, direction)
    queryset = queryset[int(start):int(start)+int(limit)]

    dati = model_s.get_dati(queryset)
        
    data = '{"success": true, "total": %s, "data": %s}' % (total, simplejson.dumps(dati, ensure_ascii=False))
    return HttpResponse(data, mimetype='text/javascript;')


class CategorySearch:
    default_sort = 'name'
    
    def get_query_set(self):
        queryset = Category.objects.all()
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by('type', orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['name'] = x.name
            d['status'] = x.get_status_display()
            d['type'] = x.get_type_display()                
            d['redirect'] = x.redirect
            d['editable'] = x.editable
            dati.append(d)
        return dati        
        

class TimeRuleSearch:
    default_sort = 'name'
    
    def get_query_set(self):
        return TimeRule.objects.all()

    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by(orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['name'] = x.name
            d['status'] = x.get_status_display()
            d['rule_list'] = x.rule
            dati.append(d)    
        return dati

class IPGroupSearch:
    default_sort = 'odv'
    
    def get_query_set(self):
        return IPGroup.objects.all()

    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by(orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['name'] = x.name
            d['status'] = x.get_status_display()
            d['ip_list'] = x.ip_list
            d['odv'] = x.odv
            dati.append(d)    
        return dati


class ACLSearch:
    default_sort = 'odv'
    
    def get_query_set(self):
        return ACL.objects.all()

    def order_by(self, queryset, groupby, orderby, dir):            
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        return queryset.order_by(orderby)
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['odv'] = x.odv
            d['status'] = x.get_status_display()
            d['ip_group'] = x.ip_group.name
            d['time_group'] = x.time_rule.name
            d['category_list'] = x.get_string_perms()
            d['default'] = x.get_default_display()
            dati.append(d)    
        return dati    
