from django.db import models
from jungle.address.models import Address

# Create your models here.

STATUS_CODES = (
    ( 1, "Active"),
    ( 0, "Inactive")
)

########################################
# GIORNO SETTIMANA                     #
########################################
## ATTENZIONE!!! inserire anche lingua di riferimento
class WeekDay (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        null=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        null=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Sigla Giorno",
    )
    name  = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Giorno della Settimana",
    )
    weekday = models.PositiveSmallIntegerField (
        unique=True,
        max_length=1,
        null=True,
        verbose_name="Number of Week",
    )
    isoweekday = models.IntegerField (
        default=0,
        unique=True,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "^code", "name",)
        date_hierarchy = "date_create"
        list_per_page = 20
        list_display = ("status", "code", "name",)
        list_display_links = ("name",)
        fields = (
            (None, {"fields":(
                ("status", "code", "name", "weekday", "isoweekday",),
            ),}),
        )
        ordering = ("isoweekday",)

    class Meta:
        unique_together = (
            ("code",),
            ("name",),
            ("weekday",),
        )
        verbose_name = "Giorno della Settimana"
        verbose_name_plural = "Giorni della Settimana"

    def __unicode__(self):
        return unicode(self.code)
    
############################
# MESE                     #
############################
## ATTENZIONE!!! inserire anche lingua di riferimento
class Month (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Sigla Mese",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Mese",
    )
    num_month = models.PositiveSmallIntegerField (
        unique=True,
        max_length=2,
        null=True,
        verbose_name="Numero Mese",
    )

    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("status", "^code", "name",)
        date_hierarchy = "date_create"
        list_per_page = 30
        list_display = ("status", "code", "name",)
        list_display_links = ("name",)
        fields = (
            (None, {"fields":(
                ("status", "code", "name", "num_month", "odv",),
            ),}),
        )
        ordering = ("odv", "name",)

    class Meta:
        verbose_name = "Mese"
        verbose_name_plural = "Mesi"

    def __unicode__(self):
        return unicode(self.name)
    
################################
# NAZIONE                      #
################################
## ATTENZIONE!!! inserire anche lingua di riferimento
class Nation (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        primary_key=True,
        verbose_name="Sigla Nazione",
    )
    name  = models.CharField (
        max_length=100,
        verbose_name="Nazione",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        search_fields = ("^code", "name",)
        date_hierarchy = "date_create"
        list_per_page = 20
        list_display = ("status", "code", "name",)
        list_display_links = ("name",)
        fields = (
            (None, {"fields":(
                ("status", "code", "name", "odv",),
            ),}),
        )
        ordering = ("odv", "code","name",)

    class Meta:
        verbose_name = "Nazione"
        verbose_name_plural = "Nazioni"

    def __unicode__(self):
        return unicode(self.code+" "+self.name)
    
###############################
# REGIONE                     #
###############################
## ATTENZIONE!!! inserire anche lingua di riferimento
class Region (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ref_nation = models.ForeignKey (
        "Nation",
        verbose_name="Nazione",
        to_field="code",
        db_column="ref_nation_code",
   )
    ## Campi
    code = models.CharField (
        primary_key=True,
        max_length=15,
        verbose_name="Sigla Regione",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Regione",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "ref_nation",)
        search_fields = ("^ref_nation", "^code", "name",)
        date_hierarchy = "date_create"
        list_per_page = 30
        list_display = ("status", "ref_nation", "code", "name",)
        list_display_links = ("name",)
        fields = (
            (None, {"fields":(
                ("status", "ref_nation", "code", "name", "odv",),
            ),}),
        )
        ordering = ("odv", "ref_nation", "name",)

    class Meta:
        verbose_name = "Regione"
        verbose_name_plural = "Regioni"

    def __unicode__(self):
        return unicode(self.name)
    
##################################
# PROVINCIA                      #
##################################
## ATTENZIONE!!! inserire anche lingua di riferimento
class Province (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ref_region = models.ForeignKey (
        "Region",
        verbose_name="Regione",
        to_field="code",
        db_column="ref_region_code",
    )
    ## Campi
    code = models.CharField (
        max_length=15,
        primary_key=True,
        verbose_name="Sigla",
    )
    name = models.CharField (
        max_length=100,
        verbose_name="Provincia",
    )
    odv = models.IntegerField (
        default=0,
        verbose_name="OdV",
    )

    class Admin:
        list_filter = ("status", "ref_region",)
        search_fields = ("^code", "name",)
        date_hierarchy = "date_create"
        list_per_page = 20
        list_display = ("status", "ref_region", "code", "name",)
        list_display_links = ("name",)
        fields = (
            (None, {"fields":(
                ("status", "ref_region", "code", "name", "odv",),
            ),}),
        )
        ordering = ("odv", "ref_region", "name",)

    class Meta:
        verbose_name = "Provincia"
        verbose_name_plural = "Province"

##################################
# FESTIVITA                      #
##################################
## ATTENZIONE!!! inserire anche lingua di riferimento
class Holiday (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ref_nation = models.ForeignKey (
        "Nation",
        verbose_name="Nazione",
        to_field="code",
        db_column="ref_nation_code",
    )
    ## Campi
    day = models.IntegerField (
        max_length=2,
        verbose_name="GG",
    )
    month = models.IntegerField (
        max_length=2,
        verbose_name="MM",
    )
    year = models.IntegerField (
        max_length=4,
        verbose_name="AAAA",
    )
    name  = models.CharField (
        max_length=100,
        verbose_name="Festivita'",
    )

    class Admin:
        list_filter = ("status", "ref_nation",)
        search_fields = ("^ref_nation", "^name",)
        date_hierarchy = "date_create"
        list_per_page = 30
        list_display = ("status", "day", "month", "year", "name",)
        list_display_links = ("name",)
        fields = (
            (None, {"fields": (
                ("status", "ref_nation",),
                ("day", "month", "year",),
                ("name",),
            ),}),
        )
        ordering = ("name", "day", "month",)

    class Meta:
        verbose_name = "Festa"
        verbose_name_plural = "Feste"

    def __unicode__(self):
        return unicode(self.name)
    
##################################
# BANCA ABI                      #
##################################
class BankABI (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## Campi
    code_ABI = models.CharField (
        max_length=5,
        primary_key=True,
    )
    name_bank = models.CharField (
        max_length=80,
        verbose_name="Banca",
        unique=True,
    )

##################################
# BANCA                          #
##################################
class BankCAB (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default="1",
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    code_ABI = models.ForeignKey (
        BankABI,
        verbose_name="ABI",
    )
    ## Campi
    code_CAB = models.CharField (
        max_length=5,
        verbose_name="CAB"
    )
    name_office = models.CharField (
        max_length=40,
        verbose_name="Agenzia" ## sportello
    )
    office_number = models.IntegerField (
        verbose_name="Numero Agenzia"
    )
    #address = models.ManyToManyField(
        #Address,
        #null=True, blank=True,
        #verbose_name="Indirizzo",
        #raw_id_admin=True,
    #)
    phone = models.CharField (
        null=True,
        blank=True,
        max_length=20,
        verbose_name="Telefono",
    )
    phone2 = models.CharField (
        null=True,
        blank=True,
        max_length=20,
        verbose_name="Telefono",
    )
    fax = models.CharField (
        null=True,
        blank=True,
        max_length=20,
        verbose_name="Fax",
    )

    class Admin:
        pass

    class Meta:
        verbose_name = "Banca"
        verbose_name_plural = "Banche"
        unique_together = (("code_ABI", "code_CAB"),)

    def __unicode__(self):
        return unicode(self.code_ABI+" "+self.code_CAB+" "+self.name+" "+self.office_number)
