import django.newforms as forms
from django.core import validators 
from django.contrib.auth.models import User 
from jungle.registration.models import UserProfile
import datetime, random, sha 
from django.core.mail import send_mail
from jungle.template import mako_loader as loader
from pdb import set_trace
from django.conf import settings
import re
from jungle.templatetags.jungle_tags import ugettext as _

class RegistrationForm(forms.Form):
    username = forms.CharField(max_length=30)
    first_name = forms.CharField(max_length=30)
    last_name = forms.CharField(max_length=30)
    email    = forms.CharField(max_length=75)
    password1 = forms.CharField(max_length=60, widget=forms.PasswordInput)
    password2 = forms.CharField(max_length=60, widget=forms.PasswordInput)

    def clean_username(self): 
        #set_trace()
        msg = []

        if re.search('[^a-z0-9]', self.cleaned_data['username']):
            msg += [_('Please, use only lower letteres, digits and underscores')]

        try: 
            User.objects.get(username=self.cleaned_data['username'].lower()) 
            msg += [_('The username "%(username)s" is already taken.' % self.cleaned_data)]
        except User.DoesNotExist: 
            pass

        if msg:
            raise forms.ValidationError(msg)

        return self.cleaned_data['username']

    
     
    def clean(self):
        #set_trace()
        msg = []
        if self.cleaned_data['password1'] != self.cleaned_data['password2']:
            msg += [_('Passwords must match.')]

        if msg:
            raise forms.ValidationError("<br>".join(msg))
               
        return self.cleaned_data
    
    def save(self, request):
        #set_trace()
        new_user = User.objects.create_user(
            username = self.cleaned_data['username'].lower(), 
            email = self.cleaned_data['email'].lower(), 
            password = self.cleaned_data['password1'])
        new_user.is_active = False
        new_user.first_name = self.cleaned_data['first_name']
        new_user.last_name = self.cleaned_data['last_name']
        new_user.save() 

        # Build the activation key for their account                                                                                                                     
        salt = sha.new(str(random.random())).hexdigest()[:5] 
        activation_key = sha.new(salt+new_user.username).hexdigest() 
        key_expires = datetime.datetime.today() + datetime.timedelta(2) 
         
        # Create and save their profile                                                                                                                                  
        new_profile = UserProfile(user=new_user, 
                                  activation_key=activation_key, 
                                  key_expires=key_expires) 
        new_profile.save() 
         
        # Send an email with the confirmation link                                                                                                                       
        email_subject = getattr(settings, 'REGISTRATION_WELLCOME', u'Wellcome')
        t = loader.get_template('mail_login_acknoledge.txt')
        SERVER_PORT = request.META.get('SERVER_PORT')
        if request.META.get('SERVER_PORT') in (0, '0'):
            SERVER_PORT = 80
            

        context = { 'new_user'    : new_user.username, 
                    'link'        : "%s:%s/registration/confirm/%s/" % (
                                     request.META['SERVER_NAME'],
                                     SERVER_PORT,
                                     new_profile.activation_key,               
                                     ),
                    }
        email_body = t.render([context])
        new_user.email_user(email_subject, email_body)

             


        return new_user
