"""
This module needs the following templates to be defined:
registration/mail_login_acknoledge.txt:  e-mail content before confirmation
registration/register.html:              html page with form for registration
registration/confirm.html:               html confirmation page 


The latter may have in the context 3 boolean:
   * has_accont: for users that already have an account
   * expired:    too long time passed since the first time they required an account
   * succeed:    ok, now they got a login

To correctly activate UserProfile you need to add in settings.py:
AUTH_PROFILE_MODULE = app_name.UserProfile
"""

from models import UserProfile 
from registration_form import RegistrationForm 
from django import newforms as forms
from django.shortcuts import get_object_or_404  
from django.core.mail import send_mail
from django.template import RequestContext
from jungle.template.mako_loader import render_to_response
import datetime

def register(request): 
    #import pdb; pdb.set_trace()
    if request.user.is_authenticated(): 
        # They already have an account; don't let them register again 
        return render_to_response('registration/register.mako',
                {'has_account': True},
                context_instance=RequestContext(request),) 

    Form = RegistrationForm
    if request.POST: 
        form = Form(request.POST.copy())

        if form.is_valid(): 
            try:
                new_user = form.save(request)
            except:
                #import inspect
                #filename = inspect.getsourcefile(form.save)
                #code, line =  inspect.getsourcelines(form.save)
                
                #print "file: %s:%s\n%s" % (filename, line, "".join(code))
                raise
             
            return render_to_response('registration/register.mako',
                    {'created': True},
                context_instance=RequestContext(request),) 
    else: 
        form = Form()

    return render_to_response('registration/register.mako',
            {'form': form},
        context_instance=RequestContext(request),) 


def confirm(request, activation_key): 
    if request.user.is_authenticated(): 
        return render_to_response('registration/confirm.mako',
                {'has_account': True},
                context_instance=RequestContext(request),) 

    user_profile = get_object_or_404(UserProfile, 
                                     activation_key=activation_key) 
    if user_profile.key_expires < datetime.datetime.today(): 
        return render_to_response('registration/confirm.mako',
                {'expired': True},
                context_instance=RequestContext(request),) 
    user_account = user_profile.user 
    user_account.is_active = True 
    user_account.save() 
    return render_to_response('registration/confirm.mako',
            {'success': True},
        context_instance=RequestContext(request),) 
