from django.db import models
from jungle.address.models import Address, AddressScope, AddressType
from jungle.product.models import Product, ProductType

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

##########################
# MAGAZZINO              #
##########################
class Store (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
     ## ForeignKey
    address = models.ManyToManyField(
        Address,
        blank=True,
        null=True,
        verbose_name="Indirizzo",
        raw_id_admin=True,
    )
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Magazzino",
    )
    description = models.TextField (
        blank=True,
        verbose_name="Descrizione",
    )

    class Admin:
        search_fields = ("status", "code", "^name", "address", )
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name",)
        list_display_links = ("code", "name", )
        list_per_page = 30
        ordering = ("name","-code",)
        fields = (
            (None, {"fields": (
                ("status", "code", "name",),
                ("address",),
                ("description"),
            ),}),
        )

    class Meta:
        verbose_name = "Magazzino"
        verbose_name_plural = "Magazzini"

    def __unicode__(self):
        return unicode(self.name)
    
################################
#  BOLLETTARIO MAGAZZINO       #
################################
class StoreDocument (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
     ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Bollettario",
    )
    description = models.TextField (
        blank=True,
        verbose_name="Descrizione",
    )

    class Admin:
        search_fields = ("status", "code", "^name", "address", )
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name",)
        list_display_links = ("code", "name", )
        list_per_page = 30
        ordering = ("name","-code",)
        fields = (
            (None, {"fields": (
                ("status", "code", "name",),
                ("description"),
            ),}),
        )

    class Meta:
        verbose_name = "Bollettario Magazzino"
        verbose_name_plural = "Bollettari Magazzino"

    def __unicode__(self):
        return unicode(self.name)

################################
# CAUSALE MAGAZZINO            #
################################
MOVE_TYPE = (
    ("Carico", "Carico"),
    ("Scarico", "Scarico",),
)
STOCK_BUSY_TYPE = (
    ("+", "+",),
    ("-", "-",),
)
class StoreCause (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    store = models.ForeignKey (
        Store,
        verbose_name="Magazzino",
    )
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=100,
        verbose_name="Causale",
    )
    description = models.TextField (
        blank=True,
        verbose_name="Descrizione",
    )
    move = models.CharField (
        blank=True,
        max_length=10,
        choices=MOVE_TYPE,
        verbose_name="Movimentazione",
    )
    ddt = models.BooleanField (
        verbose_name="Segue DDT",
    )
    invoice = models.BooleanField (
        verbose_name="Segue Fattura",
    )
    stock = models.CharField (
        blank=True,
        max_length=1,
        choices=STOCK_BUSY_TYPE,
        verbose_name="Giacenza",
    )
    busy = models.CharField (
        blank=True,
        max_length=1,
        choices=STOCK_BUSY_TYPE,
        verbose_name="Disponibilita'",
    )

    class Admin:
        pass

    class Meta:
        verbose_name = "Causale Magazzino"
        verbose_name_plural = "Causali Magazzino"

    def __unicode__(self):
        return unicode(self.name)

##############################
#  MOVIMENTO MAGAZZINO       #
##############################
class StoreMovement (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    store = models.ForeignKey (
        Store,
        verbose_name="Magazzino",
    )
    ## Campi
    date_last = models.DateField (
        verbose_name="Ultima Data",
    )
    product_type = models.ForeignKey (
        ProductType,
        verbose_name = "Tipo Prodotto",
    )
    product = models.ForeignKey (
        Product,
        verbose_name = "Prodotto",
    )
    stock = models.CharField (
        blank=True,
        max_length=1,
        choices=STOCK_BUSY_TYPE,
        verbose_name="Giacenza",
    )
    busy = models.CharField (
        blank=True,
        max_length=1,
        choices=STOCK_BUSY_TYPE,
        verbose_name="Disponibilita'",
    )

    class Admin:
        list_display = ("store", "date_last", "product_type", "product", "stock", "busy",)
        list_display_links = ("product",)

    class Meta:
        verbose_name = "Movimento Magazzino"
        verbose_name_plural = "Movimenti Magazzino"

