from jungle.middleware.thread_local import get_current_user, _thread_locals
from django.core.exceptions import ObjectDoesNotExist
from jungle.functions.tools import send_mail_template, Item
#from django.newforms.util import smart_unicode
from jungle.templatetags.jungle_tags import ugettext as _
#from django.conf import settings

EMAIL_FROM = u'%s <%s>'

def evaluate_and_save(sender, instance, signal, *args,
    **kwargs):
    from jungle.ticket.models import Ticket
    if instance._get_pk_val():
        old_ticket = instance.__class__.objects.get(
                                        pk=instance._get_pk_val())
        old = Ticket(**old_ticket.__dict__)
        old.assigned_to_id = old_ticket.assigned_to_id # foreign key doesn't get set!
        
        _thread_locals.mode = 'UPDATE'
        _thread_locals.old = old
    else:
        _thread_locals.mode = 'INSERT'

def get_manager_list(instance):
    # Get a list of the email addresses the message
    # should be sent to.

    recipient_list = [man.email for man in instance.project.manager.all()]
    return recipient_list

def mail_on_change(sender, instance, signal, *args,    **kwargs):
    """
    Send an email out to a number of people informing them
    of a new Ticket.
    """
    mode = _thread_locals.mode
    send_the_mail = False

    if mode == 'INSERT':
        subject  = _(u"New Ticket #%(id)s '%(title)s' for project %(project)s") % (
            Item(instance))
        template = 'ticket/ticket_opened_email.mako'
        context  = {'ticket': instance,}
        mail_for_managers(instance, template, context, subject)

        ## send to the assegnee as well
        if instance.assigned_to:
            mail_for_operator(instance)

        return

    elif mode == 'UPDATE':
        template = 'ticket/ticket_changed_email.mako'
        old = _thread_locals.old
        context  = {'ticket': instance, 'old' : old}

        if instance.assigned_to != old.assigned_to:
            send_the_mail = True
            subject = "Changed assegnee for ticket #%(id)s" % Item(instance)
            if instance.assigned_to:
                ## tell the assegnee
                mail_for_operator(instance, title="operator")

        if old.status != instance.status:
            mail_for_submitter(instance, old, )

        if old.status != instance.status or old.priority != instance.priority:

            send_the_mail = True
            if instance.status == 0 and old.status != 0:
                subject = _(u"Closed ticket #%(id)s %(title)s ") % (
                    Item(instance))
            else:
                subject = _("Priority changed in ticket #%(id)s %(title)s") % (
                    Item(instance))

        ## some update (eg: description) do not need notification
        if send_the_mail == True:
            mail_for_managers(instance, template, context, subject)

def mail_for_managers(instance, template, context, subject, title='responsabile'):
    context.update({'title':title})

    kw = {
        'template'        : template,
        'context'        : context,
        'subject'        : subject,
        'from_email'     : EMAIL_FROM % (instance.project,
                                         get_current_user().email),
        'recipient_list' : get_manager_list(instance),
        }

    send_mail_template(**kw)
        

def mail_for_operator(instance, title='incaricato'):
    kw = {
          'template'        : 'ticket/ticket_assigned_email.mako',
          'context'        : {'ticket': instance,
                              'title' : title,
                              },                              
          'subject'        : _(u"Ticket %(project)s/%(title)s assigned") % (
                                    Item(instance)),
          'from_email'     : EMAIL_FROM % (instance.project,
                                           get_current_user().email ),
          'recipient_list' : [instance.assigned_to.email]
    }

    send_mail_template(**kw)

def mail_for_submitter(instance, old, title='persona che ha aperto il ticket'):
    kw = {
          'template'        : 'ticket/ticket_status_changed_email.mako',
          'context'        : {'ticket': instance,
                              'old' : old,
                              'title' : title,
                              },
          'subject'        : _(u'Ticket status changed for ticket #%(id)s: %(title)s') % (
                               Item(instance)),
          'from_email'     : EMAIL_FROM % (instance.project,
                                           get_current_user().email ),
          'recipient_list' : [instance.submitter.email]
    }

    send_mail_template(**kw)
