from django.newforms.models import form_for_model
from django.newforms import fields
from jungle.comments.models import FreeComment
from models import Ticket
from jungle.templatetags.jungle_tags import ugettext as _

class Comment(form_for_model(FreeComment)):

    def __init__(self, *args, **kwargs):
        super(Comment, self).__init__(*args, **kwargs)
        #self.ticket_id = ticket_id
        self.fields['manager'] = fields.BooleanField(required=False)
        self.fields['staff'] = fields.BooleanField(required=False)
        self.fields['owner'] = fields.BooleanField(required=False)

    def save(self):
        from jungle.middleware.thread_local import get_current_user
        import send_comment

        self.comment = super(Comment, self).save()
        self.user = get_current_user()
        recipients = set()

        #import ipdb; ipdb.set_trace()
        if self.cleaned_data['manager']:
            recipients = recipients.union(set(self.ticket.project.manager.all()))
            
        if self.cleaned_data['staff']:
            recipients = recipients.union(set(self.ticket.project.staff.all()))
                
        if self.cleaned_data['owner']:
            recipients.add(self.ticket.submitter)

        if recipients:
            self.send_mail(recipients)

        return self.comment

    def send_prepare(self):
        from jungle.template import mako_loader

        self.t = mako_loader.get_template('ticket/email_comment.mako')


        self.context = {
            'ticket': self.ticket,
            'comment': self.comment,
            'user'   : self.user,
            }
        info = {'id': self.ticket.id, 'project':self.ticket.project}
        self.msg = self.t.render([self.context]).encode('latin-1')
        self.subj = _("New Comment for project #%(id)s '%(project)s'").encode(
            'latin-1') % info
            #self.ticket.id, self.ticket.project
                
    def send_mail(self, dest):

        from jungle.functions import mail
        self.send_prepare()
#        to = [u.email for u in dest if u.email if u != self.user]
        to = [u.email for u in dest if u.email ]
        mail.send_mail(self.user.email, frozenset(to), self.subj, self.msg)
                               
    def get_ticket(self):
        return Ticket.objects.get(pk=self.ticket_id)

    ticket = property(get_ticket)
