from django.conf import settings
from jungle.views.decorators import login_required
from jungle.views import mgeneric as generic
from jungle.ticket.models import Ticket, Project, Action
from django.utils import simplejson
from django.http import HttpResponse
from django.db.models import Q

@login_required
def object_list(request, model, filter_cnd={}, order_by='-date_create',
                search_params={}, form=None, **kw):

    if model == 'ticket':
        order_by = 'priority'

        Q1 = Q(project__staff = request.user)
        Q2 = Q(project__manager = request.user)
        Q3 = Q(assigned_to = request.user)
        Q4 = Q(assigned_by = request.user)
        queryset = Ticket.objects.all()

        if 'my_tickets' in request.GET:
            queryset = Ticket.objects.filter(Q3)

        if 'my_projects' in request.GET:
            queryset = Ticket.objects.filter(Q2 | Q4)

    elif model == 'project':
        #queryset = Project.objects.filter(staff = request.user)
        order_by = 'name'
        queryset = Project.my_objects.all()

    elif model == 'action':
        queryset = Action.objects.filter(project__staff = request.user)

    return generic.jobject_list(request, queryset, model, form,
                                filter_cnd, order_by, **kw)

@login_required
def add(request, model=None, **kw):

    ## action needs the ticket as a hidden
    if model == 'action':
        ticket = request.GET['ticket']
        kw['extra_context'] = {'ticket' : ticket}
        kw['post_save_redirect'] = '%sticket/ticket/%s/' % (
            settings.LOCATION, ticket)

    if model == 'ticket':
        from ticket_form import TicketForm
        kw['form'] = TicketForm
        
    return generic.create_object(request, model, **kw)

@login_required
def update(request, model=None, **kw):
    #from jungle.templatetags.jungle_tags import ugettext as _
    kw['extra_context'] = {'mode' : 'update'}

    return generic.update_object(request, model, ext="_update", **kw)

@login_required
def object_detail(request, model, object_id):
    return generic.object_detail(request, model, object_id)

def add_comment(request, model='freecomment', object_id=None,
                content_type=None, **kw):
    from send_comment import Comment

    class FreeComment(Comment):
        Comment.ticket_id = object_id
        
    if object_id and content_type:
        kw.update({ 'extra_context' : {
                        'object_id'    : object_id,
                        'content_type' : content_type,},
                    'form' : FreeComment,
                    'template_name' : 'ticket/freecomment_form.mako',
                    'post_save_redirect' : '/comments/freecomment/%(id)s/',
                        })

    return generic.create_object(request, model, **kw)


@login_required
def get_ticket_project(request, model, object_id):
    #import pdb; pdb.set_trace()
    user = request.user
    get =  request.GET.copy()

    start = get.get('start', 0)
    limit = get.get('limit', 25)
    groupby = get.get('groupBy', '')
    orderby = get.get('sort', 'priority')
    direction = get.get('dir', 'ASC')

    queryset = Ticket.objects.filter(project=object_id, project__staff=request.user)
    
    total = queryset.count()

    if direction == 'DESC':
        orderby = "-%s" % orderby
        
    if groupby: #== 'category':
        queryset = queryset.order_by(groupby, orderby, '-status')
    else:
        queryset = queryset.order_by(orderby, '-status')

    dati = []
    for x in queryset:
        d = {}
        d['id'] = x.id
        d['data_creazione'] = x.date_create.strftime("%m/%d/%Y")
        d['priority'] = x.get_priority_display()
        d['submitter'] = x.submitter.username                
        d['titolo'] = x.title
        d['progetto'] = x.project.name
        d['category'] = x.category.description
        if x.assigned_by:
            d['assigned_by'] = x.assigned_by.username
        else:
            d['assigned_by'] = ''

        if x.assigned_to:                
            d['assigned_to'] = x.assigned_to.username
        else:
            d['assigned_to'] = ''
        d['status'] = x.get_status_display()
        d['data_ultima_mod'] = x.date_last_modify.strftime("%m/%d/%Y")                       
        dati.append(d)    
    
    #queryset = queryset[int(start):int(start)+int(limit)]    
    data = '{"success": true, "total": %s, "data": %s}' % (total, simplejson.dumps(dati, ensure_ascii=False))
    return HttpResponse(data, mimetype='text/javascript;')      


@login_required
def get_search_data(request, model):       
    user = request.user
    get =  request.GET.copy()

    if model == 'ticket':
        model_s = TicketSearch()

    start = get.get('start', 0)
    limit = get.get('limit', 25)
    groupby = get.get('groupBy', '')
    orderby = get.get('sort', model_s.default_sort)
    direction = get.get('dir', 'ASC')
        
    queryset = model_s.get_query_set(request)

    # Gestione della ricerca
    #
    from jungle.views.fast_search import search2
    filter_cnd, oldorder_by, form = search2(request, model)

    if filter_cnd:
        queryset = queryset.filter(**filter_cnd)

            
    total = queryset.count()
    queryset = model_s.order_by(queryset, groupby, orderby, direction)
    queryset = queryset[int(start):int(start)+int(limit)]

    dati = model_s.get_dati(queryset)
        
    data = '{"success": true, "total": %s, "data": %s}' % (total, simplejson.dumps(dati, ensure_ascii=False))
    return HttpResponse(data, mimetype='text/javascript;')


class TicketSearch:
    default_sort = 'priority'
    
    def get_query_set(self, request):
        queryset = Ticket.objects.filter(project__staff=request.user)        
        return queryset
        
    def order_by(self, queryset, groupby, orderby, dir):  
        if dir == 'DESC':
            orderby = "-%s" % orderby
        
        if groupby == '':
            groupby = 'project'
        
        if groupby: #== 'category':
            queryset = queryset.order_by(groupby, orderby, '-status')
        else:
            queryset = queryset.order_by(orderby, '-status')
                    
        return queryset
        
    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['data_creazione'] = x.date_create.strftime("%m/%d/%Y")
            d['priority'] = x.get_priority_display()
            d['submitter'] = x.submitter.username                
            d['titolo'] = x.title
            d['project'] = x.project.name
            d['category'] = x.category.description
            if x.assigned_by:
                d['assigned_by'] = x.assigned_by.username
            else:
                d['assigned_by'] = ''
    
            if x.assigned_to:                
                d['assigned_to'] = x.assigned_to.username
            else:
                d['assigned_to'] = ''
            d['status'] = x.get_status_display()
            d['descrizione'] = x.description
            d['data_ultima_mod'] = x.date_last_modify.strftime("%m/%d/%Y")                       
            dati.append(d)
        return dati
