# -*- coding: latin-1 -*-
from django.newforms.models import form_for_model
from django.newforms import fields, widgets, forms
from jungle.forms import fields as jfields
from jungle.templatetags.jungle_tags import ugettext as _
from models import Transit, Iter, Report
from jungle.ref.models import WeekDay 
from jungle.forms.fields import JungleDateField
from datetime import *
import re
from jungle.functions.utils import Dict2Obj
#from jungle.middleware.thread_local import get_current_user, _thread_locals
from jungle.timereport.time_display import HOLIDAYS
from django.conf import settings
from jungle.ticket.models import Project
from decimal import Decimal

#user = get_current_user
JOBS_PERSISTANCE = 30
class JoinReportForm(forms.BaseForm):

    def __init__(self, *args, **kwargs):
        fields = {}
        fields['job'] = Report._meta.get_field('job').formfield()
        fields['job'].widget.choices = get_jobs(self.user)
        self.base_fields = fields
        super(JoinReportForm, self).__init__(*args, **kwargs)
        
    def save(self):
        if not Report.get_jobs(self.user.id, JOBS_PERSISTANCE,
                               self.cleaned_data['job'].id, ):
            r = Report(user=self.user, job=self.cleaned_data['job'],
                       status=1, date=date.today(), qty=0, unit='h',
                       description='join')
            r.save()
            return r
        else:
            j = Report.objects.filter(
            user=self.user, job=self.cleaned_data['job'])[0]
            self.user.message_set.create(
                message="%s: job %s already joined" % (self.user, j.job))

            
def get_jobs(user):
    #for job in Project.all_objects.filter(status=1):
    for job in Project.objects.filter(status=1).order_by('name'):
        yield (job.id, job.name)
    #for job in Report.get_jobs(user.id, days=30):
    #    yield (job.job_id, job.name)
        
class ReportForm(form_for_model(Report)):
    def __init__(self, *args, **kwargs):

        super(ReportForm, self).__init__(*args, **kwargs)
        # 'fill' means one job gets the rest of unattributed hours
        self.fill = None    
        for name in ('status', 'description', 'unit', 'job', 'user', 'qty'):
            self.fields[name].required = False
            
        if self.is_bound:
            newdata = self.data
        else:
            newdata = self.initial

        self.fields['date'] = JungleDateField(label='Date')
        self.layout = []
        self.jobs = Report.get_jobs(self.user.id, days=JOBS_PERSISTANCE)
        #import pdb; pdb.set_trace()
        HOUR_CHOICES = [(0,''), (-1, _('Fill'))] + [(i, i) for i in range(1,11)]
        MIN_CHOICES=((0,''), (25, 15),(50, 30),(75, 45))
        JOB_CHOICES=[(j.job_id, j.name) for j in self.jobs]
        for job in self.jobs:
            name = "job_%s" % job.job_id
            #self.fields[name] = self.fields['job']
            self.fields[name] = fields.ChoiceField(
                required=False, choices=JOB_CHOICES)            
            self.fields[name+'_h'] = fields.ChoiceField(
                required=False, choices=HOUR_CHOICES, label='')
            self.fields[name+'_min'] = fields.ChoiceField(
                required=False, label='', choices=MIN_CHOICES)
            self.fields[name+'_desc'] = fields.CharField(
                max_length=30, required=False, label='')
            self.layout += ["%s %s_h  %s_min t=%s_desc:.60" % (
                name, name, name, name)]
            
            if not self.is_bound:
                newdata.update({name : job.job_id})

        #self.fields['job'] = Report._meta.get_field('job').formfield()
        #self.fields['job'].widget.choices = get_jobs()

        self.layout = '\n'.join(self.layout)

    def clean(self):
        for job in self.jobs:
            name = "job_%s" % job.job_id
            if self.get_time_qty(name) and self.descr_is_empty(name):
                msg = _("All descriptions need to be filled in.")
                raise forms.ValidationError(msg)
        return self.cleaned_data
        
    def save(self):
        
        for job in self.jobs:
            name = "job_%s" % job.job_id
            if self.get_time_qty(name):
                self.add_or_update_job_time(job, name)
        if self.fill:
            name = "job_%s" % self.fill.job_id
            self.add_or_update_job_time(self.fill, name)

        return self.create_obj()

    def create_obj(self):
        ## the following is just to use post_save_redirect:
        ## /report/day/%(date)s/%(user)s/
        class A: pass
        a= A()
        a.date = self.cleaned_data['date'].strftime("%Y/%m/%d")
        a.user = self.user.id
        return a


    def get_time_qty(self, name):
        """return a decimal of hour + minutes corresponding to hours"""
        ### missing a float I'm using an integer...
        
        try:
            h = int(self.cleaned_data[name + '_h'] or 0) 
            m = int(self.cleaned_data[name + '_min'] or 0)
            return h*100 + m
        except TypeError:
            # in case this is called by clean and no hours defined for this job
            return None
            
    def descr_is_empty(self, name):
        key = "%s_desc" % name
        if key in self.cleaned_data and self.cleaned_data[key]:
            return False
        else:
            return True
           

    def add_or_update_job_time(self, job, name):
        
        # Originally I wanted to be able to update, now I'm happy just with
        # insert so that I can add 2 times the same job

        ### NOTE: I use job.id as an index NOT as a real job.id
        ##  since the widget is a menu of jobs the user can change the
        ##  value of the job so that eg.: cleaned_data['job_1'] is really 2
        data = dict(
            status = 1,
            unit = u'h',
            job_id = self.cleaned_data[name],
            #job=Project.objects.get(pk=job.job_id),            
            user = self.user,
            date = self.cleaned_data['date'],
            )
        #entries = Report.objects.filter(**data)
        #if entries:
        #    obj = entries[0]
        #    obj.description = self.cleaned_data[name + '_desc']
        #    obj.qty = self.get_time_qty(name)
        #else:
        data.update({
            'description': self.cleaned_data[name + '_desc'],
            'qty': self.get_time_qty(name),
            })
        obj = Report(**data)
        obj.save()

    def get_day_times(self, date):
        records = Report.objects.filter(
            user=self.user, date=date)

        data = self.initial
        data['date'] = date
        #import pdb; pdb.set_trace()
        for R in records:
            name = "job_%s" % R.job_id            
            hours, minutes = divmod(R.hours(),1)

            data[name] = R.job_id
            data[name + '_h'] = int(hours)
            data[name + '_min'] = int(minutes*100)
            data[name + '_desc'] = R.description
