from django.newforms.models import form_for_model
from django.newforms import fields
from jungle.forms.fields import JungleTimeField, JungleDateField
from jungle.templatetags.jungle_tags import ugettext as _
from models import Transit
from jungle.ref.models import WeekDay 
from django.newforms.util import ErrorList, ValidationError, smart_unicode
import datetime
import re
from jungle.timereport.badges import DEFAULT_REASON

class TransitForm(form_for_model(Transit)):
    """ this form is only used to *add* new fields not to edit them
    the real data that is needed in this context is the time, all other
    data are already present as hidden fields
    """

    def __init__(self, *args, **kwargs):
        super(TransitForm, self).__init__(*args, **kwargs)
        
        self.fields['time'] = JungleTimeField(required=True)
        self.fields['date_reference'] = JungleDateField(required=True)
        self.fields['date_time'] = fields.DateTimeField(required=False)


    def save(self):
        """we need to save a datetime but we only ask for a time
        """
        date_time = datetime.datetime.combine(
            self.cleaned_data['date_reference'],
            self.cleaned_data['time'],
            )
        if not self.cleaned_data['reason']:
            self.cleaned_data['reason'] = DEFAULT_REASON
        else:
            self.cleaned_data['type'] = 4  # transit with reason
        ## FIXME: illness and holiday become real transit
        ## while they should be 0: no transit: who cares?
          
        self.cleaned_data.update({
            'date_time': date_time,
            'status' : '2',  # manually inserted
            })
        return super(TransitForm, self).save()
