from django.db import models

# Create your models here.

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

#################################
# ANAGRAFICA US                 #
#################################
class Profile (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default='1',
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    title = models.ForeignKey (
        "Title",
        verbose_name="Ruolo Anagrafica",
    )
    office = models.ForeignKey (
        "Office",
        verbose_name="Ufficio",
    )
    ## Campi
    corporate = models.CharField (
         null=True,
        blank=True,   
        #unique=True,
        max_length=50,
        verbose_name="Ragione Sociale",
    )
    lastname = models.CharField (
        max_length=35,
        verbose_name="Cognome",
    )
    firstname = models.CharField (
        max_length=35,
        verbose_name="Nome",
    )
    email = models.EmailField (
        max_length=100,
        null=True,
        blank=True,
        verbose_name="Email",
    )
    note = models.TextField (
        max_length=255,
        null=True,
        blank=True,
        verbose_name="Note",
    )
    vat_number = models.CharField (
        max_length=11,
        null=True,
        blank=True,
        verbose_name="Partita IVA",
    )
    code_tax = models.CharField (
        max_length=16,
        null=True,
        blank=True,
        verbose_name="Codice Fiscale",
    )
    phone = models.CharField (
        null=True,
        blank=True,
        max_length=20,
        verbose_name="Telefono",
    )
    mobile = models.CharField (
        null=True,
        blank=True,
        max_length=20,
        verbose_name="Cellulare",
    )
    #### FIXME: non scommentare se non ti sei posto il probelma di
    ####        aggiornare tutti i db che non ce l' hanno!!!

    registration = models.CharField (
         null=True,
         blank=True,
         max_length=15,
         verbose_name='Matricola',
     )

    class Admin:
        search_fields = ("status", "title", "office",)
        date_hierarchy = "date_create"
        list_display = ("status", "office", "title", "corporate", "lastname", "firstname", "phone", "mobile", "email",)
        list_display_links = ("corporate", "lastname", "firstname",)
        list_per_page = 30
        fields = (
            (None, {"fields": (
                ("status", "office", "title",),
                ("corporate", "lastname", "firstname"),
                ("phone", "mobile", "email",),
                ("vat_number", "code_tax", "registration",),
                ("note",),
            ),}),
        )
        ordering = ("lastname", "firstname",)

    class Meta:
        unique_together = (
            ("vat_number", "code_tax",),
        )
        verbose_name = "Profilo Utente"
        verbose_name_plural = "Profili Utente"

    def __unicode__(self):
        return unicode(self.lastname+" "+self.firstname)
    
#######################################
# ANAGRAFICAUS RUOLO                  #
#######################################
class Title (models.Model):
    ## Campi Generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice",
    )
    name = models.CharField (
        unique=True,
        max_length=50,
    )

    class Admin:
        search_fields = ("status", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "name",)
        list_display_links = ("name", )
        list_per_page = 30
        ordering = ("name",)
        fields = (
            (None, {"fields": (
                ("status", "code","name",),
            ),}),
        )

    class Meta:
        unique_together = (("code", "name",),)
        verbose_name = "Ruolo Utente"
        verbose_name_plural = "Ruoli Utente"

    def __unicode__(self):
        return unicode(self.name)
    
#############################
# UFFICIO                   #
#############################
class Office (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (
        auto_now_add=True,
        verbose_name="Data Creazione",
    )
    date_last_modify = models.DateTimeField (
        auto_now=True,
        verbose_name="Data Modifica",
    )
    status = models.IntegerField (
        max_length=3,
        default=1,
        choices=STATUS_CODES,
        verbose_name="Stato",
    )
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique=True,
        max_length=15,
        verbose_name="Codice"
    )
    name = models.CharField (
        max_length=50,
        verbose_name="Ufficio"
    )
    description = models.TextField (
        null=True,
        blank=True,
        max_length=255,
        verbose_name="Descrizione"
    )
    phone = models.CharField (
        max_length=20,
        verbose_name="Telefono",
    )
    fax = models.CharField (
        max_length=20,
        verbose_name="Fax",
    )
    email = models.EmailField (
        max_length=100,
        null=True,
        blank=True,
    )

    class Admin:
        search_fields = ("status", "code", "name",)
        date_hierarchy = "date_create"
        list_display = ("status", "code", "name", "phone", "fax", "email",)
        list_display_links = ("code", "name", )
        list_per_page = 30
        ordering = ("name",)
        fields = (
            (None, {"fields": (
                ("status", "code","name",),
                ("description",),
                ("phone", "fax", "email",),
            ),}),
        )

    class Meta:
        verbose_name = "Ufficio"
        verbose_name_plural = "Ufficio"

    def __unicode__(self):
        return unicode(self.name)
