from django.db.models import fields as db_fields
from django.newforms import widgets as wid
from jungle.forms.fields import JungleDateField, JungleTimeField, \
     JungleDateTimeField, JungleNullBooleanField
from jungle.forms.widgets import DateTimeWidget,  DateWidget
from django.newforms.widgets import SplitDateTimeWidget

def ffc():
    return FormfieldChooser().formfield

class FormfieldChooser(object):
    """
    Options to configure how a fast search should display data
    """
    def __init__(self, user=None):
        self.required = None
        self.user = user
        
    def formfield(self, field, initial=None):
        """check if an overload is present otherwise return default w/
        required=False"""
        ff = getattr(self, "formfield_%s" % field.name, self.formfield_default)
        ## FIXME sno solo questi i defaults da tenere in considerazione?
        defaults = {'required': not field.blank, 'label': field.verbose_name.title(), 'help_text': field.help_text}
        return ff(field, initial=initial, defaults=defaults, )
        
    def formfield_default(self, field, initial=None, defaults=None):
        """return the formfield widget. This is the formfield_callback given to
        form_for_model
        """
        opts = {'initial' :initial}
        opts.update(defaults)
        if self.required == False:
            opts['required'] = False
            #import pdb;pdb.set_trace()
        ## FIXME: we are missing vDateTimeField        
        if isinstance(field, db_fields.DateTimeField):

            opts.update({
                'widget' : DateTimeWidget(),
                #'widget' : SplitDateTimeWidget(),
            })
            return JungleDateTimeField(**opts)
        #return field.formfield(**opts)
    
        elif isinstance(field, db_fields.DateField):
            opts.update({
                'widget' :  DateWidget()
                })
            return JungleDateField(**opts)
        
        elif isinstance(field, db_fields.TimeField):
            opts.update({
                'widget' : wid.TextInput(attrs={
                             'size' : 8,
                             'maxlength' : 8,
                             'class' : 'vTimeField',}),
            })
            return JungleTimeField(**opts)
            #return field.formfield(**opts)
        

        elif isinstance(field, db_fields.NullBooleanField):
            return JungleNullBooleanField(**opts)
        
        elif field.choices:
            opts.update({'widget' : wid.Select(choices=field.get_choices())})
            return field.formfield(**opts)
        else:
            return field.formfield(**opts)

    def choices(self, queryset, defaults=None, initial=None):
        opts = {'initial' :initial}
        if self.required == False:
            opts['required'] = False

        from django.newforms.models import ModelChoiceField

        wdg = ModelChoiceField(required=False, queryset=queryset)
        return wdg
        

