"""
JungleDateTime allows some symbolic math. This allows relative time definition:
  m-1: beginnning of last month
  M+1: end of next month

"""
import django.newforms as forms
from django.newforms.fields import DateField, DateTimeField, TimeField, \
     NullBooleanField, ChoiceField
import re
from datetime import *
from jungle.functions.dateutils.relativedelta import relativedelta
from django.utils.translation import gettext
from widgets import JungleNullBooleanSelect, JungleSelect, \
     JDateTimeWidget, JDateWidget, DualPasswordWidget

DateTimeWidget = JDateTimeWidget 
DateWidget = JDateWidget 

def transform(m):
    TODAY = date.today()
    period = m.group('period')
    math = m.group('math') 
    try:
        math = int(math)
    except TypeError:
        math = None
    
    if m.group('period') == 'd':
        computed = TODAY
        if math is not None:
            computed = computed + relativedelta(days=+math)

    if m.group('period') in ('w', 'W'):
        computed = TODAY - timedelta(days=TODAY.isoweekday()-1)
        if math is not None:
            computed = computed + relativedelta(weeks=+math)
        if period == 'W':
            computed = computed + relativedelta(weeks=+1, days=-1)
            

    if period in ('m', 'M'):
        computed =  TODAY.replace(day=1)
        if math is not None:
            computed = computed + relativedelta(months=+math)

        if period == 'M':
            computed = computed + relativedelta(months=+1, days=-1)
            
    if m.group('period') in ('y', 'Y'):
        j = int(TODAY.strftime('%j'))
        computed = TODAY - timedelta(days=j-1)
        if math is not None:
            computed = computed + relativedelta(years=+math)
        if period == 'Y':
            computed = computed + relativedelta(years=+1, days=-1)
            

    return computed

class JungleDateField(DateField):
    widget = JDateWidget
    DATE_PATTERN = '(?P<double>@)?(?P<period>[mMdDyYwW])(?P<math>[+-][0-9]*)?$'

    def __init__(self, default=None, *args, **kw):
        super(JungleDateField, self).__init__(*args, **kw)
        self.widget.attrs.update({'size' : 10, 'maxlength' : 10})
        if default:
            self.widget.attrs.update({'default' : default})
        
    def clean(self, value):
        ## we will handle any form of fancy way of using dates
        if isinstance(value, (str, unicode)):
            m = re.match(self.DATE_PATTERN, value)

            if m:
                value = transform(m)

        return super(JungleDateField, self).clean(value)


class JungleDateTimeField(DateTimeField):
    widget = JDateWidget
    DATE_PATTERN = '(?P<double>@)?(?P<period>[mMdDyYwW])(?P<math>[+-][0-9]*)?$'

    def __init__(self, default=None, *args, **kw):
        super(JungleDateTimeField, self).__init__(*args, **kw)
        self.widget.attrs.update({'size' : 18, 'maxlength' : 18})
        if default:
            self.widget.attrs.update({'default' : default})
        
    def clean(self, value):
        ## we will handle any form of fancy way of using dates
        if isinstance(value, (str, unicode)):
            m = re.match(self.DATE_PATTERN, value)

            if m:
                value = transform(m)

        ret = super(JungleDateTimeField, self).clean(value)
        return ret



class JungleTimeField(TimeField):
    def __init__(self, *args, **kw):
        super(JungleTimeField, self).__init__(*args, **kw)
        
    def clean(self, value):
        if isinstance(value, (str, unicode)):
            value = value.replace('.',':')
            m = re.match('(?P<hour>\d\d)[.:]?$', value)
            if m:
                value = "%s:00" % m.group('hour')
            
        ## we will handle any form of fancy way of using dates
        return super(JungleTimeField, self).clean(value)


class JungleNullBooleanField(NullBooleanField):
    widget = JungleNullBooleanSelect
    def __init__(self, *args, **kwargs):
        NullBooleanField.__init__(self, *args, **kwargs)

    def clean(self, value):
        if value in (4, '4', 'NULL'):
            return 'NULL'
        return NullBooleanField.clean(self, value)
        
class JungleNullBooleanFieldRev(JungleNullBooleanField):
    def clean(self, value):
        """0|1 passed via post need to be transformed into int otherwise
        a 0 that is passed in a GET is understood as True"""
        
        value = super(JungleNullBooleanField, self).clean(value)
        if value is not None:
            value = not value
        return value


class JungleChoiceField(ChoiceField):
    def __init__(self, *args, **kwargs):
        ChoiceField.__init__(self, widget=JungleSelect, *args, **kwargs)        
    
# def set_today(date):
#     """this is just for the benefit of tests, to be able to set what is
#     considered 'today' """
#     global TODAY
#     TODAY = date

# def get_today():
#     return TODAY
class DualPasswordField(forms.Field):  
     """ 
     Field used to prevent password typos by asking the same password twice 
     and checking if it's the same 
     """  
     widget = DualPasswordWidget()  
     def __init__(self, *args, **kwargs):  
         kwargs.setdefault('help_text', 'Insert your password twice.')
         super(DualPasswordField, self).__init__(*args, **kwargs)  
   
     def clean(self, value):  
         super(DualPasswordField, self).clean(value)  
         if value:  
             pass0, pass1 = value  
             if pass0 == pass1:  
                 return pass0  
             raise forms.ValidationError('Password Missmatch.')  
