from django import newforms as forms
from django.newforms import widgets
from django.utils.translation import gettext
from django.utils.encoding import force_unicode
from django.conf import settings
import datetime
import time

# DATETIMEWIDGET
calbtn = u"""<img src="%simg/icon_calendar.gif" alt="calendar" id="%s_btn" style="cursor: pointer; margin-left:6px;" title="Select date and time"
            onmouseover="this.style.background='#444444';" onmouseout="this.style.background=''" />
<script type="text/javascript">
    Calendar.setup({
        inputField     :    "%s",
        ifFormat       :    "%s",       
        button         :    "%s_btn",
        singleClick    :    true,
        showsTime      :    true
    });
</script>"""

class JDateTimeWidget(widgets.TextInput):
    ## FIXME: should depend on US/European input format
    dformat = '%d/%m/%Y %H:%M'
    default_input_formats = ('%d/%m/%Y %H:%M',  '%d/%m/%y %H:%M') +forms.fields.DEFAULT_DATE_INPUT_FORMATS

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''

        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '': 
            try:
                final_attrs['value'] = \
                                   force_unicode(value.strftime(self.dformat))
            except:
                final_attrs['value'] = \
                                   force_unicode(value)
        
        if not final_attrs.has_key('id'):
            final_attrs['id'] = u'%s_id' % (name)
        id = final_attrs['id']
        
        jsdformat = self.dformat #.replace('%', '%%')
        cal = calbtn % (settings.MEDIA_URL, id, id, jsdformat, id)
        
        d_value = ""
        if final_attrs.has_key('default'):
            d_value = final_attrs["default"]
            
        a = u'<input%s value="%s"/>%s' % (forms.util.flatatt(final_attrs), d_value, cal)
        return a

    def value_from_datadict(self, data, files, name):

        empty_values = forms.fields.EMPTY_VALUES
        
        value = data.get(name, None)

        if value in empty_values:
            return None
        if isinstance(value, datetime.datetime):
            return value
        if isinstance(value, datetime.date):
            return datetime.datetime(value.year, value.month, value.day)
        for format in self.default_input_formats:
            try:
                return datetime.datetime(*time.strptime(value, format)[:6])
            except ValueError:
                return value

        return None

class JDateWidget(JDateTimeWidget):
    dformat = '%d/%m/%Y'
    default_input_formats = ('%d/%m/%Y',  '%d/%m/%y') +forms.fields.DEFAULT_DATE_INPUT_FORMATS

DateWidget = JDateWidget
DateTimeWidget = JDateTimeWidget

# class DateTimeWidgetOld(Widget):
#     dformat = '%d/%m/%Y %H:%M'
#     date_field = '%s_date'
#     time_field = '%s_time'

#     def __init__(self, attrs=None):
#         self.attrs = attrs or {}

#     def render(self, name, value, attrs=None):
#         date = time = None
#         if value:
#             try:
#                 date, time = value.split(' ')
#             except (AttributeError, TypeError, ValueError):
#                 if len(value_splitted) == 1:
#                     date = value_splitted[0]

#         date_widget = TextInput(attrs={
#                                          'size' : 10,
#                                     'maxlength' : 10,
#                                         'class' : 'vDateField',})

#         time_widget = TextInput(attrs={
#                                          'size' : 8,
#                                     'maxlength' : 8,
#                                         'class' : 'vTimeField',})

#         return u''.join([date_widget.render(self.date_field % name, date),
#                            time_widget.render(self.time_field % name, time)])

#     def value_from_datadict(self, data, files, name):
#         ## when the form is filled from the db (rather that from POST)
#         ## we need to split the datetime value into 2 different values
#         if self.date_field % name not in data and name in data:
#             if data.get(name):
#                 data.setlist(self.date_field % name, [data.get(name).date()]) 
#                 data.setlist(self.time_field % name, [data.get(name).time()]) 
            
#         d, t = data.get(self.date_field % name), data.get(self.time_field % name)
        
#         if d and t:
#             return '%s %s' % (d, t)
#         return ''


class JungleNullBooleanSelect(widgets.NullBooleanSelect):
    """Added to be used in fast_search widget
    let define if you don't want to filter or if you filter a NULL value
    To be used along with oper .bool in fast_search view/templatetag"""
    
    def __init__(self, attrs=None):
        choices = ((u'1', '---'), (u'2', gettext('Yes')),
                   (u'3', gettext('No')),)
        widgets.Select.__init__(self, attrs, choices)

    def render(self, name, value, attrs=None, choices=()):
        try:
            value = {True: u'2', False: u'3', u'NULL' : u'4',
                     u'2': u'2', u'3': u'3', u'4': u'4'}[value]
        except KeyError:
            value = u'1'
        return widgets.Select.render(self, name, value, attrs, choices)

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        return {u'NULL' : u'NULL', '4': u'NULL', u'2': True,
                u'3': False, True: True, False: False}.get(value, None)


class JungleSelect(widgets.Select):    
    #def __init__(self, attrs=None, choices=(), empty_text="-----"):
        #import pdb
        #pdb.set_trace()
        #self.attrs = attrs or {}
        #self.choices = list(choices)
        #self.choices.insert(0, ('', empty_text))

     #   widgets.Select.__init__(self, attrs, choices)

    def render(self, name, value, attrs=None, choices=()):
        #import pdb
        #pdb.set_trace()
        self.choices.insert(0, ('', '-----'))
        
        return widgets.Select.render(self, name, value, attrs, choices)

class DualPasswordWidget(forms.Widget):  
    pass0_field = '%s_pass0'  
    pass1_field = '%s_pass1'  
   
    def __init__(self, *args, **kwargs):  
        super(DualPasswordWidget, self).__init__(*args, **kwargs)  
   
    def render(self, name, value, *args, **kwargs):  
        out = ( forms.PasswordInput().render(self.pass0_field % name, None),  
                forms.PasswordInput().render(self.pass1_field % name, None) )  
        return '<br />'.join(out)  
  
    def value_from_datadict(self, data, files, name):  
        pass0 = data.get(self.pass0_field % name, None)  
        pass1 = data.get(self.pass1_field % name, None)  
        if pass0 and pass1:  
            return (pass0, pass1)  
        return None
