import os, re
from threading import local

_thread_locals = local()

try:
    LOCAL_SETTINGS = os.environ['DJANGO_SETTINGS_MODULE'].replace(
        '.settings', '.local_settings')
except:
    ##  we are probably using manage.py from this same directory
    ## or we are using a test
    LOCAL_SETTINGS = 'local_settings'

try:
    local_settings = __import__(LOCAL_SETTINGS, '','',[''])
except:
    local_settings = False
    
def set_cwd(CWD):
    _thread_locals.CWD = CWD

def get_cwd():
    return getattr(_thread_locals, 'CWD', None)

def set_interactive_user(username):
    from django.conf import settings
    from django.contrib.auth.models import User
    setattr(settings, 'INTERACTIVE_USER', User.objects.get(username=username))

def set_default_date_input_european():
    try:
        from django.forms import fields
    except Exception, ee:
        print ee
    
    try:
        from django.forms import widgets
    except Exception, ee:
        print ee

    DATE_INPUT_FORMATS = (
        '%Y-%m-%d', '%Y/%m/%d',             # '2008-10-25', '2008/10/25'
        '%d-%m-%Y', '%d/%m/%Y',             # '25-10-2006', '25/10/2006'
        '%d-%m-%y', '%d/%m/%y',             # '25-10-06', '25/10/06'
    )
    DATETIME_INPUT_FORMATS = (
        '%Y-%m-%d %H:%M:%S',     # '2006-10-25 14:30:59'
        '%Y-%m-%d %H:%M',        # '2006-10-25 14:30'
        '%Y-%m-%d',              # '2006-10-25'
        '%d/%m/%Y %H:%M:%S',     # '10/25/2006 14:30:59'
        '%d/%m/%Y %H:%M',        # '10/25/2006 14:30'
        '%d/%m/%Y',              # '10/25/2006'
        '%d/%m/%y %H:%M:%S',     # '10/25/06 14:30:59'
        '%d/%m/%y %H:%M',        # '10/25/06 14:30'
        '%d/%m/%y',              # '10/25/06'
    )
     
         
    setattr(fields, 'DEFAULT_DATE_INPUT_FORMATS', DATE_INPUT_FORMATS)
    setattr(fields, 'DEFAULT_DATETIME_INPUT_FORMATS', DATETIME_INPUT_FORMATS)
    setattr(widgets.DateTimeInput, 'format', '%d/%m/%Y %H:%M:%S')

    
def get_class(class_path):
    path = class_path.split('.')
    myclass = path.pop()
    mod_path = '.'.join(path)
    mod = __import__(mod_path, {}, {}, [''])
    return getattr(mod, myclass)

def get_from_local_settings(var, default=None):
    """return var from local_settings.py if any, otherwise default"""


    if var in dir(local_settings):
        return getattr(local_settings, var)
    else:
        return default

get = get_from_local_settings

def is_running_in_apache():
    """test if we are running as mod_python or standalone dev server"""
    
    try:
        from mod_python import apache
        return True
    except:
        return False

def add_jungle_modules_path(INSTALLED_APPS):
    import sys
    mod_jungle = __import__('jungle')
    
    for app in INSTALLED_APPS:
        if app.startswith('jungle.'):
            sys.path.append("%s/%s" % (
                mod_jungle.__path__[0], app.replace('jungle.','')))
    

def jungle_app_include(apps):
    """return a list of include base on jungle applications

    apps = 'ticket,fax,userprofile,address,mailserver'
    urlpatterns += jungle_app_include(apps)

    apps may be a comma separated string or a list/tuple

    JUNGLE_URLS = 'ticket fax userprofile address mailserver agenda ' + \
                  'organization userprofile faq docs'

    urlpatterns += jungle_app_include(JUNGLE_URLS)
    """
    from jungle.middleware.add_location import lpatterns
    from django.conf.urls.defaults import include
    from django.conf import settings

    patterns = []
    if isinstance(apps, (str, unicode)):
        apps = re.split('[ ,]+', apps)
        
    for app in apps:
        app = app.strip()
        if app == 'doc':
            patterns += lpatterns('', (r'^(%s)/' % settings.DOCUMENT_URL.lstrip('/'),
                                  include('jungle.docs.urls' )))
        elif '.' in app:
            patterns += lpatterns('', (r'^%s/' % app.split('.').pop(),
                                              include('%s.urls' % app)))
        else:
            if app:
                patterns += lpatterns('', (r'^%s/' % app,
                                           include('jungle.%s.urls' % app)))
    return patterns
        

def jungle_default_patterns(default_view=None, default_app=None,
                            default_model=None, default_url=None):
                            
    """ return a list of default patterns we always want

    Default view/model are the view/mode where we will be next'ed after login
    If we don't arrive explicitely require one.
    
    default_view: a string that refers to the default view
                  eg: 'jungle.ticket.views.object_list'

    default_model: a string that refers to a model:
                  eg: 'ticket'

    to be used as::

        urlpatterns += jungle_default_patterns(
             'jungle.ticket.views.object_list', 'ticket')

    """

    from jungle.middleware.add_location import lpatterns
    from django.conf.urls.defaults import include
    from django.conf import settings

    if not default_url:
        if not default_model:
            default_url = '/%s/' % (default_app)
        else:
            default_url = '/%s/%s/' % (default_app, default_model)


    return lpatterns('',
       (r'^admin/', include('django.contrib.admin.urls')),
       (r'^comments/', include('jungle.comments.urls')),
       (r'^help/', include('jungle.help.urls')),
       (r'^registration/', include('jungle.registration.urls')),
       ## default
       (r'^$', 'django.views.generic.simple.redirect_to', {'url': default_url}),
       ## account                    
       (r'^accounts/profile/$', 'django.views.generic.simple.redirect_to', {'url': default_url}),
       (r'^accounts/login/$', 'jungle.views.auth.login'),
       (r'^accounts/logout/$', 'jungle.views.auth.logout_then_login',
        {'login_url': '%saccounts/login' % settings.LOCATION}),
       (r'^accounts/change_password/$', 'jungle.views.auth.password_change'),        
       (r'^accounts/change_password/done/$', 'jungle.views.auth.password_change_done'),
       ## internationalization
       (r'^i18n/', include('django.conf.urls.i18n')),
       (r'^jsi18n/$', 'django.views.i18n.javascript_catalog',
        {'packages': 'django.conf'}),
       ## static - normally overwritten by apache
       (r'^static/(.*)$', 'django.views.static.serve',
         {'document_root': settings.MEDIA_ROOT}),
       ## file
       (r'^file/', include('jungle.file.urls')),
       (r'^views/close_popup/', 'jungle.views.mgeneric.close_popup'),
       ## Load Ext Menu
       (r'^ajax/load_menu/', 'jungle.views.ajax.load_menu'),
    )


    
def add_doc_root_path(urlpatterns, url, login_required=False):
    """ helper function to add documentation, possibly with login::
    
    add_doc_root_path('jungle')
    add_doc_root_path('thunder', login_required=True)

    you are supposed to add entryes in settings.py eg:
    JUNGLE_DOCS_PATH='path_do_docs'
    THUNDER_DOCS_PATH='path_do_docs'

    """

    from django.conf import settings
    from django.conf.urls.defaults import patterns

    EXT = 'txt|jpe?g|png|pdf|ps|xls|doc|odp'

    document_root = getattr(settings, "%s_DOCS_PATH" % url.upper())
    document_url =  getattr(settings, "%s_URL" % url.upper(), url)
    info = {'document_root' : document_root,
            'document_url' :"/%s" % document_url,
            'login_required' : login_required}

    urlpatterns += patterns('',
    (r'^%s/(?P<path>.*\.(?:%s))$' % (url, EXT),
     'django.views.static.serve', {'document_root' : document_root}),
                            (r'^%s/(?P<path>.*)/?$' % document_url,
                             'jungle.apps.docs.views.doc_view', info))
                            
