"""
this  module provides write, dshow and ddir, should be imported as:
from jungle.functions.dbg import dbg

write: w/ no args (dbg.write()) writes calling scope and caller
       otherwise writes the caller/line of code and the args
dshow: prints a dictionary in a nice way (no recursion)
ddir:  prints a dir(obj) in a nice way

the file where debug is written can be set in settings.DEBUG_FILE and defaults
    to /var/log/django.log. Must be writeable by user/www-data
"""

import os, sys
try:
    from django.conf import settings
    if settings.DEBUG:
        DBG = 1
    else:
        DBG = 0
except:
    DBG = 1

try:
    DBG_File = settings.DEBUG_FILE
except:
    DBG_File = '/var/log/django.log'

Log = open(DBG_File,mode='a+')
DBGLogger = None

def debug(x=None):
    global DBG
    global Log
    global dbg_file
    if x is None:
        return DBG
    if x is True:
        DBG = 1
        return
    if x is False:
        DBG = 0
        return
    if isinstance(x , int):
        DBG = x
        return
        
    Log.write("""dbg.debug now is called dbg.write, dbg.debug accept True|False
    and will switch debugging on or off\n""".encode('latin-1'))
    Log.flush()
    
def write(*text, **kwargs):
    global DBG
    global Log
    global DBGLogger
    if 'filter' not in kwargs:
        filter = None
    
    if 'sfilter' not in kwargs:
        sfilter = None
    
    if 'direct' not in kwargs:
        kwargs['direct'] = False

    if 'meth' not in kwargs:
        meth = caller()
    else:
        meth = kwargs['meth']
        
    if 'called_by' not in kwargs:
        called_by = ''
    else:
        called_by = kwargs['called_by']
        
    if DBG == 0:
        return
    if not text:
        Log.write(u"%s (line %s) locals(): \n%s\n".encode('latin-1') % (caller(), caller(mode='lineno'),
                          dshow(caller(mode='loc', level=2), ret=True,
                                filter=filter, sfilter=sfilter,mute=True)))
                                               
        Log.flush()
        return
    
    txt = u''
    for i in text:
        txt += u' ' + unicode(i)

    if not kwargs['direct'] and DBGLogger:
        try:
            instance=caller(mode='instance')
            Class_str = instance.nick
        except Exception, e:
            Class_str = ''
        
        DBGLogger.write(txt=txt.encode('latin-1'), line=caller(mode='lineno'), meth=meth,
                        mode='write', Class=Class_str, caller=called_by)
    else:
        #print "dbg.write: QUI 2, DBGLogger:", DBGLogger, "direct:", kwargs['direct']
        Log.write( u'%s (line %s): %s\n'.encode('latin-1') % (caller(), caller(mode='lineno'), txt))
        Log.flush()
    
def caller(mode=None, level=2):
    """
    mode can be:
    lineno:  just return line number for the calling frame
    loc:     return the local variables as dict
    None:    return (class_name, caller_func)
    Class :   return class
    instance : return class if applicable
    """
    
    frame = sys._getframe(level)

    if mode == 'lineno':
        return frame.f_lineno
    
    if mode == 'loc':
        return frame.f_locals

    caller = frame.f_code.co_name
    class_name = ''


    try:
        a = frame.f_locals['self'].__class__
        class_name =  re.search("'(.*)'", str(a)).group(1) + "."

        if mode == 'Class':
            return a

        if mode == 'instance':
            return frame.f_locals['self']

    except:
        pass
    return "%s%s" % (class_name, caller)
    
def show_caller(filter=None, sfilter=None, loc=True):
    if DBG != 0:
        if loc:
            Locals = dshow(caller(mode='loc', level=3), ret=True,
                           filter=filter, sfilter=sfilter,mute=True)
        else:
            Locals = ''
            
        Log.write("%s called from: %s (line %s) \n%s\n".encode('latin-1') % (
            caller(),
            caller(level=3), caller(level=3, mode='lineno'), Locals ))
        Log.flush()
    
def dshow(diz, filter=None, sfilter=None, mute=False, ret=False, name=None):
    """print a dictionary in a friendly way
    filter:  only print keys that match 'filter'
    sfilter: only print keys that _don't_ match 'filter'
    """

    global DBG
    global Log
    
    if DBG == 0:
        return
    txt = []
    if not mute:
        txt += [caller()]
    filtered = {}
    for k,v in diz.iteritems():
        if filter:
            if not re.search(filter, k):
                continue
        if sfilter:
            if re.search(sfilter, k):
                continue
        txt += ['    %s: %s' % (k, v)]
        filtered[k] = v
    if ret:
        return u'\n'.join(txt)
    else:
        try:
            iter = DBGLogger.write(meth=caller(), txt=name, mode='dict')
            for k,v in filtered.iteritems():
                DBGLogger.write(meth=k, txt=v, mode='value', parent=iter)
        except:
            Log.write( u'\n'.join(txt) + '\n'.encode('latin-1'))
            Log.flush()
    
def ddir(obj, filter=None, sfilter=None, mute=False, ret=False):
    """print dir of an object in a readable fasion
    filter:  only print keys that match 'filter'
    sfilter: only print keys that _don't_ match 'filter'
    """

    global DBG
    global Log
    
    if DBG == 0:
        return
    txt = []
    if not mute:
        txt += [caller()]
    for k in dir(obj):
        if filter:
            if not re.search(filter, k):
                continue
        if sfilter:
            if re.search(sfilter, k):
                continue
        v = getattr(obj, k)
        txt += [u'    %s: %s' % (k, v)]
    if ret:
        return "\n".join(txt)
    else:
        Log.write( u'\n'.join(txt).encode('latin-1') + '\n')
        Log.flush()
    
    

