# -*- coding: utf-8 -*-

r"""
>>> import os
>>> os.environ['DJANGO_SETTINGS_MODULE'] = 'jobs.settings'
>>> from jungle.ticket.models import Ticket
>>> from django import newforms as forms
>>> from django.newforms.fields import *
>>> from django.utils.datastructures import MultiValueDict
>>> from django.newforms.widgets import *
>>> from django.conf import settings
>>> settings.LANGUAGE_CODE = 'en-en'
>>> form = forms.form_for_model(Ticket)()
>>> form.fields.keys()
['date_create', 'date_last_modify', 'status', 'project', 'submitter', 'assigned_to', 'title', 'keywords', 'description', 'priority', 'public', 'duration', 'attach_file']
>>> form.data
{}
>>> form.fields['new_field'] = CharField()
>>> form.fields.keys()
['date_create', 'date_last_modify', 'status', 'project', 'submitter', 'assigned_to', 'title', 'keywords', 'description', 'priority', 'public', 'duration', 'attach_file', 'new_field']
>>> d = MultiValueDict({'new_date' : ['2006-12-25'], 'name': ['Adrian', 'Simon'], 'position': ['Developer']})
>>> form.data = d
>>> form.data
<MultiValueDict: {'position': ['Developer'], 'new_date': ['2006-12-25'], 'name': ['Adrian', 'Simon']}>
>>> form.is_bound
False
>>> form.is_bound = True
>>> form.is_bound
True
>>> from django.newforms.widgets import *
>>> form.fields['new_date'] = DateField(required=False, widget=TextInput(attrs={'size':10,'maxlength':10,'class':'vDateField',}))
>>> form.fields.keys()
['date_create', 'date_last_modify', 'status', 'project', 'submitter', 'assigned_to', 'title', 'keywords', 'description', 'priority', 'public', 'duration', 'attach_file', 'new_field', 'new_date']
>>> form.is_valid()
False
"""
## Riprendo
r"""
>>> from django.newforms.util import ErrorDict
>>> form.errors == {'status': [u'Il campo \xe8 obbligatorio.'], 'priority': [u'Il campo \xe8 obbligatorio.'], 'new_field': [u'Il campo \xe8 obbligatorio.'], 'project': [u'Il campo \xe8 obbligatorio.'], 'keywords': [u'Il campo \xe8 obbligatorio.'], 'submitter': [u'Il campo \xe8 obbligatorio.']}
True
>>> form.__errors = {}
>>> form.is_bound = True
>>> form.is_valid()
False
>>> form = forms.models.form_for_model(Ticket, formfield_callback=lambda f: f.formfield(required=False))(d)
>>> form.data
<MultiValueDict: {'position': ['Developer'], 'new_date': ['2006-12-25'], 'name': ['Adrian', 'Simon']}>
>>> form.is_bound
True
>>> form.fields['new_date'] = DateField(required=False, widget=TextInput(attrs={'size':10,'maxlength':10,'class':'vDateField',}))
>>> form.is_valid()
True

>>> form.data['new_date']
'2006-12-25'
>>> print form['new_date']
<input name="new_date" maxlength="10" id="id_new_date" type="text" class="vDateField" value="2006-12-25" size="10" />
>>> form.errors
{}
>>> form.fields['new_date'].widget.render('date', '2006-12-25')
u'<input name="date" value="2006-12-25" maxlength="10" type="text" class="vDateField" size="10" />'
>>> form.fields['submitter'].widget.render('mittente', value='1')
u'<select name="mittente">\n<option value="">---------</option>\n<option value="1" selected="selected">admin</option>\n</select>'
>>> from django.db.models import fields as db_fields
>>> def check(field):
...     if isinstance(field, db_fields.DateField):
...         return field.formfield(required=False, widget=TextInput(attrs={'size' : 10,'maxlength' : 10,'class' : 'vDateField',}))
...     else:
...         return field.formfield(required=False)
>>> F = forms.models.form_for_model(Ticket, formfield_callback=check)
>>> form = F()
>>> print form['date_create']
<input name="date_create" maxlength="10" id="id_date_create" type="text" class="vDateField" size="10" />
"""

if __name__ == "__main__":
    import doctest
    doctest.testmod()
