import time, datetime

"""utils that in no way are specific of django
ie: they don't require the django framework
"""


class InputStream(object):
    """ Simple Wrapper for File-like objects. [c]StringIO doesn't provide
        a readline function for use with generate_tokens.
        Using a iterator-like interface doesn't succeed, because the readline
        function isn't used in such a context. (see <python-lib>/tokenize.py)
    """
    def __init__(self, data):
        self.__data = [ '%s\n' % x for x in data.splitlines() ]
        self.__lcount = 0
    def readline(self):
        try:
            line = self.__data[self.__lcount]
            self.__lcount += 1
        except IndexError:
            line = ''
            self.__lcount = 0
        return line

class DictWithDefault(dict):
    """dict with default: eg::
    >>> a = DictWithDefault(default=0)
    >>> a['b']
    0
    """
    
    def __init__(self, default=''):
        super(DictWithDefault, self).__init__()
        self.default = default

    def __getitem__(self, name):
        if name not in self.keys():
            return self.default
        else:
            return super(DictWithDefault,self).__getitem__(name)

class MatrixDict(dict):
    """A dict that does not need to define element in a matrix fascion.
    you don't need to define all elements:
    >>> A = MatrixDict()
    >>> A[1] = 5
    >>> A[2][3]['a']=  4
    >>> A[5][6]=  7
    A= {1: 5, 2: {3: {'a': 4}}, 5: {6: 7}}
    
    """

    def __getitem__(self, name):
        if name not in self.keys():
            super(MatrixDict,self).__setitem__(name, self.__class__())
        return super(MatrixDict,self).__getitem__(name)

        
class Dict2Obj(object):
    """handy way of transforming a dictionary into an attribute accessed object
    main use is to get more readeable data:
      data = Attr(self.cleaned_data)
      print data.name, self.cleaned_data['name'], data.get(name)

    """
    def __init__(self, dict):
        self.dict = dict
    def __getattr__(self, name):
        return self.dict[name]
    def get(self, name):
        return self.dict[name]

def zerofill(x, width=2):
    return "%0*d" % (width, x)

def get_formatted_date(date, format="%Y%m%d"):
    for fmt in ("%Y-%m-%d", "%y-%m-%d", "%Y/%m/%d", "%y/%m/%d", 
                 "%d/%m/%y", "%d/%m/%Y", "%d-%m-%y", "%d-%m-%Y"):
        try:
            data = time.strptime(date, fmt)
            data = datetime.date(data[0], data[1], data[2])
            return data.strftime(format)
        except ValueError, e:
            pass


def get_formatted_time(ms):
    ftime= "0s"
    s=ms/1000
    m,s=divmod(s,60)
    h,m=divmod(m,60)
    d,h=divmod(h,24)
    if s > 0:
        ftime = "%ss" % s
    if m > 0:
        ftime = "%sm %s" % (m, ftime)
    if h > 0:
        ftime = "%sh %s" % (h, ftime)
    if d > 0:
        ftime = "%sg %s" % (d, ftime)
    
    return ftime  