#!/usr/bin/env python

"""
This script is executed in the project root.

Its purpose is to import the menu entries stored in each application
as text formatted file -menu.txt-.

Pay attention: you must specify PROJECT name as arg.

'menu.txt' syntax:
=======================================================
NameA_Link_level_0, path, Description, Extra
    NameA1_Link_level_1, path, Description, Extra
    NameA2_Link_level_1, path, Description, Extra
NameB_Link_level_0, path, Description, Extra
NameC_Link_level_0, path, Description, Extra
    NameC1_Link_level_1, path, Description, Extra
    NameC2_Link_level_1, path, Description, Extra
        NameC21_Link_level_2, path, Description, Extra
=======================================================

Extra is the only optional argument.

"""

USAGE ="""
  usage: %prog [options] project
    -a, --app=APPLICATION: limits menu creation to APPLICATION;
    -c, --clear: needs option '-a|--app' - removes menu entries instead of creating them;
    -v, --verbose: prints more information on output;
    -t, --test: calls a simulation to test the script.
    -p, --project=PROJECT: project (or last arg)

"""

import os
import sys
import re
import jungle.scripts.optionparse as options
import unittest

pattern = re.compile('^(?P<offset> *)(?P<data>.+)')
father = [1]
index = 0
prev_level = 0
filename = ''
delete = False
dirs_list = []
list_remove = []
opt = False # options are disabled

def get_or_create_father_menu_entry(): 
    global father
    from jungle.sys.models import Menu
    
    x = Menu.all_objects.get_or_create(name='root', description='dummy entry', path='admin', level=0)
    father = [x[0].id]

def run_for_apps(apps):
    """same format as INSTALLED_APPS eg.: ('jungle.timereport', 'jungle.sys')
    """
    for module in apps:
        tmp = __import__(module, {}, {}, [''])
        if tmp.__file__: elements = tmp.__file__.split('/')
        elements.pop()
        dirs_list.append('/'.join(elements))

    for d in dirs_list:
        print 'Import Menus Running for App: %s' % d.split('/').pop()
        filename = '%s/menu.txt' % d
        try:
            f = open(filename, 'r')
        except IOError, e:
            if opt and opt.verbose:
                print "'menu.txt' not in '%s'" % d
            continue
    
        menu_entries_import(f)

def menu_entries_import(f):
    line_number = 0
    comment_pattern = re.compile('^\s*#')
    whitespaces_pattern = re.compile('^\s*$')

    for line in f.readlines():
        line_number += 1
        # comment
        if comment_pattern.match(line) or whitespaces_pattern.match(line):
            if opt and opt.verbose:
                print "Empty line or comment in line %s." % line_number
            continue

        parse_line(line, line_number)

def parse_line(line, line_number):
    global father, index, prev_level, list_remove

    result = pattern.match(line)
    offset = len(result.group('offset'))
    rest = offset % 4

    if (rest > 0):
        transaction.rollback()
        sys.exit( "Indentation Error (%s spaces) %s:%s\n%s" % (
            offset, filename, line_number, line ) )
        #sys.exit( "Indentation Error on line '%s' -> '%s'" % (line_number, filename) )
    else:
        current_level = offset / 4

        data   = result.group('data')
        elements = data.split(',')

        arguments = len(elements)

        if (arguments == 4) or (arguments == 3):
            if (prev_level + 1 == current_level):
                father.append(index)
                prev_level = current_level
            elif (prev_level > current_level):
                for i in range(prev_level-current_level):
                    father.pop()
                    prev_level -= 1

            if (arguments == 4):
                extra = elements[3].lstrip()
            else:
                extra = ''
        
            index = create_or_remove( entry_name   = elements[0],
                                    entry_path   = elements[1].lstrip(),
                                    entry_father = father[-1],
                                    entry_description = elements[2].lstrip(),
                                    entry_extra       = extra,
                                    entry_level  = current_level+1 )

        else:
            transaction.rollback()
            sys.exit( "Format Error on line '%s' -> '%s'" % (line_number, filename) )

def delete_entries(application_father):
    from jungle.sys.models import Menu
    
    list_remove = [application_father]
    
    results = []

    for querysets in list_remove:
        for element in querysets:
            results = Menu.all_objects.filter( father = Menu.all_objects.get(
                                           id = element.id) )
            list_remove.append( results )

    list_remove.reverse()

    for querysets in list_remove:
        for element in querysets:
            if opt and opt.verbose:
                print "Element '%s' deleted." % element.name
            element.delete()

    transaction.commit()
    sys.exit("Deletion complete")

def create_or_remove( entry_name,
                        entry_path,
                        entry_father,
                        entry_description,
                        entry_level,
                        entry_extra ):

    from jungle.sys.models import Menu

    if delete:
        application_father = Menu.all_objects.filter( name = entry_name,
               father = Menu.all_objects.get(id=entry_father), level = entry_level, )

        delete_entries(application_father)
    
    else:
        try:
            menu_entry = Menu.all_objects.get( name = entry_name,
               father = Menu.all_objects.get(id=entry_father), level = entry_level, )
            if opt and opt.verbose:
                print "Element '%s' exists, skipped." % menu_entry.name
        except Menu.DoesNotExist:
            try:
                menu_entry = Menu.all_objects.create(
                             name = entry_name,
                             path = entry_path,
                             status = 1,
                             father = Menu.all_objects.get(id=entry_father),
                             description = entry_description,
                             level = entry_level,
                             extra = entry_extra )
            except Exception, e:
                print e
                raise
                from django.db import connection
                #print connection.queries[-1]
                sys.exit()

            menu_entry.save()

            if opt and opt.verbose:
                print "Element '%s' created." % menu_entry.name

    return menu_entry.id

#def test():
    #FIXME test has to be done
#    sys.argv = [__file__]
    
#    MENU_FILE ="""Ticket, ticket/, ticket home, orizzontal_menu
#    Aggiungi Ticket, ticket/ticket/add, Crea un nuovo Ticket, orizzontal_menu
#    Lista Ticket, ticket/ticket/list, Vedi tutti i Ticket, orizzontal_menu
#Project, project/, project home, orizzontal_menu
#    Aggiungi Project, ticket/project/add, Crea un nuovo progetto, orizzontal_menu
#    Lista Project, ticket/project/list, Vedi tutti i progetti, orizzontal_menu"""
    
#   unittest.main()

# http://www.djangoproject.com/documentation/faq/#how-can-i-see-the-raw-sql-queries-django-is-running

# BEGIN MAIN
if __name__ == '__main__':

    opt, args = options.parse(USAGE)
    
    if not opt and not args:
        options.exit()
    elif opt.test:
        test()
    else:
        project_settings = opt.project or args[0] 
        if opt.app:
            single_app = True
            if opt.clear:
                delete = True
            else:
                pass
        else:
            single_app = False

    if not project_settings:
        options.exit()

    os.environ['DJANGO_SETTINGS_MODULE'] = '%s.settings' % project_settings

    from django.db import transaction
    transaction.enter_transaction_management()
    transaction.managed()
    try:
        from django.conf import settings
    except EnvironmentError, e:
        sys.exit("Could not find '%s.settings' project." % project_settings)

    get_or_create_father_menu_entry()

    apps = settings.INSTALLED_APPS

    if single_app:
        apps = [ elem for elem in apps if elem==opt.app ]

    if (apps == []) and (opt.app):
        sys.exit("Application '%s' not found.\n\nHint: Are you sure you added it to INSTALLED_APPS in 'settings.py?" % opt.app)

    run_for_apps(apps)
    transaction.commit()
    #transaction.leave_transaction_management()
