#!/usr/bin/env python

"""
This script is executed in the project root when Django manage.py is
generated.

Its objective is to create a module -map.py- containing a dictionary in
which each Django Model has got his import path.

The generated module provides also a dyn_import function that import
the right models.py and return the class object requested.



"""

import sys
import os
import os.path as path

import pyclbr # pyclbr -- Python class browser support


def add_import_modules_map():
    try:
        f = open('settings.py')
    except IOError, e:
        sys.exit("Error reading settings file.")
        
    new = 'from map import MODULES_MAP\n\n' + f.read()
    
    try:
        f = open('settings.py', 'w')
    except IOError, e:
        sys.exit("Error writing settings file.")
    
    f.writelines(new)
    f.close()

dirs = os.getcwd().split('/')
projectName =  dirs[-1]

settings_name = projectName + '.settings'
try:
    settings_module = __import__(settings_name, {}, {}, [''])
except ImportError, e:
    sys.exit("Error loading '%s'." % settings_name)

try:
    INSTALLED_APPS = getattr(settings_module, 'INSTALLED_APPS')
except AttributeError, e:
    sys.exit("'settings.py' probably malformed.")

try:
    modulesMapFile = open('map.py', 'w')
except IOError, e:
    sys.exit("Error creating map file.")
    
modulesMapFile.write(r'''
"""
This module is dynamically generated by modules_map.py script and
provides a dictionary in which each Django Model has got his import
string.



"""
''')

modulesMapFile.writelines('MODULES_MAP = {\n')

for module in INSTALLED_APPS:
    try:
        model = module + '.models'
        mod = pyclbr.readmodule_ex(model)
        if mod.items():
            for classname, descriptor in mod.items():
                modulesMapFile.writelines("\t'%s': {" % classname.lower())
                modulesMapFile.writelines("\n\t\t'import':\t'%s.%s',\n" %
                                  (model, classname))
                modulesMapFile.writelines("\t\t},\n")
    except:
        print "\n'%s' module does not contain Models.\n" % module

modulesMapFile.writelines("\t}")

try:
    MODULES_MAP = getattr(settings_module, 'MODULES_MAP')
except AttributeError, e:
    add_import_modules_map()
    
modulesMapFile.close()
sys.exit( "'map.py' for '%s' project created." % projectName.upper() )