from django.conf import settings
from django.db.models.loading import get_model
from jungle.functions.tools import import_custom
from django.newforms import fields, widgets
from jungle.templatetags.forms import LayoutNewForm, FIELD_NAME_PATTERN
from jungle.forms.fields import JungleDateField, JungleNullBooleanField, JungleNullBooleanFieldRev
from django.db.models import fields as db_fields
from django.db.models.fields.related import *
import django.newforms as forms
from django.newforms import widgets as wid
from datetime import date
import re
from jungle.db import formfield
from jungle.template.mako_loader import get_def_from_template
from django.utils.translation import gettext

OPERATORS = {
              '=': 'exact',     # equal
             'i=': 'iexact',    # equal case insensitive
           'LIKE': 'icontains', # contains case insensitive
             'gt': 'gt',
            'gte': 'gte',
             'lt': 'lt',
            'lte': 'lte',
             'in': 'in',
           'NULL': 'isnull',
           'bool': None,
          'start': 'istartswith',
           'end':  'iendswith',
#             '~' : 'iregex',
#         'regex' : 'regex',
            }

DEFAULT_OPER = 'LIKE'

class MissingOperatorException(Exception):
    pass

class FSOpts(formfield.FormfieldChooser):
    """
    Options to configure how a fast search should display data
    """
    def __init__(self, user=None):
        self.required = False
        self.user = user
        
def mako_fast_search(context, model=None, template_name=None, **kw):

    form     = context.get('form', None)
    user     = context.get('user', None)
    order_by = context.get('order_by', None)
    request  = context.get('request', None)

    try:
        # in mako templates the model name is in the format: <app_label>.<model_name>
        app_label, model = model.split('.')
    except:
        path = request.META['PATH_INFO']
        path = path.replace(settings.LOCATION, '', 1)
        url_pieces = path.split('/')
        app_label  = url_pieces[0]
        model = model or url_pieces[1] 

    if 'app_label' in kw:
        app_label = kw['app_label']
        del kw['app_label']

    return fast_search(model, app_label=app_label, form=form, user=user,
                       order_by=order_by, request=request,
                       template_name=template_name, **kw)

def fast_search(model, app_label=None, layout=None, data=None, render=True,
                order_by='',form=None, name='fast_search', user=None,
                request=None, template_name=None, **kw):

    if not data:
        data={}

    kw['model'] = model
    Model = get_model(app_label, model)
    lfs = LayoutForFastSearch
    
    mod = import_custom('fast_search', app_label)
    try:
        opts = mod.FSOpts(user)
    except AttributeError:
        opts = FSOpts(user)
    
    try:
        lfs = mod.LayoutForFastSearch
    except AttributeError:
        pass
        
    if not form:
        form = forms.models.form_for_model(
            Model, formfield_callback=opts.formfield)()
        if data:
            form.data = data 
            form.is_bound = True
    else:
        form = form
    kw['name'] = name

    if 'action' not in kw:
        kw['action'] = '/%s/%s/list/' % (app_label, model)

    if 'order_by' not in data:
        data['order_by'] = order_by or Model._meta.pk.attname 

    ## data is a MultiDict and **expands them as list
    data_dict = {'request':request}
    for i in data:
        data_dict[str(i)] = data[i]

    t_name = '%s/%s_list.mako' % (app_label, model)
    
    if template_name:
        t_name = template_name
        
    if not layout:
        layout = get_def_from_template(t_name,
                                       data=data_dict)
    
    lay = lfs(layout, form=form, app_label=app_label, data=data, method='GET',
                              popup='', **kw)
    if render:
        return lay.render()
    else:
        return lay

class LayoutForFastSearch(LayoutNewForm):
    def __init__(self, layout, app_label, form, data, **kw):
        self.paths, self.opers = {}, {}

        self.attrs  = []
        self.layout = layout
        self.has_hiddens = False # h=layout ... added just one time

        LayoutNewForm.__init__(self, layout=layout, app_label=app_label, form=form, **kw)

    def table_footer(self, txt):

        if not self.has_hiddens:
            #LayoutNewForm.hidden(self, 'layout=%s' % layout_base64)
            LayoutNewForm.hidden(self, 'model=%s' % self.model_name)
            LayoutNewForm.hidden(self, 'page=1' )
            #LayoutNewForm.hidden(self, 'action=%s' % self.action)

        self.has_hiddens = True
        txt += ['</table>']
        return txt

    def to_html(self, name):
        if isinstance(self.form.fields[name].widget, widgets.Textarea):
            return self.form[name].as_text()
        else:
            return self.form[name]
    
#### row parsing
    def item(self, name, level=0):
        """each name il split in 3 parts: field spec, filter spec, operator"""

        parts = name.split('.')
        if len(parts) == 1:
            ## submit
            if parts[0].startswith('s=') or parts[0].startswith('b='):
                return LayoutNewForm.item(self, parts[0], level=level)

        m = re.match(FIELD_NAME_PATTERN, name)
        wdg   = m.group('wdg')   # the html widget
        field = m.group('name')  # the name of the field

        field_spec = parts[0]
        try:
            filter_path = parts[1] or field
        except IndexError:
            filter_path = field
        
        if field.startswith('_'):
            field = field.lstrip('_')
            field_spec = "%s=%s" % (wdg, field)
        else:
            self.attrs.append(field)


        ## filter paths are composed of a field_name and a __operator
        self.paths[field] = filter_path
        #if wdg == 'h':
        #    import pdb; pdb.set_trace()
        method = "widget_%s" % wdg
        returned_widget = getattr(self, method, lambda w, f, p, l : False)(
            wdg, field, parts, level)
        if returned_widget:
            return ''
            
        ## normal items we need to add a 'path' and an 'operator' hidden field
        self.form.fields['path_%s' % field] = fields.CharField(required=False)
        LayoutNewForm.item(self, 'h=path_%s=%s' % (field, filter_path), level=level)


        try:
            oper = self.opers[field] = parts[2]
        except:
            oper = self.opers[field] = 'LIKE'

        self.form.fields['oper_%s' % field] = fields.CharField(required=False)
        LayoutNewForm.item(self, 'h=oper_%s=%s' % (field, oper), level=level)

        try:
            return LayoutNewForm.item(self, field_spec, level)
        except KeyError, e:
            self.form.fields[field] = fields.CharField(required=False)
            return LayoutNewForm.item(self, field, level)

    def widget_h(self, wdg, field, parts, level):

        ## hidden fields do not need any particular handling
        #if re.match('(h=)?order_by|mode|paginate_by', field):
        field, value = field.split('=')
        self.form.fields[field] = fields.CharField(required=False)
        LayoutNewForm.item(self, parts[0], level=level)
        return True
        
    def widget_D(self, wdg, field, parts, level):
        self.form.fields[field] = JungleDateField(required=False, )

    def widget_T(self, wdg, field, parts, level):
        self.form.fields[field] = fields.TimeField(required=False,
                                                   widget=wid.TextInput())

        #import pdb; pdb.set_trace()
        ## NullBooleanSelect
    def widget_nbs(self, wdg, field, parts, level):
        self.form.fields[field] = JungleNullBooleanField(required=False)
        self.form.fields[field].widget.choices = ((u'1', '---'),
                                                  (u'2', gettext('Yes')),
                                                  (u'3', gettext('No')),
                                                  (u'4', gettext('NULL')),)

    def widget_rnbs(self, wdg, field, parts, level):
        ## Reversed NullBooleanSelect
        self.form.fields[field] = JungleNullBooleanFieldRev(required=False)

    def widget_c(self, wdg, field, parts, level):
        ## checkbox
        self.form.fields[field] = fields.BooleanField(required=False)

    

    def search(self):
        """return filter condition suitable for list view
        it is called from fast_search view to create the args to QuerySet.filter()
        """
        order_by = ''
        data = {}

        ## self.item() has triggered form.full_clean() when the form did not
        ## already have all fields, so we run it again
        self.form.full_clean()
        if self.form.is_valid():
            data = self.form.cleaned_data
        where_dict = {}
        for elem in self.attrs:
            # if is not an empty value
            if ((elem in data) or ('path_%s' % elem in data)) and \
                                data[elem] not in ('', [], None):
                self.cond2dict(
                    (self.paths[elem], self.opers[elem], data[elem]), where_dict)
        return where_dict

    def cond2dict(self, cond, where_dict):
        """transforms a tuple representing a SQL condition into a dictionary
           suitable for QuerySet.filter() syntax::

        ('date', '>', '1/1/07')  ==>  {'date__gt': '1/1/07'}

        """
        #import pdb; pdb.set_trace()
        filter_path, oper, value = cond
        if oper in OPERATORS:
            key = "%s__%s" % (filter_path, OPERATORS[oper])
            ## special case for boolean filtering: True/False/isnull
            if oper == 'bool':
                if value == 'NULL':
                    where_dict[str("%s__isnull" % filter_path)] = True
                else:
                    where_dict[str(filter_path)] = bool(value)
            else:
                ## filter conditions myst be str non unicode!
                if oper == 'NULL':
                    value = bool(value)
                where_dict[str(key)] = value
        else:
            raise MissingOperatorException
        return where_dict

