# -*- coding: utf-8 -*-
#### docstring
"""
this module provides `create_form` a function that understand a minimal
description language to assemble html code for templates

in templates a tipical example should be::

  #import jungle.function.forms
  #def layout

     # General data
     first_name
     last_name
     # Work
     Organization
     Level

     s=submit
  #end def layout

$create_form($layout, $form)

Language description
====================

Rows prepended by '#='  sign will be turned into 'legend'
Rows prepended by '#'   sign will be turned into <h2> title in a fieldset
Rows prepended by '%'   sign will be turned into <h2> in the parent fieldset
s=text will become a submit input, s= will be stripped the remaining
       will be passed to gettext. A 'class="submit"' is added.
h=name=value a hidden field, with name="name" value="value"

For each further token a pair of label/input is created following the same
css suggestions as for admin section. 

Needed fields are marked `required` in the label (class="required").

Permission to edit/view the field will be checked (when permission framework
will be worked out.... ;-). 

bypassing defaults
------------------
It is possible to bypass defaults writing attributes of Layout Object as follows:
  #set $f = $create_form($layout, $form, render=False)
  $f.value.first_name = "<a href='...'>my text</a>"
  $f.help.first_name  = "this will show up as a nice baloon"
  $f.html.last_name  = '<td>label</td><td>value</td>'
"""

import re, os
from django.conf import settings
from jungle.templatetags.jungle_tags import ugettext as _
try:
    from django.db.models.fields import FileField
except ImportError:
    from django.db.models import FileField
from django.db.models.fields.related import ManyToManyField
from django.db.models.loading import get_model
import datetime
from django.template import  defaultfilters as filters
try:
    from django import newforms as forms
    from forms.util import smart_unicode
except ImportError:
    from django import forms
    from django.utils.encoding import smart_str, smart_unicode

from django.db.models.fields.related import ForeignKey
from jungle.templatetags.jungle_tags import smart_uni

FIELD_NAME_PATTERN = re.compile(r'''
   (?:(?P<wdg>\w+)=)?  # a possible modifier: D= M= TX=
   (?P<token_value>
   (?P<name>[^:.]*)        # the name of the field up to the first ':'
   (?::                    # after possible ':' dimentions 
     (?P<h>\d*)            # h = height
     (?:[.x]               # a dot or an 'x'
        (?P<w>\d*))?)?     # w = width
   .*?$)
''', re.VERBOSE)

class MalformedHiddenInput(Exception): pass
class MissingModelOrForm(Exception): pass
class UnbalancedLayoutCurlyBaces(Exception): pass

def create_form(layout, form, render=True, **kw):
    lay = LayoutOldForm(layout, form, **kw) 
    if render:
        return lay.render()
    else:
        return lay

def mako_create_form(context, layout, form=None, model=None,render=True,**kw):
    from mako.runtime import capture
    layout =  capture(context, layout) 
    return create_form(layout, form=form, render=render,**kw)

def prova(context, func):
    from mako.runtime import capture
    txt =  capture(context, func) 
    return txt

def create_nform(layout, form=None, app_label=None, model=None, render=True, **kw):
    if not form and not model:
        raise MissingModelOrForm
    
    lay = LayoutNewForm(layout, form=form, app_label=app_label, model=model, **kw) 
    if render:
        return lay.render()
    else:
        return lay

def mako_create_nform(context, layout, form=None, model=None, render=True,**kw):
    from mako.runtime import capture
    layout =  capture(context, layout)
    request =  context.get('request', None)

    try:
        # in mako templates the model name is in the format: <app_label>.<model_name>
        app_label, model = model.split('.')
    except:
        path = request.META['PATH_INFO']
        path = path.replace(settings.LOCATION, '', 1)
        url_pieces = path.split('/')
        app_label  = url_pieces[0]
        model = model or url_pieces[1]

    if 'app_label' in kw:
        app_label = kw['app_label']
        del kw['app_label']

    request = context.get('request')
    if 'popup' in getattr(request, request.method):
        if layout:
            layout += "h=popup=True" 
    return create_nform(layout, form=form, app_label=app_label, model=model,render=render,**kw)

#create_form = create_nform
def table_data(layout, obj, render=True, **kw):
    lay = TableData(layout, obj, **kw)
    if render:
        return lay.render()
    else:
        return lay
    
def mako_table_data(context, layout_func, obj, render=True, **kw):
    from mako.runtime import capture
    layout =  capture(context, layout_func) 
    lay = TableData(layout, obj, **kw)
    if render:
        return lay.render()
    else:
        return lay
    

class Info(object): pass
#     def __getattr__(self, name):
#         if name in self.keys():
#             return _(super(Info, self).__getattr__(name))
#         else:
#             return super(Info, self).__getattr__(name)
                     
class Layout(object):
    def __init__(self, layout, form=None, extra=None, conf=None, action='',
                 name='', method='POST', style='-', popup='*', cls='form'):

        self.popup = self.parse_popup(popup)
        self.action = action
        self.method = method
        self.cls = cls
        self.html = Info()     # an html that will substitute label+value
        self.help = Info()     # any possible help, will be rendered in a balloon
        self.value = Info()    # the value part

        if re.search('^\s+$', layout):
            self.layout = self.create_default_layout()
        else:
            self.layout = layout

        ## colspan depends on style
        self.style = style
        if style == "-":
            self.delta = 2
        else:
            self.delta = 1

        self.name = name
        self.fieldsets = {}
        self.hidden_fields = []
        self.legends = {}

        self.enctype = False
    def parse_popup(self, popup):
        if isinstance(popup, str):
            popup = popup.split(',')
        return popup
        
    def parse_layout(self, lay):
        """return a list of lists. The outer list corresponds to all fieldsets.
        inner lists are the single tokens that build up the lists

        """
        lay = lay.strip("\n \t")
        lvl = 0
        comment = 0
        where = u'out'
        ret = []
        nested = u''
        fieldsets = []
        header = 0
        i = 0
        ## mako will strip comment of the form '##'
        for c in lay:
            i += 1

            # headers may be: #, #=, %
            if c in ('#', '%', '-'):
                ## fieldset start if #=, # or --
                if c == '#' or (c == '-' and len(lay)> i and lay[i] == '-'):
                    if ret:
                        fieldsets += [ret]
                        ret = []
                if c in ('#', '%'):
                    header = 1
                    header_str = ''
            if c in '\n':
                try:
                    ret += [header_str]
                    del header_str
                except:
                    pass
                header = 0

            if header == 1:
                header_str += c
                continue
            # entering a nested
            if c in '{':
                lvl +=1
            # lowering the nested level
            if c in '}':
                lvl -=1
            if  lvl >= 1:
                nested += c
            if lvl == 0:
                if c in (' ','\t','@','\n'):
                    if len(nested) > 0:
                        ret += [nested]
                        nested = ''
                    if c in ('\n','@'):
                        ret += ["\n"]
                else:
                    nested += c

        if len(nested) > 0:
            ret += [nested]

        if lvl != 0:
            raise UnbalancedLayoutCurlyBaces(lay)

        ## transform any default widget (those w/o modifier '[a-Z]+=') into
        ## L=name e=name. This happens if '-' is in the options
        if ret and ret is not ['\n']:
            fieldsets += [ret]
        return fieldsets
        #if True:
        if '-' in self.opts:
            new_ret = []
            for el in ret:
                if re.match('([a-zA-Z]+)=|[%-^\n@{]', el):
                    new_ret += [el]
                else:
                    pattern = re.compile(r"""
                      (?P<val>[^/:]*)         # value
                      (?:/(?P<var>[^:]+))?    # variable
                      (?::(?P<w>[-\d]*))?        # width
                      .?
                      (?P<h>[-\d]+)?           # height
                    """, re.VERBOSE)
                    m = pattern.search(el)
                    if m:
                        new_ret += [ 'L=%s:%s' % (m.group('val') or '',
                                                  m.group('w') or '')]
                        new_ret += [ 'ae=%s:%s' % (m.group('val') or '',
                                                  m.group('h') or '')]

            ret = new_ret
        return ret
    def fieldset(self, definition):
        """each fieldset is split into lines... each line is worked
        self.fieldsets[] holds the text
        """
        txt = ['<fieldset class="%s">' % self.cls]
        self.table_header(txt)
        line = [] 
        for tk in definition:
            if tk == '--':
                continue
            if tk == '\n':
                txt += [self.parse_row(line)]
                line = []
            else:
                if re.match('^[%#]', tk):
                    self.legend(tk)
                else:
                    line.append(tk)

        if line:
            txt += [self.parse_row(line)]

        self.table_footer(txt)
        txt += ['</fieldset>']

        if self.n_fieldset in self.legends:
            txt.insert(1, self.legends[self.n_fieldset])

        self.fieldsets[self.n_fieldset] = u"\n".join(txt)
    
    def parse_row(self, line_list):
        """handle each row, split into tokens 
        """
        #line_list = re.split('\s+', line.strip(' '))
        ## FIXME
        self.colspan = [1] * (len(line_list) )
        txt = []
        ## handle span
        i=0
        for token in line_list:
            if token == '-':
                self.colspan[i-1] += self.delta
                continue
            i += 1

        ## create input types
        i = 0
        for token in line_list:
            if token == '-':
                continue
            token_str = self.item(token, level=i)
            if token_str:
                txt += [token_str]
            i += 1
        rdr = self.render_row(u"\n".join(txt))
        return self.offset_txt(rdr,  0)  #2
    
    def render_row(self, content):
        """this is supposed to be overloaded """
        return content
    
    def item(self, name, level=0):
        """this is supposed to be overloaded """

        ## let's grant the possibility to add an html instead of
        ## normal output
        if name in dir(self.html):
            return getattr(self.html, name)
        return name
    
    def table_header(self, txt):
        """this is supposed to be overloaded """
        pass
    
    def table_footer(self, token):
        """this is supposed to be overloaded """
        pass
    def parse_fieldsets(self):
        self.n_fieldset = 0
        if 'layout' in dir(self):    # TableData does not have layout...
            for f in self.parse_layout(self.layout):
                self.n_fieldset += 1
                self.fieldset(f)
                
    def render_td(self, name, info, label=True):
        if name in dir(self.html):
            return getattr(self.html, name)

        if self.style == '=':
            ROW = u'<td colspan="%(colspan)s">%(label)s: %(errors)s <br>%(wdg)s %(js)s</td>\n'
        else:
            ROW = ""
            if info['label']:
                if label:
                    ROW += u'<td>%(label)s: %(errors)s</td>\n'
                ROW += u'<td colspan="%(colspan)s"> %(wdg)s %(js)s</td>\n'
            else:
                if label:
                    ROW += u'<td>%(errors)s</td>\n'
                ROW += u'<td colspan="%(colspan)s"> %(wdg)s %(js)s</td>\n'

        ret = self.offset_txt(ROW % (info),  0) #2
        return ret

        
    def offset_txt(self, txt, amount, space=" "):
        ## offset text to get a prettier output
        p = re.compile("^", re.MULTILINE)
        return p.sub(space * amount, txt)
    
    def render(self):
        """join the fieldsets and add form specific parameters
        """
        self.parse_fieldsets()
        txt = []
        for i in range(1, self.n_fieldset+1):
            txt += [self.fieldsets[i]]
        rows = "\n".join(txt)

        info = {
            'rows' : self.offset_txt(rows, 0),  #2
            'method' : self.method,
            'enc'    : self.encdata_definition(),
            'action' : self.action,
            'hidden_fields' : self.offset_txt(u"\n".join(self.hidden_fields), 0), #2
            'name'   : self.name,
            }
            
        ret = u'<form id="jform" name="%(name)s" action="%(action)s" ' + \
              '%(enc)s method="%(method)s">\n%(rows)s   \n%(hidden_fields)s\n</form>'
        return ret % info 
    
#### MISC
    def create_default_layout(self):

        HIDE = ('id', 'date_create', 'date_last_modify')
        field_names = [f.name for f in self._meta.fields if f.name not in HIDE]
        legend = "#= tabella: %s" % (self._meta.db_table)
        if not isinstance(self, TableData):
            more = ['--', 's=send']
        else:
            more = []
        layout = '\n'.join([legend] + field_names + more)
        return layout
    def encdata_definition(self):
        pass
    
    def legend(self, text=''):
        "Add a legend if #=, an h2 title if # or % is "
        if text.startswith('#='):
            self.legends[self.n_fieldset] =  u"<legend><a name='%s'>%s</a></legend>" % (text.lstrip('#='), text.lstrip('#='))
        else:
            m = re.match('^[#%] ?(.*)', text)
            if m:
                self.legends[self.n_fieldset] =  u"<h2><a name='%s'>%s</a></h2>" % (m.group(1), m.group(1))
        return 
    
    def to_html(self, name):
        return self.form[name]

class LayoutNewForm(Layout):
#     def __init__(self, layout, form=None, model=None, data={}, clone=None, name='', extra=None, conf=None,
#                  action='', style='-',  method='POST', popup=None):
    def __init__(self, layout, form=None, app_label=None, model=None, clone=None, **kw):
        """
        style: -|=. - display label on the same lavel of the widget
        """
        
        self.clone  = clone
        self.model_name = model
        self.app_label = app_label
        if not form:
            if not model:
                raise MissingModelOrForm
            self.model = get_model(self.app_label, self.model_name)
            self.form = forms.form_for_model(self.model)()
        elif self.clone:
            from jungle.db.formfield import ffc
            self.model = self.clone.__class__
            self.form = forms.form_for_instance(self.clone, formfield_callback=ffc())()
        elif hasattr(form, '_model'):
            self.model_name = form._model._meta.module_name
            self.model = get_model(self.app_label, self.model_name)
            self.form = form
        else: # form is not from a model
            self.model_name = None
            self.form = form

        if self.model_name:
            self._meta = self.model._meta

        if re.search('\S+', layout):
            self.layout = layout
        else:
            self.layout = self.create_default_layout()

        super(LayoutNewForm, self).__init__(layout, form=form, **kw)

#### Widget
    def label(self, name, level=0):
        class_list = []
	
        field = self.form.fields[name]
        if field.label is u'':  ## field.label = None can be used as a way not to return
            return None

        title = getattr(self.help, name, u'') or getattr(field, 'help_text', u'')
        if title:
            ## title="text" cannot ahve \", protecting them doesn't seem to
            ## work properly to me...
            
            title = _(title)
            title = u'title="%s"' % (title.replace('"',"'"))
            class_list.append('help_text')
	
        if field.required:
            class_list.append('required')
            
        if class_list:
            class_str = 'class="%s"' % " ".join(class_list)
        else:
            class_str = ''


        label = None
        if field.label:
            label = _(field.label)

        return  u'<label for="id_%s" %s %s>%s</label>' % \
                (name, title, class_str,
                ( label or _( name.capitalize().replace('_', ' ') )))
    
    def submit(self, name, level=0):
        ## FIXME: non sto usando il livello per mettere piu submit in una riga
        if not hasattr(self, 'reset'):
            reset = u'<input type="reset" style="margin-left: 20px;">'
            self.colspan[level] = self.colspan[level] + 1
            self.reset = True
        else:
            reset = ''
            
        txt = u'%s%s' % ( getattr(self.value, name,
                        u'<input type="submit" class="submit" value="%s" name="%s">'
                        % (_(name.replace('_',' ')), name) ), reset )
	
        return txt
    
    def hidden(self, name):
        """hidden fields are hold in dict 'hidden_fields' and rendered at last"""
        m = re.match('(?P<name>[^=]+)=(?P<value>.*)', name)
        #m = re.match('(?P<name>\w+)=(?P<value>\w+)', name)

        if m:
            name = m.group('name')
            value = m.group('value')
        else:
            raise MalformedHiddenInput(name)

        self.hidden_fields += [u'<input type="hidden" name="%s" value="%s">' % \
                   (name, value)]
        return []
    

#### row parsing
    def item(self, token, level=0):

        #import pdb; pdb.set_trace()
        attrs = {}
        if token in dir(self.html):
            return getattr(self.html, token)

        ## eg.: T=description:80x10 o T=descr:80 T=descr:.10 T=descr:80.10
        wdg = ''
        
        m = re.match(FIELD_NAME_PATTERN, token)
        if m:
            wdg   = m.group('wdg')
            name  = m.group('name')
            token_value  = m.group('token_value')
            try:
                height= m.group('h')
                width = m.group('w')
                if width:
                    attrs = {'size': int(width)}
            except Exception, e:
                pass

        ## submit INPUT
        if wdg == 's':
            info = {
                'label' : '',
                'js': '',  
                'errors' : '',
                'wdg' :  self.submit(name, level=level),                
                'colspan' : self.colspan[level]
                }               
            return unicode(self.render_td(name, info, label=False))
        
        ## button INPUT
        if wdg == 'b':
            info = {
                'label' : '',
                'colspan' : self.colspan[level],
                'errors' : '',
                'js': '',
                'wdg' :  "<input type='button' id='%s' value='%s'/></td>" % (_(name.lower()), _(name))
                }                        
            return unicode(self.render_td(name, info, label=False))            
        
        try:
            f = self.form.fields[name]
            ## if we know the model_field related we can use it to get more info
            try:
                model_field = self._meta.get_field(name)
            except:
                model_field = None

            if height and isinstance(f.widget, forms.SelectMultiple):
                f.widget.attrs.update({'size':height})
            if isinstance(f, forms.FileField):
                self.enctype = True
                wdg = unicode(self.form[name]) + unicode(self.form["%s_file" % name])
        except:
            f = None

        ## widgets
        if wdg == 'h':
            wdg_html = self.hidden(token_value)
            return
        
        if wdg == "M":
            has_admin_ManyToMany_widget = True
        else:
            has_admin_ManyToMany_widget = False
	
        js_txt = '' # useful to add some javascript for particular widgets
        if has_admin_ManyToMany_widget:
            if model_field:
                if isinstance(model_field, ManyToManyField):
                    js_txt = u'<script type="text/javascript">addEvent(window, "load", function(e) {' \
                              ' SelectFilter.init("id_%s", "%s", %s, "%s"); });</script>\n' % (
                                    model_field.name, model_field.verbose_name.replace('"', '\\"'),
                                    (model_field.rel.filter_interface or 1)-1, settings.ADMIN_MEDIA_PREFIX)
	
        value = getattr(self.value, name, u'')
        try:
            errors = self.form[name].errors
        except KeyError, e:
            #raise e
            errors = ''

        if not value:
            if wdg == 't':
                wdg_html = self.form[name].as_text(attrs=attrs)

            elif wdg == 'T' or (f and isinstance(f.widget, forms.Textarea)):
                attrs = {'rows': height or 0, 'cols' : width or 0}
                wdg_html = self.form[name].as_textarea(attrs=attrs)
            elif wdg == 'l':
                label = ''
                wdg_html = u"<span class='label_header'>%s</span>" % _(name)
            else:
                if f:
                    f.widget.attrs.update(attrs)
                wdg_html = self.to_html(name)
                try:
                    fdb = self._meta.get_field(name)
                    if isinstance(fdb, (ForeignKey,ManyToManyField)):
                        wdg_html = self.make_fk_popup_link(fdb.rel.to._meta, name)
                except:
                    pass
#                 else:
#                     wdg_html = self.form[name]
        if not 'label' in locals():
            label = self.label(name)

        info = {
            #'label' :  self.form[name].label_tag(),
            'label'  : label,
            'errors' : errors,
            'colspan' : self.colspan[level],
            'wdg' :  value or wdg_html,
            'js': js_txt,
            }

        return unicode(self.render_td(name, info))
    
    def make_fk_popup_link(self, meta, name):
        """return a link suitable to popup a window to add an entry in the fk table

        there may be several rules to decide if a link must appear:
        1. a popup arg (a list) is passed to make_nform
        2. a 
        """

        if name not in self.popup and '*' not in self.popup:
            return unicode(self.form[name])
        ## user should never be added this way. FIXME: should be configurable
        if meta.module_name == 'user' and meta.app_label == 'auth':
            return unicode(self.form[name])
            

        url = "/%s/%s/add/?popup=True" % (meta.app_label, meta.module_name)
        opts = "top=200, left=200, width=600, height=400, status=no, menubar=no, " + \
               "toolbar=no, resizeable=yes, scrollbar=yes"
        onclick = """javascript:window.open('%s', '', '%s');return false;""" % (
            url, opts)

        info = {
            'wdg' : unicode(self.form[name]),
            'img' : '/static/img/icon_addlink.gif',
            'onclick' : onclick,
            }
        wdg_html = u'%(wdg)s <a href="#" onclick="%(onclick)s"><img src="%(img)s" alt="Add"></a>' 
            
        return wdg_html % info
        
    def table_header(self, txt):
        """this is supposed to be overloaded """
        txt += ['<table class="%s">' % self.cls]
        return txt
    
    def table_footer(self, txt):

        txt += [u'</table>']
        return txt
    
    def render_row(self, content):
        return self.offset_txt(u'<tr>\n%s</tr>\n' % (content),0) #2
    
        
    def encdata_definition(self):
        if self.enctype:
            return u'enctype="multipart/form-data"'
        return u''
    

class TableData(LayoutNewForm):
    """this is meant to show data of an object
    No particular handling is performed apart:

      * label is translated
      * field value is substituted
      * dates will be formatted as per django.settings
      * permission to read this field is checked against group/row perm (not yet)
      
    """
    def __init__(self, layout, obj, extra=None, conf=None,
                 action='',
                 method='POST', style='-'):
        #self._meta = form.manipulator.model._meta
        self.obj = obj
        self._meta = obj._meta
        Layout.__init__(self,  layout, form=None, extra=None, conf=None,
                 action='',
                 method='POST', style=style)
    
    
    def render(self):
        """join the fieldsets and add form specific parameters
        """
        self.parse_fieldsets()
        txt = []
        for i in range(1, self.n_fieldset+1):
            txt += [self.fieldsets[i]]
        return self.offset_txt("\n".join(txt), 0) #2
    
    def item(self, name, level=0, Type=None):

        m = re.match('RST=(?P<name>.*)', name)
        if m:
            from jungle.templatetags.jungle_tags import ReST
            name = m.group('name')
            if not getattr(self.value, name, u''):
                field_value = ReST(getattr(self.obj, name))
        else:


            if name in dir(self.html):
                return getattr(self.html, name)
            ## let's handle submit INPUT
            if name.startswith('s='):
                raise SyntaxError("no submit button in TableData")

            elif name.startswith('h='):
                raise SyntaxError("no hidden input in TableData")

            ## field_value may be a value or a ManyRelatedManager object
            field_value = getattr(self.obj, name, None)

            if field_value is None:
                field_value = ''
            elif field_value is False:
                field_value = 'NO'
            elif field_value is True:
                field_value = 'SI'



        ## f is the field as defined in the model
        ## eg: CharField or ManyToManyField
        value =  getattr(self.value, name, u'')
        if not value:
            f = self.obj._meta.get_field(name)
            internal_type = f.get_internal_type()
            if internal_type == 'Date':
                field_value = filters.date(field_value) or u''
            elif internal_type == 'DateTime':
                field_value = filters.datetime(field_value) or u''
            elif internal_type == 'Time':
                field_value = filters.time(field_value) or u''
            elif internal_type == 'ManyToManyField':
                field_value = u' '.join([unicode(i) for i in field_value.all()])
            else:
                try:
                    field_value = getattr(self.obj, 'get_%s_display' % name)()
                except:
                    pass
        try:
            verb_name = smart_unicode(_(f.verbose_name.capitalize()))
        except:
            verb_name = _(name.capitalize())
        info = {
            ## FIXME: meglio usare css...
            'label' :  u'<span class="label"><b>%s</b></span>' % verb_name,
            'errors' : u'',
            'colspan' : self.colspan[level],
            'wdg' :  value or smart_uni(field_value) or u'',
            'js': u'',
            }
        return self.render_td(name, info)
        

