"""
the relevant structure of the sys_menu table will be:

CREATE TABLE sys_menu (
  menu_id     PRIMARY KEY SEQUENCE,
  name        varchar(20),
  path        varchar(255),
  description varchar(250),
  status    integer,
  father    integer,
  level     integer,
  extra     varchar(250),
);
"""
from jungle.sys.models import Menu
import re
from django.conf import settings
from menu_sd import create_sdmenu

root_father_id = 1

def mako_create_menu(context, **kw):
    request = context.get('request', '/')
    return create_menu(request.path, **kw)

def mako_create_hmenu(context, **kw):
    current_path = context.get('path', '/')

    return create_hmenu(current_path, **kw)

def create_hmenu(current_path, father=None, levels_to_show=1, mode='vertical', level_to_start=0, order_by='id', public=False):
    global root_father_id
    try:
        LOCATION = settings.LOCATION
    except:
        LOCATION = '/'

    father_id = root_father_id = -1

    r = Menu.objects.filter(name='root', description='dummy entry', level=0)
    if r:
        father_id = root_father_id = r[0].id
    
    code = ['<ul id="treemenu">\n']
    
    return gen_hlist(father_id, current_path.replace(LOCATION, '/', 1),
            levels_to_show, code, order_by, public=public) + '</ul>\n'    

def gen_hlist(father_id, current_path, level, code, order_by, active='',
              public=False):
    """
    Called by 'create_menu' this function creates the menu list and its
    possible sublists.
    """
    lst = Menu.objects.filter(father__exact = father_id, status = 1).order_by(order_by)

#     ## verifica che l'utente autenticato abbia i permessi per visualizzare i menu
    from jungle.middleware.thread_local import get_current_user
    user = get_current_user()
    if not public and not user.is_authenticated():
        return ''.join(code)
    
#     if not user.is_staff:
#         perms = user.get_all_permissions()
#         list_perms=[]
#         for perm in perms:
#             if perm.startswith('sys.'):
#                 list_perms.append(perm.lstrip('sys.').replace('.', '/'))
#         ##list_perms += ['Agenda', 'Concessione', 'Ricerca Concessioni', 'Ricerca Appuntamenti']
#         lst = lst.filter(name__in=list_perms)

    id_current = ''
    href_class = ''
    li_class = ''
    for element in lst:
        if current_path.strip('/').startswith(element.path.strip('/')):
            id_current = active
        else:
            id_current = ''
        
        if father_id == root_father_id:
            href_class = ' class="mainfoldericon"'
            li_class = ' class="mainli"'
        # links are generated with '//' prefix for add_location
        code.append('<li%s><a href="//%s" title="%s"%s><span>%s</span></a>\n' %
                (li_class, element.path, element.description, href_class, element.name))
        
        child_lst = Menu.objects.filter(father__exact = element.id, status = 1).order_by(order_by)

        if len(child_lst) > 0:
            code.append('<ul>\n')
            gen_hlist(element.id, current_path, level, code, order_by,
                      active='', public=public)
            code.append('</ul>\n')
        else:
            code.append('</li>\n')

    return ''.join(code)
            
def create_menu(current_path, father=None, levels_to_show=1, mode='vertical', level_to_start=0, order_by='id'):
    """
    This method is directly called by the mako template in which you
    intend to generate the menu.
    By default it is able to build a list menu from your root menu entries
    (one level) using our standard css 'vertical' class.
    Of course you can choose how many levels to print, the css class to use
    and which is the menu father to select the entries in
    'jungle.sys.models.Menu'.

    'gen_list' is the function in which the list and its possible sublists
    are created.
    """

    try:
        LOCATION = settings.LOCATION
    except:
        LOCATION = '/'

    father_id = 1
    sons = []
    sons_indexes = {}
    sons_numbers = []
    if father.isdigit():
        father_id = int(father)
    else:
        sons = Menu.objects.filter(
           path__istartswith = father.strip('/'),
           level__exact = level_to_start )

    # It was a FIXME: Now It's more precise then the previous one
    sons_numb = len(sons)
    if sons_numb > 0:
        if sons_numb == 1:
            father_id = sons[0].father
        else:
            count = 0
            for son in sons:
                tmp_sons = Menu.objects.filter(
                    father = Menu.objects.get(id=son.id),
                    level__exact = level_to_start+1)
                how_many = len(tmp_sons)
                sons_indexes[how_many] = count
                sons_numbers.append(how_many)
                count+=1
            Max = max(sons_numbers)
            father_id = sons[sons_indexes[Max]].father

    code = ['<div id="navcontainer" class="%s">\n<ul id="navlist">\n' % mode]
    
    return gen_list(father_id, current_path.replace(LOCATION, '/', 1),
            levels_to_show, code, order_by) + '</ul>\n</div>\n'

def gen_list(father_id, current_path, level, code, order_by, active=' id="active"'):
    """
    Called by 'create_menu' this function creates the menu list and its
    possible sublists.
    """
    lst = Menu.objects.filter(father__exact = father_id, status = 1).order_by(order_by)
#     ## verifica che l'utente autenticato abbia i permessi per visualizzare i menu
#     from jungle.middleware.thread_local import get_current_user
#     user = get_current_user()
#     if not user.is_authenticated():
#         return ''.join(code)
    
#     if not user.is_staff:
#         perms = user.get_all_permissions()
#         list_perms=[]
#         for perm in perms:
#             if perm.startswith('sys.'):
#                 list_perms.append(perm.lstrip('sys.').replace('.', '/'))
#         ##list_perms += ['Agenda', 'Concessione', 'Ricerca Concessioni', 'Ricerca Appuntamenti']
#         lst = lst.filter(name__in=list_perms)

    id_current = ''
    for element in lst:
        if current_path.strip('/').startswith(element.path.strip('/')):
            id_current = active
        else:
            id_current = ''
        # links are generated with '//' prefix for add_location
        code.append('<li class="menu-entry"%s><a href="//%s" title="%s"><span>%s</span></a></li>\n' %
                (id_current, element.path, element.description, element.name))
        if id_current != '' and level > 1:
            level=level-1
            code.append('<li><ul id="subnavlist">\n')
            gen_list(element.id, current_path, level, code, order_by, active=' id="subactive"')
            code.append('</ul></li>\n')

    return ''.join(code)

def get_fatherid(current_path):
    father_id = 0

    path_elements = current_path.split('/')
    # deleting initial slash
    for n in range(len(path_elements)):
        if (path_elements[0] == ''):
            path_elements.pop(0)

    candidates_all = Menu.objects.all().order_by('father')
    tmp_candidates = []

    while (1):
        tmp_path = '/'.join(path_elements)
        candidates = candidates_all.filter(path__istartswith = tmp_path )

        results = len(candidates)
        
        if results > 1:
            if tmp_candidates == []:
                tmp_candidates = candidates
            if path_elements == []:
                break
            else:
                path_elements.pop()
        elif results == 0:
            if path_elements == []:
                break
            else:
                path_elements.pop()
        else: #results == 1
            break

    if results == 1:
        father_id = candidates[0].father
    elif results > 1:
        father_id = candidates[results/2 -1].father
    else:
        try:
            father_id = tmp_candidates[len(tmp_candidates)/2 -1].father
        except IndexError:
            father_id = 1

    return father_id
