"""
the relevant structure of the sys_menu table will be:

CREATE TABLE sys_menu (
  menu_id     PRIMARY KEY SEQUENCE,
  name        varchar(20),
  path        varchar(255),
  description varchar(250),
  status    integer,
  father    integer,
  level     integer,
  extra     varchar(250),
);

<ul id="navmenu">
  <li><a href="">Blog</a></li>

  <li><a href="">Work +</a>
    <ul>
      <li><a href="">Websites +</a>
        <ul>
          <li><a href="">qrayg</a></li>
          <li><a href="">qArcade</a></li>
          <li><a href="">qLoM</a></li>

          <li><a href="">qDT</a></li>
        </ul>
      </li>
      <li><a href="">Pen and Ink</a></li>
      <li><a href="">Free Interfaces</a></li>
    </ul>
  </li>

  <li><a href="">Learn +</a>
    <ul>
      <li><a href="">Fireworks +</a>
        <ul>
          <li><a href="">aquaButton</a></li>
          <li><a href="">aquaButton2</a></li>
          <li><a href="">waterDrop</a></li>

          <li><a href="">lightFX</a></li>
          <li><a href="">lightFX2</a></li>
        </ul>
      </li>
      <li><a href="">CSS +</a>
        <ul>
          <li><a href="">footerStick</a></li>

          <li><a href="">spriteNav</a></li>
          <li><a href="">@import</a></li>
        </ul>
      </li>
    </ul>
  </li>
  <li><a href="">Info</a></li>

</ul>
"""
from jungle.sys.models import Menu
import re
from django.conf import settings


def mako_create_menu(context, **kw):
    current_path = context.get('path', '/')

    return create_menu(current_path, **kw)

def create_sdmenu():
    return HtmlMenu().render_menu()

class HtmlMenu(object):
    def __init__(self, id='navmenu', top_level=1):
        self.id = id
        self.entries = {}
        self.fathers = {}  #keep track of the children
        self.top_level = 1
        queryset = Menu.objects.all()
        for m in queryset.order_by('level', 'father_id', 'id'):
            if m.level == top_level:
                self.top_level = m.father_id
            self.add_to_father(m)

    def add_to_father(self,m):
        if m.father_id in self.fathers:
            self.fathers[m.father_id] += [m.id]
        else:
            self.fathers[m.father_id] = [m.id]
        self.entries[m.id] = m

    def render_entry(self, idx):
        m = self.entries[idx]
        ## render the children
        inner = []
        if idx in self.fathers:
            if idx > self.top_level:     # open new level
                inner += ["%s<ul>" % ('  '*(m.level+1))]

            for e in self.fathers[idx]:  # place all children 
                inner += self.render_entry(e)

            if idx > self.top_level:     # close the tag
                inner += ["%s</ul>" % ('  '*(m.level+1))]

        ## render itself
        if inner:
            inner.insert(0,'\n')
        ## for root entry we want to hide the entry and show the children
        if idx == 1:
            return ['\n'.join(inner)]
        else:
            if inner:
                a_capo = "\n" + '  '*m.level
            else:
                a_capo = ''
            return ['%s<li><a href="/%s">%s</a>%s%s</li>' % (
                '  '*m.level, m.path,  m.name,'\n'.join(inner),a_capo )]
                
    
    def render_menu(self):
        txt = ['<ul id="%s">' % self.id]
        txt += self.render_entry(self.top_level)
        txt += ['</ul>']
        return '\n'.join(txt)
