PAGE_NAVIGATOR_TABLE = u"""
    <table id="page_navigator">
        <tbody>
            <tr>
                <td style="text-align:right">
                    %s
                </td>
            </tr>
        </tbody>
    </table>
"""

START_WITH_PREVIOUS =  """
    <a href="#" onclick="page('1');">
        <img src="/static/img/start_on.gif" alt="|<" title="Start">
        &nbsp;
        Start
    </a>
"""
START_NO_PREVIOUS = """
    <img src="/static/img/start_off.gif" alt="|<" title="Start">
    &nbsp;
    Start
"""
PREVIOUS = """
    <a href="#" onclick="page('%s');">
        <img src="/static/img/previous_on.gif" alt="<" title="Previous">
        &nbsp;
        Previous
    </a>
"""
NO_PREVIOUS = """
    <img src="/static/img/previous_off.gif" alt="<" title="Previous">
    &nbsp;
    Start
"""
NEXT = """
    <a href="#" onclick="page('%s');">
        Next
        &nbsp;
        <img src="/static/img/next_on.gif" alt=">" title="Next">
    </a>
"""

NO_NEXT = """
    Next
    &nbsp;
    <img src="/static/img/next_off.gif" alt=">" title="Next">
"""

END_WITH_NEXT = """
    <a href="#" onclick="page('%s');">
        End
        &nbsp;
        <img src="/static/img/end_on.gif" alt=">|" title="End">
    </a>
"""
END_NO_NEXT = """
    End
    &nbsp;
    <img src="/static/img/end_off.gif" alt=">|" title="End">
"""

def simple_page_navigator(context, js_function=None):
    return SimplePageNavigator(context, js_function=js_function).render()

def page_navigator(context, js_function=None):
    return PageNavigator(context, js_function=js_function).render()

def fancy_page_navigator(context, js_function=None):
    return FancyPageNavigator(context, js_function=js_function).render()

class SimplePageNavigator(object):
    def __init__(self, context, pages_range=5, js_function=None):
        self.is_paginated = context.get('is_paginated')
        self.pages = context.get('pages')
        self.page = context.get('page')
        self.first_on_page = context.get('first_on_page')
        self.last_on_page = context.get('last_on_page')
        self.hits = context.get('hits')
        self.has_previous = context.get('has_previous')
        self.previous = context.get('previous') 
        self.has_next = context.get('has_next')
        self.next = context.get('next')
        self.pages_range = pages_range
        self.js_function = js_function

    def previous_links(self):
        txt = []
        # PREVIOUS HTML
        if self.has_previous:
            for i in range(1, self.previous+1):
                txt += [self.link(i)]
        return ''.join(txt)

    def next_links(self):
        txt = []
        # NEXT
        if self.has_next:
            for i in range(self.next, self.pages+1):
                txt += [self.link(i)]
        return ''.join(txt)

    def index(self):
        return ''

    def status(self):
        return self.page

    def link(self, i):
        return """<a href="#" onclick="page('%s');">&nbsp;&nbsp;%s&nbsp;&nbsp;</a>""" % (i, i)

    def get_content(self):
        return "%s %s %s %s" % (self.index(), self.previous_links(),
                                self.status(), self.next_links() )

    def render(self):
        if not self.is_paginated:
            return '&nbsp;'
        else:
            code = PAGE_NAVIGATOR_TABLE % (self.get_content())
            if self.js_function:
                code = code.replace('onclick="page(', 'onclick="%s(' % self.js_function)
            return code


class PageNavigator(SimplePageNavigator):
    def __init__(self, context, pages_range=5, js_function=None):
        SimplePageNavigator.__init__(self, context, pages_range, js_function=None)
        # PREVIOUS
        if self.has_previous:
            self.last_previous = self.pages_range
            if self.page <= self.pages_range:
                self.last_previous = self.page - 1
        # NEXT
        if self.has_next:
            self.first_next = self.pages - self.pages_range + 1
            if self.page >= self.pages - self.pages_range + 1:
                self.first_next = self.page + 1

        # CURRENT BEFORE
        if hasattr(self, 'last_previous') and self.last_previous:
            if self.last_previous < (self.page - self.pages_range / 2):
                self.first_current = self.page - self.pages_range / 2
            else:
                self.first_current = self.last_previous + 1
        else:
            self.first_current = self.page

        # CURRENT AFTER
        if hasattr(self, 'first_next') and self.first_next:
            if self.first_next > (self.page + self.pages_range / 2):
                self.last_current = self.page + self.pages_range / 2
            else:
                self.last_current = self.first_next - 1
        else:
            self.last_current = self.page

    def previous_links(self):
        txt = []
        # PREVIOUS HTML
        if self.has_previous:
            for i in range(1, self.last_previous+1):
                txt += [self.link(i)]
            txt += "..."
        return ''.join(txt)

    def next_links(self):
        txt = []
        # NEXT HTML
        if self.has_next and hasattr(self, 'first_next'):
            txt += "..."
            for i in range(self.first_next, self.pages+1):
                txt += [self.link(i)]
            return ''.join(txt)

    def status(self):
        txt = []
        # CURRENT HTML
        for i in range(self.first_current, self.page):
            txt += [self.link(i)]
        txt += [str(self.page)]
        for i in range(self.page+1, self.last_current+1):
            txt += [self.link(i)]
        return ''.join(txt)

class FancyPageNavigator(SimplePageNavigator):

    def index(self):
        return "Page %s of %s Pages" % (self.page, self.pages,)

    def previous_links(self):
        if self.has_previous:
            start = START_WITH_PREVIOUS
            previous = PREVIOUS % self.previous
        else:
            start = START_NO_PREVIOUS
            previous = NO_PREVIOUS

        return "%s %s" % (start, previous)

    def next_links(self):
        if self.has_next:
            next = NEXT % self.next
            end = END_WITH_NEXT % self.pages
        else:
            next = NO_NEXT
            end = END_NO_NEXT
        return "%s %s" % (next, end)

    def status(self):
        return "(%s - %s of %s)"%(self.first_on_page,self.last_on_page, self.hits,)

if __name__ == '__main__':
    from mako.runtime import Context
    C1 = Context(None, **dict(
        is_paginated = True,
        pages = 20,
        page = 12,
        first_on_page = 111,
        last_on_page = 120,
        hits = 197,
        has_previous = True,
        previous = 11,
        has_next = True,
        next = 13))
    C2 = Context(None, **dict(
        is_paginated = True,
        pages = 10,
        page = 1,
        first_on_page = 1,
        last_on_page = 10,
        hits = 97,
        has_previous = False,
        has_next = True,
        next = 2))
    C3 = Context(None, **dict(
        is_paginated = True,
        pages = 10,
        page = 8,
        first_on_page = 71,
        last_on_page = 80,
        hits = 97,
        has_previous = True,
        has_next = True,
        previous = 7,
        next = 9))
    print '#########################'
    print '# SIMPLE PAGE NAVIGATOR #'
    print '#########################'
    print '# 1 2 3 4 5 6 7 8 9 10 11 >12< 13 14 15 16 17 18 19 20'
    print simple_page_navigator(C1)
    print '# >1< 2 3 4 5 6 7 8 9 10'
    print simple_page_navigator(C2)
    print '# 1 2 3 4 5 6 7 >8< 9 10'
    print simple_page_navigator(C3)
    print '#########################'
    print '# PAGE NAVIGATOR        #'
    print '#########################'
    print '# 1 2 3 4 5 ... 10 11 >12< 13 14 ... 16 17 18 19 20'
    print page_navigator(C1)
    print '# >1< 2 3 ... 6 7 8 9 10'
    print page_navigator(C2)
    print '# 1 2 3 4 5 ... 6 7 >8< ... 9 10 '
    print page_navigator(C3)
    print '#########################'
    print '# FANCY PAGE NAVIGATOR  #'
    print '#########################'
    print '#Page 12 of 20 Pages  |< Start  < Previous  (111 - 120 of 197)  Next >  End >|'
    print fancy_page_navigator(C1)
    print '#Page 1 of 10 Pages  |< Start  < Previous  (1 - 10 of 97)  Next >  End >|'
    print fancy_page_navigator(C2)
    print '#Page 8 of 10 Pages  |< Start  < Previous  (71 - 80 of 97)  Next >  End >|'
    print fancy_page_navigator(C3)

