from docutils import nodes
from docutils.parsers.rst import directives
try:
    from pygments import highlight
    from pygments.lexers import get_lexer_by_name
    from pygments.formatters import HtmlFormatter

    pygments_formatter = HtmlFormatter()
    normal_fmter = HtmlFormatter()
    lineno_fmter = HtmlFormatter(linenos=True)

    def pygments_directive(name, arguments, options, content, lineno,
                           content_offset, block_text, state, state_machine):
        try:
            lexer = get_lexer_by_name(arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = get_lexer_by_name('text')
        formatter = ('linenos' in options) and lineno_fmter or normal_fmter
        parsed = highlight(u'\n'.join(content), lexer, formatter)
        return [nodes.raw('', parsed, format='html')]

except:
    def pygments_directive(name, arguments, options, content, lineno,
                           content_offset, block_text, state, state_machine):
        text = "<pre>%s</pre>" % u'\n'.join(content)
        return [nodes.raw('', text, format='html')]    

pygments_directive.arguments = (1, 0, 1)
pygments_directive.content = 1
pygments_directive.options = {'linenos': directives.flag}
directives.register_directive('sourcecode', pygments_directive)


def ReST(value):
    from django.conf import settings
    try:
        from docutils.core import publish_parts
    except ImportError:
        if settings.DEBUG:
            raise template.TemplateSyntaxError, "Error in ReST filter: " + \
                  "The Python docutils library isn't installed."
        return value
    else:
        parts = publish_parts(source=value, writer_name="html4css1")

        return parts["html_body"]

