"""
UrlTest is a class that fills in test_ methods from declarative variables to easy
the way tests are prepared. The following attributes are handled:

  * app_label: this will set app_label and look for menu.txt in the app directoory
               and add a test for each url that is present and not commented

  * post_get:  this will be a tuple of tuples:
              (  ('/mailserver/domain/add', dict(attr1=va1, attr2=val2...), )
              for each such path, the page will be retrieved with a POST, the
              redirect will be followed to GET the redirected page (200 expected)

  * get:      this is a tuple of urls to retrieve, a 200 status_code is expected

  NOTE: all values inserted via POST are not deleted from TEST_DATABASE so that are
        available by all other subsequent tests



"""

import unittest
from django.test.client import Client
from django.conf import settings
import subprocess as sub
from jungle.sys.models import Menu
from jungle.scripts.import_menus import run_for_apps, get_or_create_father_menu_entry
from django.utils.functional import curry
from django.db.models import get_app, get_apps
import os, re
from urlparse import urlparse
from  pdb import set_trace
from django.test import TestCase
from django.core import management# import load_data

failed = []
def get_page(self, path, my_dict):
    response = self.client.get(path, my_dict)
    if response.status_code != 200:
        print "\n%s\n URL '%s' has a problem [dict = %s]." % (
            '='*80, path, my_dict)
        failed.append(path)
    self.failUnlessEqual(response.status_code, 200)

def login_required(self, path, my_dict):
    if not path.endswith('/'):
        path += '/'
    anonimous_client = Client()
    response = anonimous_client.get(path, my_dict)
    if response.status_code == 404:
        set_trace()
    self.failUnlessEqual(response.status_code, 302)

def post_get(self, path, my_dict):
    """post a page and follow the redirect to see if we can view the detail
    """
    response = self.client.post(path, my_dict)
    self.failUnlessEqual(response.status_code, 302)
    _, _, redirect, _, data, _= urlparse(response['Location'])
    response_get = self.client.get(redirect, my_dict)
    self.failUnlessEqual(response_get.status_code, 200)

def post_context(self, path, my_dict, obj):
    response = self.client.post(path, my_dict)
    ## occorre testare che obj sia in response.context
    
def get_menu_path(app_label):
    module = get_app(app_label)

    if module.__file__:
        return os.path.join(os.path.dirname(module.__file__), 'menu.txt')

def read_menu_file(app_label):
    f = open( get_menu_path(app_label) )
    for line in f:
        line = re.sub('[ \t\n]', '', line)
        line = re.sub('#.*', '', line)
        if line:
            try:
                url = line.split(',')[1]
            except:
                continue
            
            current_path = url
            parts = current_path.split('?')
            current_path = parts[0]
            
            parms = {}            
            try:
                pieces = parts[1].split('&')
                for p in pieces:
                    key, value = p.split('=')
                    parms[key] = value
            except:
                pass

            test_name = 'test_%s' % url
            yield (test_name, clean_path(current_path), parms)
    

def clean_path(path):
    if not path.startswith('/'):
        path = "/%s" % (path)
        
    if not path.endswith('/'):
        path = "%s/" % (path)

    return path
    
def fill_menu_entries_for_app(attrs):
    """each class must have an attribute 
    """
    if 'app_label' in attrs:
        app_label = attrs['app_label']
        app_path = ".".join( get_app(app_label).__name__.split('.')[:-1])

        return app_label

    return ''

def prepare_urls(tests, prefix):
    n = 0
    ## if you miss a ',' in the declarations of tests...
    if not isinstance(tests, tuple):
        tests = [tests]
    for test in tests:
        n += 1
        try:   # post ha params as well
            path, params = test
        except: # get
            path = test
            params = {}
        path = clean_path(path)
        test_name = 'test_%s_%2.2d_%s' % (prefix, n, path )
        yield (test_name, path, params)

    
class UrlMetaclass(type):
    def __new__(cls, name, bases, attrs):
        if 'app_label' in attrs:
            for test_name, path, params in read_menu_file(attrs['app_label']):
                attrs[test_name] =  curry(get_page, path=path, my_dict=params)

        if 'post_get' in attrs:
            for test_name, path, params in prepare_urls(attrs['post_get'], 'add'):
                attrs[test_name] =  curry(post_get, path=path, my_dict=params)

        if 'get' in attrs:
            for test_name, path, params in prepare_urls(attrs['get'], 'get'):
                attrs[test_name] =  curry(get_page, path=path, my_dict=params)
            
        if 'login_required' in attrs:
            for test_name, path, params in prepare_urls(attrs['login_required'], 'lreq'):
                attrs[test_name] =  curry(login_required, path=path, my_dict=params)
            
        if 'public' in attrs:
            for test_name, path, params in prepare_urls(attrs['public'], 'pub'):
                attrs[test_name] =  curry(login_required, path=path, my_dict=params)
            
            
        return type.__new__(cls, name, bases, attrs)


class WithFixtures(TestCase):

    __metaclass__ = UrlMetaclass

    def setUp(self):
        USERNAME = 'tester'
        PASSWORD = 'passwd'

        from django.contrib.auth.models import User
        u, existent = User.objects.get_or_create(username=USERNAME,  )
        u.set_password(PASSWORD)
        u.is_staff = True
        u.email = 'info@localhost.local'
        u.save()
        self.user = u
        self.client=Client()
        ## attenzione: questa pagine deve esistere e non deve dare errori!
        #ret = self.client.login('/userprofile/profile/add/', USERNAME, PASSWORD)
        ret = self.client.login(username=USERNAME, password=PASSWORD)

done = False
class NoFixtures(WithFixtures):

    def run(self, result=None):
        """Wrapper around default run method so that user-defined Test Cases 
        automatically call install_fixtures without having to include a call to 
        super().
        
        """
        self.client = Client()

        global done 
        if hasattr(self, 'data') and self.data and not done:
            management.call_command('loaddata', self.data)
            done = True
        
        #self.install_fixtures()
        unittest.TestCase.run(self, result)


